package com.nts.isip.tcp;

import com.nts.isip.tcp.isipgui.IcamCommandPanel;
import com.nts.isip.tcp.isipgui.IcamPropertiesPanel;
import com.nts.isip.tcp.isipgui.ImageDisplayPanel;
import com.nts.isip.tcp.isipgui.ProcessorTypeSetter;

import java.awt.BorderLayout;
import java.awt.Dimension;

import java.io.File;
import java.io.IOException;

import java.util.Properties;
import java.util.Vector;

import javax.swing.Action;
import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JTabbedPane;
import javax.swing.JToolBar;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

/**
 * The main frame of the ISIP Client application.
 * It receives a map of all Action objects to associate
 * with command buttons in the interface
 */
public class ClientFrame extends javax.swing.JFrame {
    private JLabel _statusBar;
    
    private StreamThread _strThread;
    private Vector<DirectoryRepresentation> _icamDirs;
    private java.util.Map<Integer, Action> _cmdMap;
    private IcamPropertiesPanel _icamPropertiesPanel;
    private IcamCommandPanel _icamCommandPanel;
    private JTabbedPane _mainPanel;
    
    public ClientFrame() {
        Properties prop = new Properties();

        try {
            java.io.InputStream in = getClass().getClassLoader()
                                         .getResource("version.properties")
                                         .openStream();
            prop.load(in);
        } catch (IOException ioe) {
        }

        if (prop.getProperty("version") != null) {
            this.setTitle("isip TCP client version " +
                prop.getProperty("version"));
        } else {
            this.setTitle("isip TCP client");
        }
    }

    /**
     * Creates and lays out all components and panels in the frame.
     * It is passed a map of Action objects to associate with command buttons,
     * and server connection to back the models behind panels
     */
    public void init(java.util.Map<Integer, Action> cmdMap,
        StreamThread strThread) {
        _cmdMap = cmdMap;
        initComponents(cmdMap, strThread);
        this.pack();
        this.setSize(new Dimension(800, 750));
        this.setVisible(true);
        this.setLocationRelativeTo(null);
    }

    public javax.swing.JFrame getTopFrame() {
        return this;
    }

    public ClientView getView() {
        return _icamPropertiesPanel;
    }

    /**
     * Application entry point. Create a controller object that creates
     * and initializes the view.
     */
    public static void main(String[] args) {
        ClientCtrl ctrl = new ClientCtrl();
    }

    public void setStatus(final String stat) {
        javax.swing.SwingUtilities.invokeLater(new Runnable() {
                public void run() {
                    _statusBar.setText(stat);
                }
            });
    }

    private void initComponents(java.util.Map<Integer, Action> cmdMap,
        StreamThread strThread) {
        _strThread = strThread;
        this.getContentPane().setLayout(new BorderLayout());
        _statusBar = new JLabel(" ");
        _statusBar.setBorder(BorderFactory.createEtchedBorder());
        this.getContentPane().add(_statusBar, BorderLayout.SOUTH);

        JToolBar tb = new JToolBar();
        tb.add(cmdMap.get(CmdId.CONNECT));
        tb.add(cmdMap.get(CmdId.DISCONNECT));
        //tb.add(cmdMap.get(CmdId.REPROCESS));
        tb.add(cmdMap.get(CmdId.SYSTEM_STORE));
        tb.add(cmdMap.get(CmdId.TEMPLATE_STORE));

        ProcessorTypeSetter procType;

        Vector<String> processorNames = new Vector<String>();
        processorNames.add("Parsing");

        File rematchDir = new File("");

        try {
            rematchDir = new File("Native");

            File[] dataFiles = rematchDir.listFiles();

            for (int i = 0; i < dataFiles.length; i++) {
                if (dataFiles[i].getName().endsWith(".dll") ||
                        dataFiles[i].getName().endsWith(".so")) {
                    processorNames.add("Rematching(" + dataFiles[i].getName() +
                        ")");
                }
            }
        } catch (NullPointerException e) {
        }

        File mathDir = new File("");

        try {
            mathDir = new File("Mathematica");

            File[] dataFiles = mathDir.listFiles();

            for (int i = 0; i < dataFiles.length; i++) {
                if (dataFiles[i].getName().endsWith(".m")) {
                    processorNames.add("Mathematica(" + dataFiles[i].getName() +
                        ")");
                }
            }
        } catch (NullPointerException e) {
        }

        File macrosDir = new File("");

        try {
            macrosDir = new File("IJMacros");

            File[] dataFiles = macrosDir.listFiles();

            for (int i = 0; i < dataFiles.length; i++) {
                if (dataFiles[i].getName().endsWith(".txt")) {
                    processorNames.add("ImageJ(" + dataFiles[i].getName() +
                        ")");
                }
            }
        } catch (NullPointerException e) {
        }

        processorNames.add("Custom...");

        String[] processors = processorNames.toArray(new String[0]);
        JButton editScript = new JButton("Edit script");
        editScript.setEnabled(false);
        procType = new ProcessorTypeSetter(this, strThread, processors,
                rematchDir.getPath(), macrosDir.getPath(), mathDir.getPath(),
                editScript);

        tb.add(procType);
        tb.add(editScript);
        this.getContentPane().add(tb, BorderLayout.NORTH);
        
        _mainPanel = new JTabbedPane();
        this.getContentPane().add(_mainPanel, BorderLayout.CENTER);
    }
    
    public void setICamPanels() {
        _mainPanel.removeAll();
        _icamPropertiesPanel = new IcamPropertiesPanel(_strThread, _cmdMap, this);
        _icamCommandPanel = new IcamCommandPanel(_strThread, this);

        if (_strThread != null) {
            try {
                _icamDirs = ((_strThread.getStreamSource())).getDirectoryData(
                        "/");
                _icamPropertiesPanel.setIcamFrameSource(_strThread.getStreamSource());
                _icamPropertiesPanel.updateIcamPanel(_icamDirs, null);
                _strThread.setView(_icamPropertiesPanel);
            } catch (Exception ex) {
                //ex.printStackTrace();
                JOptionPane.showMessageDialog(getTopFrame(),
                    "An error has occured while communicating with server.\n" +
                    ex.getMessage(), "Communications error",
                    JOptionPane.ERROR_MESSAGE);
            }
        }

        String tabPropertyTitel = "iCAM properties";

        if (_icamDirs != null) {
            for (int i = 0; i < _icamDirs.size(); i++) {
                if (_icamDirs.get(i).dirFullPath.equals("/")) {
                    tabPropertyTitel = _icamDirs.get(i).label;
                }
            }
        }

        _mainPanel.add(tabPropertyTitel, _icamPropertiesPanel);
        _mainPanel.add("iCam Commands", _icamCommandPanel);

        _mainPanel.addChangeListener(new ChangeListener() {
                public void stateChanged(ChangeEvent e) {
                    JTabbedPane tbp = (JTabbedPane) e.getSource();

                    if (tbp.getSelectedIndex() == 0) {
                        if (_strThread != null) {
                            try {
                                _icamDirs = (_strThread.getStreamSource()).getDirectoryData(
                                        "/");
                                _icamPropertiesPanel.setIcamFrameSource(_strThread.getStreamSource());
                                _icamPropertiesPanel.updateIcamPanel(_icamDirs,
                                    _mainPanel);
                                _strThread.setView(_icamPropertiesPanel);
                            } catch (Exception ex) {
                                //ex.printStackTrace();
                                JOptionPane.showMessageDialog(getTopFrame(),
                                    "An error has occured while communicating with server.\n" +
                                    ex.getMessage(), "Communications error",
                                    JOptionPane.ERROR_MESSAGE);
                            }
                        }
                    } else if (tbp.getSelectedIndex() == 1) {
                        _icamCommandPanel.init();
                    }
                }
            });
    }
}

class DisplayImagePanelTabs {
    protected ImageDisplayPanel pan;
    protected int tabIdx;
    protected int histidx;
}
