package com.nts.isip.tcp;

import at.igm.www.iCAMWSDL.client.ArrayOfEntries;
import at.igm.www.iCAMWSDL.client.ArrayOfFloats;
import at.igm.www.iCAMWSDL.client.ArrayOfInts;
import at.igm.www.iCAMWSDL.client.ArrayOfStrings;
import at.igm.www.iCAMWSDL.client.Directory;
import at.igm.www.iCAMWSDL.client.DirectoryEntry;
import at.igm.www.iCAMWSDL.client.ICAMgsoap;
import at.igm.www.iCAMWSDL.client.Property;
import at.igm.www.iCAMWSDL.client.holders.ArrayOfEntriesHolder;
import at.igm.www.iCAMWSDL.client.holders.ArrayOfFloatsHolder;
import at.igm.www.iCAMWSDL.client.holders.ArrayOfIntsHolder;
import at.igm.www.iCAMWSDL.client.holders.ArrayOfStringsHolder;
import at.igm.www.iCAMWSDL.client.holders.DirectoryHolder;
import at.igm.www.iCAMWSDL.client.holders.PropertyHolder;

import com.nts.isip.tcp.exceptions.CheckSumException;

import com.ntsbg.util.Stream;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import org.xml.sax.SAXException;

import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;

import java.rmi.RemoteException;

import java.util.HashMap;
import java.util.Properties;
import java.util.Vector;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.rpc.holders.ByteArrayHolder;
import javax.xml.rpc.holders.FloatHolder;
import javax.xml.rpc.holders.IntHolder;
import javax.xml.rpc.holders.StringHolder;
import javax.xml.xpath.XPath;
import javax.xml.xpath.XPathConstants;
import javax.xml.xpath.XPathExpression;
import javax.xml.xpath.XPathExpressionException;
import javax.xml.xpath.XPathFactory;


public class DirectoryServer implements ICAMgsoap {
    private static final String S_OK = "S_OK";
    private static final String S_FAIL = "S_FAIL";
    private static final String S_NOTFOUND = "S_NOTFOUND";
    protected static final String TYPE_INT = "int";
    protected static final String TYPE_FLOAT = "float";
    protected static final String TYPE_STRING = "string";
    protected static final String TYPE_INT_ARRAY = "intArray";
    protected static final String TYPE_FLOAT_ARRAY = "floatArray";
    protected static final String TYPE_STRING_ARRAY = "stringArray";
    protected static final String TYPE_ENUMERATION = "enumeration";
    protected static final String TYPE_DYNAMIC_LIST = "dynamiclist";
    protected static final String TYPE_IMAGE = "image";
    private static HashMap<String, String> _flatMap = new HashMap<String, String>();
    private static HashMap<String, Property> _relationMap = new HashMap<String, Property>();

    //private HashMap<String, String> _channelProps = new HashMap<String, String>();
    private HashMap<String, Object> _channelData = new HashMap<String, Object>();
    private ServerProperty _currentProperty;
    private String _root;
    private File _rootDir;
    private Properties _cameraProps;
    private String _currentChannel;
    private int _frameCnt;

    public DirectoryServer(String root) {
        _root = root;
        _flatMap.clear();
        _frameCnt = 0;
        _currentChannel = "";
        _cameraProps = new Properties();

        try {
            java.io.FileInputStream in = new java.io.FileInputStream(
                    "camerasim.properties");
            _cameraProps.load(in);
        } catch (IOException ioe) {
        }

        _rootDir = new File(getRootDirName());

        if (!(_rootDir.exists() && _rootDir.isDirectory())) {
            _rootDir = null;
        }
    }

    private String getRootDirName() {
        String rootDirName;

        if (_cameraProps.containsKey("PropertyDescriptionsDir")) {
            rootDirName = _root + "/" +
                _cameraProps.getProperty("PropertyDescriptionsDir");
        } else {
            rootDirName = _root + "/PropertyDescriptions";
        }

        return rootDirName;
    }

    private String getTemplateDirName() {
        String templateDirName;

        if (_cameraProps.containsKey("JobsDir")) {
            templateDirName = _root + "/" +
                _cameraProps.getProperty("JobsDir");
        } else {
            templateDirName = _root + "/JobsDir";
        }

        return templateDirName;
    }

    private String getPictogramDirName() {
        String pictogramDir;

        if (_cameraProps.containsKey("PictogramsDir")) {
            pictogramDir = _cameraProps.getProperty("PictogramsDir");
        } else {
            pictogramDir = "pictograms";
        }

        return pictogramDir;
    }

    private String getPacketDirName() {
        String packetDirName;

        if (_cameraProps.containsKey("PacketsDir")) {
            packetDirName = _root + "/" +
                _cameraProps.getProperty("PacketsDir") + "/channel";
        } else {
            packetDirName = _root + "/PacketsDir/channel";
        }

        return packetDirName;
    }

    public String clearPropertyTransaction(String arg0)
        throws RemoteException {
        return S_OK;
    }

    public String commitPropertyTransaction(String arg0)
        throws RemoteException {
        return S_OK;
    }

    public String deleteFile(String fileName) throws RemoteException {
        return null;
    }

    public String deleteUserTemplate(String key) throws RemoteException {
        String res = S_FAIL;

        File jobsdir = new File(getTemplateDirName());

        if (!jobsdir.exists()) {
            throw new RemoteException("Jobs directory does not exist");
        }

        File jobFile = new File(getTemplateDirName() + "/" + key +
                ".properties");

        if (!jobFile.exists()) {
            throw new RemoteException("The job does not exist");
        }

        if (!jobFile.delete()) {
            throw new RemoteException("Failed deleting job file for " + key +
                " from the filesystem");
        } else {
            res = S_OK;
        }

        return res;
    }

    public String execute(String commandLine, int executionTimeout)
        throws RemoteException {
        return S_OK;
    }

    public void getDirectoryDefinition(String directoryName,
        DirectoryHolder directoryDefinition, StringHolder error)
        throws RemoteException {
        if (_rootDir == null) {
            error.value = "No result from function call";
        }

        Directory dir = null;
        String properPath = _rootDir + directoryName.trim();

        if (!properPath.endsWith("/")) {
            properPath += "/";
        }

        String fullname = properPath + "index.xml";
        File idxfile = new File(fullname);
        DocumentBuilderFactory domFactory = DocumentBuilderFactory.newInstance();
        domFactory.setNamespaceAware(true); // never forget this!

        try {
            if (properPath.contains("channel")) {
                dir = new Directory();
                dir.setDierctoryName(directoryName);

                String[] fullPathParts = directoryName.split("/");
                dir.setDirectoryLabel(fullPathParts[fullPathParts.length - 1]);
                dir.setIsTransaction(false);
            } else if (idxfile.exists()) {
                DocumentBuilder builder = domFactory.newDocumentBuilder();
                Document doc = builder.parse(idxfile);

                XPathFactory factory = XPathFactory.newInstance();
                XPath xPath = factory.newXPath();

                XPathExpression exIndexRoot = xPath.compile(
                        "/directorydefinition");
                XPathExpression exIndexDirName = xPath.compile(
                        "/directorydefinition/DirectoryName/@value");
                XPathExpression exIndexDirLabel = xPath.compile(
                        "/directorydefinition/DirectoryLabel/@value");
                XPathExpression exIndexIsTransaction = xPath.compile(
                        "/directorydefinition/isTransaction/@value");
                Element rootEl = (Element) exIndexRoot.evaluate(doc,
                        XPathConstants.NODE);

                if (rootEl == null) {
                    error.value = "No result from function call";
                }

                dir = new Directory();
                dir.setDierctoryName(exIndexDirName.evaluate(doc));
                dir.setDirectoryLabel(exIndexDirLabel.evaluate(doc));

                String sIsTransaction = exIndexIsTransaction.evaluate(doc);
                dir.setIsTransaction(Boolean.parseBoolean(sIsTransaction));
            }

            directoryDefinition.value = dir;
        } catch (Exception e) {
            error.value = "No result from function call";
        }
    }

    public void getEvents(ArrayOfStringsHolder events, StringHolder error)
        throws RemoteException {
        error.value = S_FAIL;
    }

    public void getFile(String fileName, ByteArrayHolder fileContent,
        StringHolder error) throws RemoteException {
        error.value = S_OK;
    }

    public void getListOfPropertyValues(ArrayOfStrings listOfProperties,
        ArrayOfStringsHolder listOfValues, StringHolder error)
        throws RemoteException {
    }

    public void getPropertyBinary(String propertyName, ByteArrayHolder data,
        StringHolder error) throws RemoteException {
    }

    public void getPropertyDefinition(String propertyName,
        PropertyHolder propertyDefinition, StringHolder error)
        throws RemoteException {
        ServerProperty prop = new ServerProperty();
        _currentProperty = prop;

        if ((propertyName == null) ||
                (propertyName.trim().equalsIgnoreCase(""))) {
            error.value = S_FAIL;

            return;
        }

        String fullname = _rootDir + propertyName + ".xml";
        File xmlDocument = new File(fullname);

        if (!xmlDocument.exists()) {
            error.value = S_FAIL;

            return;
        }

        DocumentBuilderFactory domFactory = DocumentBuilderFactory.newInstance();
        domFactory.setNamespaceAware(true); // never forget this!

        DocumentBuilder builder;

        try {
            builder = domFactory.newDocumentBuilder();

            Document doc = builder.parse(xmlDocument);

            XPathFactory factory = XPathFactory.newInstance();
            XPath xPath = factory.newXPath();
            XPathExpression exRoot = xPath.compile("/propertydefinition");
            Element rootEl = (Element) exRoot.evaluate(doc, XPathConstants.NODE);

            if (rootEl == null) {
                error.value = S_FAIL;

                return;
            }

            XPathExpression exName = xPath.compile(
                    "/propertydefinition/Name/@value");
            XPathExpression exInternalName = xPath.compile(
                    "/propertydefinition/PropertyInternalName/@value");
            XPathExpression exGroup = xPath.compile(
                    "/propertydefinition/PropertyGroup/@value");
            XPathExpression exType = xPath.compile(
                    "/propertydefinition/PropertyType/@value");
            XPathExpression exPictogram = xPath.compile(
                    "/propertydefinition/pictogram/@value");
            XPathExpression exLabel = xPath.compile(
                    "/propertydefinition/Label/@value");
            XPathExpression exOnTyping = xPath.compile(
                    "/propertydefinition/OnTypingValidator/@value");
            XPathExpression exOnSet = xPath.compile(
                    "/propertydefinition/OnSetValidator/@value");
            XPathExpression exInfo = xPath.compile(
                    "/propertydefinition/propertyInfo/@value");
            XPathExpression exReadOnly = xPath.compile(
                    "/propertydefinition/isReadOnly/@value");
            XPathExpression exAdmin = xPath.compile(
                    "/propertydefinition/isAdmin/@value");
            XPathExpression exAdjustable = xPath.compile(
                    "/propertydefinition/isAdjustableByTheSystme/@value");
            XPathExpression exDependency = xPath.compile(
                    "/propertydefinition/Dependency/@value");
            XPathExpression exAllowedValues = xPath.compile(
                    "/propertydefinition/PropertyAllowedValues/@value");
            XPathExpression exMin = xPath.compile(
                    "/propertydefinition/MinValue/@value");
            XPathExpression exMax = xPath.compile(
                    "/propertydefinition/MaxValue/@value");
            XPathExpression exArrayColumns = xPath.compile(
                    "/propertydefinition/arrayColumns/@value");
            XPathExpression exMaxArrayRows = xPath.compile(
                    "/propertydefinition/maxArrayRows/@value");
            XPathExpression exElementOffset = xPath.compile(
                    "/propertydefinition/elementOffset/@value");
            XPathExpression exUnitsInElement = xPath.compile(
                    "/propertydefinition/unitsInElement/@value");

            prop = new ServerProperty();
            prop.setName(exName.evaluate(doc));
            prop.setPropertyType(exType.evaluate(doc));
            prop.setLabel(exLabel.evaluate(doc));
            prop.setOnTypingValidator(exOnTyping.evaluate(doc));
            prop.setOnSetValidator(exOnSet.evaluate(doc));
            prop.setInfo(exInfo.evaluate(doc));
            prop.setPropertyGroup(exGroup.evaluate(doc));

            String sReadOnly = exReadOnly.evaluate(doc);
            prop.setIsReadOnly(Boolean.parseBoolean(sReadOnly));

            String sAdmin = exAdmin.evaluate(doc);
            prop.setIsAdmin(Boolean.parseBoolean(sAdmin));

            String sIsAdjustableBySystem = exAdjustable.evaluate(doc);
            prop.setIsAdjustableByTheSystme(Boolean.parseBoolean(
                    sIsAdjustableBySystem));
            prop.setDependency(exDependency.evaluate(doc));
            prop.setPropertyAllowedValues(exAllowedValues.evaluate(doc));

            String sMinVal = exMin.evaluate(doc);

            try {
                prop.setMinValue(Float.parseFloat(sMinVal));
            } catch (NumberFormatException nfe) {
                prop.setMinValue(0);
            }

            String sMaxVal = exMax.evaluate(doc);

            try {
                prop.setMaxValue(Float.parseFloat(sMaxVal));
            } catch (NumberFormatException nfe) {
                prop.setMaxValue(0);
            }

            String sMaxArrayRows = exMaxArrayRows.evaluate(doc);

            try {
                prop.setMaxArrayRows(Integer.valueOf(sMaxArrayRows));
            } catch (NumberFormatException nfe) {
                prop.setMaxArrayRows(0);
            }

            String sArrayColumns = exArrayColumns.evaluate(doc);

            try {
                prop.setArrayColumns(Integer.valueOf(sArrayColumns));
            } catch (NumberFormatException nfe) {
                prop.setArrayColumns(0);
            }

            String picName = exPictogram.evaluate(doc);

            if ((picName == null) || (picName.trim().equalsIgnoreCase(""))) {
                picName = "default.bmp";
            }

            String fName = _root + "/" + getPictogramDirName() + "/" + picName;
            byte[] picData = new byte[0];
            File picFile = new File(fName);

            if (picFile.exists()) {
                picData = readFile(fName);
            } else {
                fName = _root + "/" + getPictogramDirName() + "/default.bmp";
                picFile = new File(fName);

                if (picFile.exists()) {
                    picData = readFile(fName);
                }
            }

            prop.setPictogram(picData);

            String sElemOffset = exElementOffset.evaluate(doc);

            try {
                prop.setElementOffset(Integer.parseInt(sElemOffset));
            } catch (NumberFormatException nfe) {
                prop.setElementOffset(0);
            }

            String unitsInElem = exUnitsInElement.evaluate(doc);

            try {
                prop.setUnitsInElement(Integer.parseInt(unitsInElem));
            } catch (NumberFormatException nfe) {
                prop.setUnitsInElement(0);
            }

            _currentProperty = prop;
            _flatMap.put(prop.getName(), exInternalName.evaluate(doc));
            _relationMap.put(prop.getName(), prop);
            propertyDefinition.value = prop;
            error.value = S_OK;
        } catch (ParserConfigurationException e) {
            error.value = S_FAIL;
        } catch (SAXException e) {
            error.value = S_FAIL;
        } catch (IOException e) {
            error.value = S_FAIL;
        } catch (XPathExpressionException e) {
            error.value = S_FAIL;
        }
    }

    private byte[] readFile(final String fullname) throws IOException {
        File file = new File(fullname);
        InputStream is = new FileInputStream(file);
        long length = file.length();

        if (length > Integer.MAX_VALUE) {
            // File is too large
        }

        byte[] bytes = new byte[(int) length];
        int offset = 0;
        int numRead = 0;

        while ((offset < bytes.length) &&
                ((numRead = is.read(bytes, offset, bytes.length - offset)) >= 0)) {
            offset += numRead;
        }

        if (offset < bytes.length) {
            bytes = null;
        }

        is.close();

        return bytes;
    }

    public void getPropertyDefinitionXML(String propertyName,
        ByteArrayHolder propertyDefinitionXML, StringHolder error)
        throws RemoteException {
    }

    public void getPropertyDirectory(String directory,
        ArrayOfEntriesHolder directoryContent, StringHolder error)
        throws RemoteException {
        if (_rootDir != null) {
            Vector<DirectoryEntry> dirEntries = new Vector<DirectoryEntry>();

            /*System.out.println(directory);
            if(!directory.equals("/channel/") && directory.startsWith("/channel")) {
                    if(!_channelProps.isEmpty()) {
                            Iterator it = _channelProps.keySet().iterator();
                            while(it.hasNext()) {
                                    String propName = (String)it.next();
                                    DirectoryEntry dirEntry = new DirectoryEntry();
                                    dirEntry.setEntryName(propName);
                                    dirEntry.setEntryType("property");
                                    dirEntries.add(dirEntry);
                                    System.out.println(propName);
                            }
                            error.value = S_OK;
                    } else {
                            directoryContent.value = new ArrayOfEntries();
                            directoryContent.value.setItem(null);
                            error.value = S_OK;
                    }
            } else {*/
            if (!directory.endsWith("/")) {
                directory = directory + "/";
            }

            String sDir = getRootDirName() + directory;
            File tgtDir = new File(sDir);

            if (!tgtDir.exists()) {
                error.value = S_NOTFOUND;
            }

            File[] dirs = tgtDir.listFiles();

            if ((dirs != null) && (dirs.length > 0)) {
                for (int i = 0; i < dirs.length; i++) {
                    if (dirs[i].isDirectory()) {
                        String newDirName = directory + dirs[i].getName() +
                            "/";
                        DirectoryEntry dirEntry = new DirectoryEntry();
                        dirEntry.setEntryName(newDirName);
                        dirEntry.setEntryType("directory");
                        dirEntries.add(dirEntry);
                    } else if (dirs[i].getName().endsWith(".xml")) {
                        if ((dirs[i].getName()).equalsIgnoreCase("index.xml")) {
                            continue; // skip the index file containing the directory definition
                        }

                        String propName = directory + dirs[i].getName();
                        StringBuffer tempBuf = new StringBuffer(propName);
                        tempBuf.delete(propName.length() - 4, propName.length());
                        propName = tempBuf.toString();

                        DirectoryEntry dirEntry = new DirectoryEntry();
                        dirEntry.setEntryName(propName);
                        dirEntry.setEntryType("property");
                        dirEntries.add(dirEntry);
                    }
                }
            }

            directoryContent.value = new ArrayOfEntries((DirectoryEntry[]) dirEntries.toArray(
                        new DirectoryEntry[0]));
            error.value = S_OK;

            //}
        } else {
            error.value = S_FAIL;
        }
    }

    private Properties getTemplateProperties() {
        Properties templateProps = new Properties();

        try {
            String templateFileName;

            if (_cameraProps.containsKey("PropertyDescriptionsDir")) {
                templateFileName = _root + "/" +
                    _cameraProps.getProperty("CameraTemplateParamsFile");
            } else {
                templateFileName = _root + "/template.properties";
            }

            java.io.FileInputStream in = new java.io.FileInputStream(templateFileName);
            templateProps.load(in);
        } catch (IOException ioe) {
        }

        return templateProps;
    }

    private Properties getSystemProperties() {
        Properties systemProps = new Properties();

        try {
            String systemFileName;

            if (_cameraProps.containsKey("PropertyDescriptionsDir")) {
                systemFileName = _root + "/" +
                    _cameraProps.getProperty("CameraSystemParamsFile");
            } else {
                systemFileName = _root + "/system.properties";
            }

            java.io.FileInputStream in = new java.io.FileInputStream(systemFileName);
            systemProps.load(in);
        } catch (IOException ioe) {
        }

        return systemProps;
    }

    private void saveTemplateParams(Properties templateParams)
        throws Exception {
        try {
            String templateFileName;

            if (_cameraProps.containsKey("PropertyDescriptionsDir")) {
                templateFileName = _root + "/" +
                    _cameraProps.getProperty("CameraTemplateParamsFile");
            } else {
                templateFileName = _root + "/template.properties";
            }

            FileOutputStream out = new FileOutputStream(templateFileName);
            templateParams.store(out, "---iCAM Job Properties---");
            out.close();
        } catch (Exception ex) {
            //ex.printStackTrace();
            throw ex;
        }
    }

    private void saveSystemParams(Properties systemParams)
        throws Exception {
        try {
            String systemFileName;

            if (_cameraProps.containsKey("PropertyDescriptionsDir")) {
                systemFileName = _root + "/" +
                    _cameraProps.getProperty("CameraSystemParamsFile");
            } else {
                systemFileName = _root + "/system.properties";
            }

            FileOutputStream out = new FileOutputStream(systemFileName);
            systemParams.store(out, "---iCAM System Properties---");
            out.close();
        } catch (Exception ex) {
            //ex.printStackTrace();
            throw ex;
        }
    }

    public void getPropertyFloatArray(String propertyName,
        ArrayOfFloatsHolder floatValuesArray, StringHolder error)
        throws RemoteException {
        //if(_channelProps.containsKey(propertyName) && _channelData.containsKey(_channelProps.get(propertyName))) {
        if (_channelData.containsKey(propertyName)) {
            //float[] res = (float[])(_channelData.get(_channelProps.get(propertyName)));
            float[] res = (float[]) (_channelData.get(propertyName));
            floatValuesArray.value = new ArrayOfFloats(res);
            error.value = S_OK;
        } else {
            Property prop;

            if (_relationMap.containsKey(propertyName)) {
                prop = _relationMap.get(propertyName);
            } else {
                PropertyHolder propertyDefinition = new PropertyHolder();
                StringHolder errorStr = new StringHolder();
                getPropertyDefinition(propertyName, propertyDefinition, errorStr);

                if (errorStr.equals(S_OK)) {
                    prop = propertyDefinition.value;
                } else {
                    error.value = S_FAIL;

                    return;
                }
            }

            Properties props;

            if ((prop != null) && (prop.getPropertyGroup() != null) &&
                    prop.getPropertyGroup().equalsIgnoreCase("template")) {
                props = getTemplateProperties();
            } else if ((prop != null) && (prop.getPropertyGroup() != null) &&
                    prop.getPropertyGroup().equalsIgnoreCase("system")) {
                props = getSystemProperties();
            } else {
                error.value = S_FAIL;

                return;
            }

            if (props.containsKey(_flatMap.get(prop.getName()))) {
                if (prop.getPropertyType().equalsIgnoreCase(TYPE_FLOAT_ARRAY)) {
                    String value = props.getProperty(_flatMap.get(
                                prop.getName()));

                    //if(value.endsWith("|")) {
                    //	value = value.substring(0, value.length() - 1);
                    //}
                    String[] valParts = value.split("\\|");
                    float[] res = new float[valParts.length];

                    for (int i = 0; i < res.length; i++) {
                        res[i] = Float.valueOf(valParts[i]).floatValue();
                    }

                    floatValuesArray.value = new ArrayOfFloats(res);
                    error.value = S_OK;
                } else {
                    error.value = S_FAIL;
                }
            } else {
                error.value = S_FAIL;
            }
        }
    }

    public void getPropertyImage(String channel, String propertyName,
        ByteArrayHolder image, StringHolder error) throws RemoteException {
        /*Iterator it = _channelProps.keySet().iterator();
        while(it.hasNext()) {
            String key = (String)it.next();
            System.out.println(key);
        }*/

        //if(_channelProps.containsKey(propertyName) && _channelData.containsKey(_channelProps.get(propertyName))) {
        if (_channelData.containsKey(propertyName)) {
            //byte[] res = (byte[])(_channelData.get(_channelProps.get(propertyName)));
            byte[] res = (byte[]) (_channelData.get(propertyName));
            image.value = res;
            error.value = S_OK;
        } else {
            error.value = S_FAIL;
        }
    }

    public void getPropertyIntArray(String propertyName,
        ArrayOfIntsHolder intValuesArray, StringHolder error)
        throws RemoteException {
        if (_channelData.containsKey(propertyName)) {
            //int[] res = (int[])(_channelData.get(_channelProps.get(propertyName)));
            int[] res = (int[]) (_channelData.get(propertyName));
            intValuesArray.value = new ArrayOfInts(res);
            error.value = S_OK;
        } else {
            Property prop;

            if (_relationMap.containsKey(propertyName)) {
                prop = _relationMap.get(propertyName);
            } else {
                PropertyHolder propertyDefinition = new PropertyHolder();
                StringHolder errorStr = new StringHolder();
                getPropertyDefinition(propertyName, propertyDefinition, errorStr);

                if (errorStr.equals(S_OK)) {
                    prop = propertyDefinition.value;
                } else {
                    error.value = S_FAIL;

                    return;
                }
            }

            Properties props;

            if ((prop != null) && (prop.getPropertyGroup() != null) &&
                    prop.getPropertyGroup().equalsIgnoreCase("template")) {
                props = getTemplateProperties();
            } else if ((prop != null) && (prop.getPropertyGroup() != null) &&
                    prop.getPropertyGroup().equalsIgnoreCase("system")) {
                props = getSystemProperties();
            } else {
                error.value = S_FAIL;

                return;
            }

            if (props.containsKey(_flatMap.get(prop.getName()))) {
                if (prop.getPropertyType().equalsIgnoreCase(TYPE_INT_ARRAY)) {
                    String value = props.getProperty(_flatMap.get(
                                prop.getName()));
                    String[] valParts = value.split("\\|");
                    int[] res = new int[valParts.length];

                    for (int i = 0; i < res.length; i++) {
                        res[i] = Integer.valueOf(valParts[i]).intValue();
                    }

                    intValuesArray.value = new ArrayOfInts(res);
                    error.value = S_OK;
                } else {
                    error.value = S_FAIL;
                }
            } else {
                error.value = S_FAIL;
            }
        }
    }

    public void getPropertyStringArray(String propertyName,
        ArrayOfStringsHolder stringValuesArray, StringHolder error)
        throws RemoteException {
        Property prop;

        if (_relationMap.containsKey(propertyName)) {
            prop = _relationMap.get(propertyName);
        } else {
            PropertyHolder propertyDefinition = new PropertyHolder();
            StringHolder errorStr = new StringHolder();
            getPropertyDefinition(propertyName, propertyDefinition, errorStr);

            if (errorStr.equals(S_OK)) {
                prop = propertyDefinition.value;
            } else {
                error.value = S_FAIL;

                return;
            }
        }

        Properties props;

        if ((prop != null) && (prop.getPropertyGroup() != null) &&
                prop.getPropertyGroup().equalsIgnoreCase("template")) {
            props = getTemplateProperties();
        } else if ((prop != null) && (prop.getPropertyGroup() != null) &&
                prop.getPropertyGroup().equalsIgnoreCase("system")) {
            props = getSystemProperties();
        } else {
            error.value = S_FAIL;

            return;
        }

        if (props.containsKey(_flatMap.get(prop.getName()))) {
            if (prop.getPropertyType().equalsIgnoreCase(TYPE_STRING_ARRAY)) {
                String value = props.getProperty(_flatMap.get(prop.getName()));
                String[] res = value.split("\\|");
                stringValuesArray.value = new ArrayOfStrings(res);
                error.value = S_OK;
            } else {
                error.value = S_FAIL;
            }
        } else {
            error.value = S_FAIL;
        }
    }

    public void getPropertyValueFloat(String propertyName,
        FloatHolder floatValue, StringHolder error) throws RemoteException {
        if (_channelData.containsKey(propertyName)) {
            float res = (Float) _channelData.get(propertyName);
            floatValue.value = res;
            error.value = S_OK;
        } else {
            Property prop;

            if (_relationMap.containsKey(propertyName)) {
                prop = _relationMap.get(propertyName);
            } else {
                PropertyHolder propertyDefinition = new PropertyHolder();
                StringHolder errorStr = new StringHolder();
                getPropertyDefinition(propertyName, propertyDefinition, errorStr);

                if (errorStr.equals(S_OK)) {
                    prop = propertyDefinition.value;
                } else {
                    error.value = S_FAIL;

                    return;
                }
            }

            Properties props;

            if ((prop != null) && (prop.getPropertyGroup() != null) &&
                    prop.getPropertyGroup().equalsIgnoreCase("template")) {
                props = getTemplateProperties();
            } else if ((prop != null) && (prop.getPropertyGroup() != null) &&
                    prop.getPropertyGroup().equalsIgnoreCase("system")) {
                props = getSystemProperties();
            } else {
                error.value = S_FAIL;

                return;
            }

            if (props.containsKey(_flatMap.get(prop.getName()))) {
                if (prop.getPropertyType().equalsIgnoreCase(TYPE_FLOAT)) {
                    String value = props.getProperty(_flatMap.get(
                                prop.getName()));
                    float res = Float.valueOf(value).floatValue();
                    floatValue.value = res;
                    error.value = S_OK;
                } else {
                    error.value = S_FAIL;
                }
            } else {
                error.value = S_FAIL;
            }
        }
    }

    public void getPropertyValueInt(String propertyName, IntHolder intValue,
        StringHolder error) throws RemoteException {
        if (_channelData.containsKey(propertyName)) {
            int res = (Integer) _channelData.get(propertyName);
            intValue.value = res;
            error.value = S_OK;
        } else {
            Property prop;

            if (_relationMap.containsKey(propertyName)) {
                prop = _relationMap.get(propertyName);
            } else {
                PropertyHolder propertyDefinition = new PropertyHolder();
                StringHolder errorStr = new StringHolder();
                getPropertyDefinition(propertyName, propertyDefinition, errorStr);

                if (errorStr.equals(S_OK)) {
                    prop = propertyDefinition.value;
                } else {
                    error.value = S_FAIL;

                    return;
                }
            }

            Properties props;

            if ((prop != null) && (prop.getPropertyGroup() != null) &&
                    prop.getPropertyGroup().equalsIgnoreCase("template")) {
                props = getTemplateProperties();
            } else if ((prop != null) && (prop.getPropertyGroup() != null) &&
                    prop.getPropertyGroup().equalsIgnoreCase("system")) {
                props = getSystemProperties();
            } else {
                error.value = S_FAIL;

                return;
            }

            if (props.containsKey(_flatMap.get(prop.getName()))) {
                if (prop.getPropertyType().equalsIgnoreCase(TYPE_INT)) {
                    String value = props.getProperty(_flatMap.get(
                                prop.getName()));
                    int res = Integer.valueOf(value).intValue();
                    intValue.value = res;
                    error.value = S_OK;
                } else {
                    error.value = S_FAIL;
                }
            } else {
                error.value = S_FAIL;
            }
        }
    }

    public void getPropertyValueString(String propertyName,
        StringHolder strValue, StringHolder error) throws RemoteException {
        Property prop;

        if (_relationMap.containsKey(propertyName)) {
            prop = _relationMap.get(propertyName);
        } else {
            PropertyHolder propertyDefinition = new PropertyHolder();
            StringHolder errorStr = new StringHolder();
            getPropertyDefinition(propertyName, propertyDefinition, errorStr);

            if (errorStr.equals(S_OK)) {
                prop = propertyDefinition.value;
            } else {
                error.value = S_FAIL;

                return;
            }
        }

        Properties props;

        if ((prop != null) && (prop.getPropertyGroup() != null) &&
                prop.getPropertyGroup().equalsIgnoreCase("template")) {
            props = getTemplateProperties();
        } else if ((prop != null) && (prop.getPropertyGroup() != null) &&
                prop.getPropertyGroup().equalsIgnoreCase("system")) {
            props = getSystemProperties();
        } else {
            error.value = S_FAIL;

            return;
        }

        if (props.containsKey(_flatMap.get(prop.getName()))) {
            if (prop.getPropertyType().equalsIgnoreCase(TYPE_STRING) ||
                    prop.getPropertyType().equalsIgnoreCase(TYPE_ENUMERATION) ||
                    prop.getPropertyType().equalsIgnoreCase(TYPE_DYNAMIC_LIST)) {
                String value = props.getProperty(_flatMap.get(prop.getName()));
                strValue.value = value;
                error.value = S_OK;
            } else {
                error.value = S_FAIL;
            }
        } else {
            error.value = S_FAIL;
        }
    }

    public String putFile(String fileName, byte[] fileContent)
        throws RemoteException {
        return S_OK;
    }

    public void selectPacket(String channel, String indexType, int index,
        IntHolder frameCounter, StringHolder error) throws RemoteException {
        _channelData.clear();

        if (!_currentChannel.equalsIgnoreCase(channel)) {
            _frameCnt = 0;
            _currentChannel = channel;
        }

        File packetDir = new File(getPacketDirName());

        if (!packetDir.exists()) {
            throw new RemoteException("No packet direcotry!");
        }

        String[] parts = channel.split("/");
        String channelName = parts[parts.length - 2];
        File channelFile = new File(getPacketDirName() + "/" + channelName);

        if (!channelFile.exists()) {
            throw new RemoteException("Invalid channel specified");
        }

        //Integer track = new Integer(-1);
        //try {
        //	track =  Integer.valueOf(parts[parts.length - 1]);
        //} catch ( NumberFormatException nfe ) {
        //	throw new RemoteException("No track specified");
        //}
        String track = "";
        track = parts[parts.length - 1];

        File trackFile = new File(getPacketDirName() + "/" + channelName + "/" +
                track);

        if (!trackFile.exists()) {
            throw new RemoteException("Invalid track specified");
        }

        String path = getPacketDirName() + "/" + channelName + "/" + track;
        File packetDirFile = new File(path);
        File[] packets = packetDirFile.listFiles();

        if (packets.length > 0) {
            if (!(indexType.equalsIgnoreCase("relative") ||
                    indexType.equalsIgnoreCase("frame") ||
                    indexType.equalsIgnoreCase("timestamp"))) {
                throw new RemoteException("Invalid index type");
            }

            if (indexType.equalsIgnoreCase("relative")) {
                switch (index) {
                case -2: {
                    _frameCnt = 0;

                    break; // first
                }

                case -1: {
                    if (_frameCnt == 0) {
                        _frameCnt = packets.length - 1;
                    } else {
                        _frameCnt--;
                    }

                    break;
                }

                case 1: {
                    if (_frameCnt == (packets.length - 1)) {
                        _frameCnt = 0;
                    } else {
                        _frameCnt++;
                    }

                    break;
                }

                case 2: {
                    _frameCnt = packets.length - 1;

                    break; // last							
                }
                }
            } else if (indexType.equalsIgnoreCase("frame")) {
                _frameCnt = index - 1;
            } else if (indexType.equalsIgnoreCase("timestamp")) {
                //PacketLister lst = new PacketLister(path, index);
                //PacketLister lst = new PacketLister(getRootDirName() + "/" +
                //        channelName + "/" + track, index);

                //try {
                //    _frameCnt = lst.locateClosest();
                //} catch (Exception e) {
                //    throw new RemoteException(e.getMessage());
                //}
            	boolean found = false;
            	for(int i = 0; i < packets.length; i++) {
            		String packetName = packets[i].getName();
            		if(packetName.startsWith("0")) {
            			packetName = packetName.substring(1);
            		}
            		if(packetName.equals(String.valueOf(index))) {
            			_frameCnt = i + 1;
            			found = true;
            			break;
            		}
            	}
            	if(!found) {
            		throw new RemoteException("No packet with the given timestamp found!");
            	}
            }

            frameCounter.value = _frameCnt + 1;

            HashMap<String, ServerProperty> propSectionInfo = new HashMap<String, ServerProperty>();
            ArrayOfEntriesHolder directoryContent = new ArrayOfEntriesHolder();
            StringHolder dirError = new StringHolder();
            getPropertyDirectory("/channel/" + channelName, directoryContent,
                dirError);

            DirectoryEntry[] channelContent;

            if (directoryContent.value != null) {
                channelContent = directoryContent.value.getItem();
            } else {
                channelContent = null;
            }

            if (channelContent != null) {
                for (int i = 0; i < channelContent.length; i++) {
                    if (channelContent[i].getEntryType()
                                             .equalsIgnoreCase("property")) {
                        PropertyHolder propertyDefinition = new PropertyHolder();
                        StringHolder errorProp = new StringHolder();
                        getPropertyDefinition(channelContent[i].getEntryName(),
                            propertyDefinition, errorProp);

                        if (errorProp.value.equals(S_OK)) {
                            //Property prop = propertyDefinition.value;
                            ServerProperty prop = _currentProperty;
                            String propName = propertyDefinition.value.getName();
                            //String type = propertyDefinition.value.getPropertyType();
                            //_channelProps.put(propName, _flatMap.get(propName));
                            //propSectionInfo.put(_flatMap.get(propName), prop);
                            propSectionInfo.put(propName, prop);
                        }
                    }
                }

                IcamDataPackageParser parser = new IcamDataPackageParser(propSectionInfo,
                        _flatMap);
                InputStream in;

                try {
                    in = new FileInputStream(packets[_frameCnt]);

                    ByteArrayOutputStream data = new ByteArrayOutputStream();
                    Stream.RedirectStream(in, data);

                    byte[] buffer = data.toByteArray();
                    data.close();
                    in.close();
                    parser.parse(buffer);

                    _channelData = parser.getPacketData();

                    /*Iterator it = _channelData.keySet().iterator();
                        while(it.hasNext()) {
                                String key = (String)it.next();
                                System.out.println(key);
                                float[] val = (float[])_channelData.get(key);
                                StringBuffer res = new StringBuffer();
                                for(int i = 0; i < val.length; i++) {
                                        res.append(val[i] + ",");
                                }
                                System.out.println(res);
                        }*/
                    error.value = S_OK;
                } catch (FileNotFoundException e) {
                    throw new RemoteException(e.getMessage());
                } catch (IOException e) {
                    throw new RemoteException(e.getMessage());
                } catch (CheckSumException e) {
                    throw new RemoteException(e.getMessage());
                }
            } else {
                if ((dirError != null) && (dirError.value != null)) {
                    error.value = dirError.value;
                } else {
                    error.value = S_FAIL;
                }
            }
        } else {
            error.value = S_OK;
        }

        /*getPropertyDirectory("/channel/" + channelName, directoryContent, dirError);
        if(dirError.value.equals(S_OK)) {
                IcamDataPackageParser parser = new IcamDataPackageParser(propSectionInfo);
            InputStream in;
                    try {
                            in = new FileInputStream(packets[_frameCnt]);
                            ByteArrayOutputStream data = new ByteArrayOutputStream();
                    Stream.RedirectStream(in, data);
                    byte[] buffer = data.toByteArray();
                    data.close();
                    in.close();
                        parser.parse(buffer);
        
                        _channelData = parser.getPacketData();
                        error.value = S_OK;
                    } catch (FileNotFoundException e) {
                            throw new RemoteException(e.getMessage());
                    } catch (IOException e) {
                            throw new RemoteException(e.getMessage());
                    } catch (CheckSumException e) {
                            throw new RemoteException(e.getMessage());
                    }
        } else {
                if(dirError != null && dirError.value != null) {
                        error.value = dirError.value;
                } else {
                        error.value = S_FAIL;
                }
        }*/
    }

    public String setPropertyBinary(String propertyName, byte[] data)
        throws RemoteException {
        return S_OK;
    }

    public String setPropertyFloatArray(String propertyName,
        ArrayOfFloats floatValuesArray) throws RemoteException {
        float[] value = floatValuesArray.getItem();

        if (value != null) {
            StringBuffer res = new StringBuffer();

            for (int i = 0; i < value.length; i++) {
                res.append(value[i] + "|");
            }

            Property prop;

            if (_relationMap.containsKey(propertyName)) {
                prop = _relationMap.get(propertyName);
            } else {
                PropertyHolder propertyDefinition = new PropertyHolder();
                StringHolder errorStr = new StringHolder();
                getPropertyDefinition(propertyName, propertyDefinition, errorStr);

                if (errorStr.equals(S_OK)) {
                    prop = propertyDefinition.value;
                } else {
                    return S_FAIL;
                }
            }

            if (prop.getPropertyType().equalsIgnoreCase(TYPE_FLOAT_ARRAY)) {
                if (prop.getPropertyGroup().equalsIgnoreCase("system")) {
                    Properties props = getSystemProperties();

                    if (_flatMap.containsKey(prop.getName())) {
                        props.setProperty(_flatMap.get(prop.getName()),
                            res.toString());

                        try {
                            saveSystemParams(props);
                        } catch (Exception e) {
                            throw new RemoteException(e.getMessage());
                        }
                    } else {
                        return S_FAIL;
                    }
                } else if (prop.getPropertyGroup().equalsIgnoreCase("template")) {
                    Properties props = getTemplateProperties();

                    if (_flatMap.containsKey(prop.getName())) {
                        props.setProperty(_flatMap.get(prop.getName()),
                            res.toString());

                        try {
                            saveTemplateParams(props);
                        } catch (Exception e) {
                            throw new RemoteException(e.getMessage());
                        }
                    } else {
                        return S_FAIL;
                    }
                }

                return S_OK;
            } else {
                return S_FAIL;
            }
        } else {
            return S_FAIL;
        }
    }

    public String setPropertyIntArray(String propertyName,
        ArrayOfInts intValuesArray) throws RemoteException {
        int[] value = intValuesArray.getItem();

        if (value != null) {
            StringBuffer res = new StringBuffer();

            for (int i = 0; i < value.length; i++) {
                res.append(value[i] + "|");
            }

            Property prop;

            if (_relationMap.containsKey(propertyName)) {
                prop = _relationMap.get(propertyName);
            } else {
                PropertyHolder propertyDefinition = new PropertyHolder();
                StringHolder errorStr = new StringHolder();
                getPropertyDefinition(propertyName, propertyDefinition, errorStr);

                if (errorStr.equals(S_OK)) {
                    prop = propertyDefinition.value;
                } else {
                    return S_FAIL;
                }
            }

            if (prop.getPropertyType().equalsIgnoreCase(TYPE_INT_ARRAY)) {
                if (prop.getPropertyGroup().equalsIgnoreCase("system")) {
                    Properties props = getSystemProperties();

                    if (_flatMap.containsKey(prop.getName())) {
                        props.setProperty(_flatMap.get(prop.getName()),
                            res.toString());

                        try {
                            saveSystemParams(props);
                        } catch (Exception e) {
                            throw new RemoteException(e.getMessage());
                        }
                    } else {
                        return S_FAIL;
                    }
                } else if (prop.getPropertyGroup().equalsIgnoreCase("template")) {
                    Properties props = getTemplateProperties();

                    if (_flatMap.containsKey(prop.getName())) {
                        props.setProperty(_flatMap.get(prop.getName()),
                            res.toString());

                        try {
                            saveTemplateParams(props);
                        } catch (Exception e) {
                            throw new RemoteException(e.getMessage());
                        }
                    } else {
                        return S_FAIL;
                    }
                }

                return S_OK;
            } else {
                return S_FAIL;
            }
        } else {
            return S_FAIL;
        }
    }

    public String setPropertyStringArray(String propertyName,
        ArrayOfStrings stringValuesArray) throws RemoteException {
        String[] value = stringValuesArray.getItem();

        if (value != null) {
            StringBuffer res = new StringBuffer();

            for (int i = 0; i < value.length; i++) {
                res.append(value[i] + "|");
            }

            Property prop;

            if (_relationMap.containsKey(propertyName)) {
                prop = _relationMap.get(propertyName);
            } else {
                PropertyHolder propertyDefinition = new PropertyHolder();
                StringHolder errorStr = new StringHolder();
                getPropertyDefinition(propertyName, propertyDefinition, errorStr);

                if (errorStr.equals(S_OK)) {
                    prop = propertyDefinition.value;
                } else {
                    return S_FAIL;
                }
            }

            if (prop.getPropertyType().equalsIgnoreCase(TYPE_STRING_ARRAY)) {
                if (prop.getPropertyGroup().equalsIgnoreCase("system")) {
                    Properties props = getSystemProperties();

                    if (_flatMap.containsKey(prop.getName())) {
                        props.setProperty(_flatMap.get(prop.getName()),
                            res.toString());

                        try {
                            saveSystemParams(props);
                        } catch (Exception e) {
                            throw new RemoteException(e.getMessage());
                        }
                    } else {
                        return S_FAIL;
                    }
                } else if (prop.getPropertyGroup().equalsIgnoreCase("template")) {
                    Properties props = getTemplateProperties();

                    if (_flatMap.containsKey(prop.getName())) {
                        props.setProperty(_flatMap.get(prop.getName()),
                            res.toString());

                        try {
                            saveTemplateParams(props);
                        } catch (Exception e) {
                            throw new RemoteException(e.getMessage());
                        }
                    } else {
                        return S_FAIL;
                    }
                }

                return S_OK;
            } else {
                return S_FAIL;
            }
        } else {
            return S_FAIL;
        }
    }

    public String setPropertyValueFloat(String propertyName, float floatValue)
        throws RemoteException {
        Property prop;

        if (_relationMap.containsKey(propertyName)) {
            prop = _relationMap.get(propertyName);
        } else {
            PropertyHolder propertyDefinition = new PropertyHolder();
            StringHolder errorStr = new StringHolder();
            getPropertyDefinition(propertyName, propertyDefinition, errorStr);

            if (errorStr.equals(S_OK)) {
                prop = propertyDefinition.value;
            } else {
                return S_FAIL;
            }
        }

        if (prop.getPropertyType().equalsIgnoreCase(TYPE_FLOAT)) {
            if (prop.getPropertyGroup().equalsIgnoreCase("system")) {
                Properties props = getSystemProperties();

                if (_flatMap.containsKey(prop.getName())) {
                    props.setProperty(_flatMap.get(prop.getName()),
                        Float.toString(floatValue));

                    try {
                        saveSystemParams(props);
                    } catch (Exception e) {
                        throw new RemoteException(e.getMessage());
                    }
                } else {
                    return S_FAIL;
                }
            } else if (prop.getPropertyGroup().equalsIgnoreCase("template")) {
                Properties props = getTemplateProperties();

                if (_flatMap.containsKey(prop.getName())) {
                    props.setProperty(_flatMap.get(prop.getName()),
                        Float.toString(floatValue));

                    try {
                        saveTemplateParams(props);
                    } catch (Exception e) {
                        throw new RemoteException(e.getMessage());
                    }
                } else {
                    return S_FAIL;
                }
            }

            return S_OK;
        } else {
            return S_FAIL;
        }
    }

    public String setPropertyValueInt(String propertyName, int intValue)
        throws RemoteException {
        Property prop;

        if (_relationMap.containsKey(propertyName)) {
            prop = _relationMap.get(propertyName);
        } else {
            PropertyHolder propertyDefinition = new PropertyHolder();
            StringHolder errorStr = new StringHolder();
            getPropertyDefinition(propertyName, propertyDefinition, errorStr);

            if (errorStr.equals(S_OK)) {
                prop = propertyDefinition.value;
            } else {
                return S_FAIL;
            }
        }

        if (prop.getPropertyType().equalsIgnoreCase(TYPE_INT)) {
            if (prop.getPropertyGroup().equalsIgnoreCase("system")) {
                Properties props = getSystemProperties();

                if (_flatMap.containsKey(prop.getName())) {
                    props.setProperty(_flatMap.get(prop.getName()),
                        Integer.toString(intValue));

                    try {
                        saveSystemParams(props);
                    } catch (Exception e) {
                        throw new RemoteException(e.getMessage());
                    }
                } else {
                    return S_FAIL;
                }
            } else if (prop.getPropertyGroup().equalsIgnoreCase("template")) {
                Properties props = getTemplateProperties();

                if (_flatMap.containsKey(prop.getName())) {
                    props.setProperty(_flatMap.get(prop.getName()),
                        Integer.toString(intValue));

                    try {
                        saveTemplateParams(props);
                    } catch (Exception e) {
                        throw new RemoteException(e.getMessage());
                    }
                } else {
                    return S_FAIL;
                }
            }

            return S_OK;
        } else {
            return S_FAIL;
        }
    }

    public String setPropertyValueString(String propertyName, String stringValue)
        throws RemoteException {
        Property prop;

        if (_relationMap.containsKey(propertyName)) {
            prop = _relationMap.get(propertyName);
        } else {
            PropertyHolder propertyDefinition = new PropertyHolder();
            StringHolder errorStr = new StringHolder();
            getPropertyDefinition(propertyName, propertyDefinition, errorStr);

            if (errorStr.equals(S_OK)) {
                prop = propertyDefinition.value;
            } else {
                return S_FAIL;
            }
        }

        if (prop.getPropertyType().equalsIgnoreCase(TYPE_STRING) ||
                prop.getPropertyType().equalsIgnoreCase(TYPE_ENUMERATION) ||
                prop.getPropertyType().equalsIgnoreCase(TYPE_DYNAMIC_LIST)) {
            if (prop.getPropertyGroup().equalsIgnoreCase("system")) {
                Properties props = getSystemProperties();

                if (_flatMap.containsKey(prop.getName())) {
                    props.setProperty(_flatMap.get(prop.getName()), stringValue);

                    try {
                        saveSystemParams(props);
                    } catch (Exception e) {
                        throw new RemoteException(e.getMessage());
                    }
                } else {
                    return S_FAIL;
                }
            } else if (prop.getPropertyGroup().equalsIgnoreCase("template")) {
                Properties props = getTemplateProperties();

                if (_flatMap.containsKey(prop.getName())) {
                    props.setProperty(_flatMap.get(prop.getName()), stringValue);

                    try {
                        saveTemplateParams(props);
                    } catch (Exception e) {
                        throw new RemoteException(e.getMessage());
                    }
                } else {
                    return S_FAIL;
                }
            }

            return S_OK;
        } else {
            return S_FAIL;
        }
    }

    public String storeSystemSettings() throws RemoteException {
        return S_OK;
    }

    public String storeUserTemplateSettings(String key)
        throws RemoteException {
        return S_OK;
    }

    public String subscribeForEvents(ArrayOfStrings listOfEventsToSubscribeFor)
        throws RemoteException {
        return S_OK;
    }

    public String unsubscribeForEvents(
        ArrayOfStrings listOfEventsToUnSubscribeFor) throws RemoteException {
        return S_OK;
    }
}
