package com.nts.isip.tcp;

import com.nts.isip.tcp.exceptions.CheckSumException;
import com.nts.isip.tcp.streams.IntelDataInputStream;

import ij.ImagePlus;

import ij.process.ByteProcessor;

import java.awt.Graphics2D;
import java.awt.Image;
import java.awt.image.BufferedImage;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;

import java.util.HashMap;
import java.util.Vector;

import javax.imageio.ImageIO;


public class IcamDataPackageParser {
    private final static int MAGICCHECKNUM = 0x12345678;
    private static HashMap<String, ServerProperty> _sectionInfo;
    private static HashMap<String, String> _flatMap = new HashMap<String, String>();
    private HashMap<String, Object> _data = new HashMap<String, Object>();

    public IcamDataPackageParser(HashMap<String, ServerProperty> sectionInfo,
        HashMap<String, String> flatMap) {
        _sectionInfo = sectionInfo;
        _flatMap = flatMap;
    }

    public void parse(byte[] data) throws IOException, CheckSumException {
        _data.clear();

        ByteArrayInputStream in = new ByteArrayInputStream(data);
        IntelDataInputStream inNum = new IntelDataInputStream(in);

        int magic = inNum.readIntelInt();
        int size = inNum.readIntelInt();
        int infoSize = inNum.readIntelInt();
        int sectnCnt = inNum.readIntelInt();
        int lastSecOff = inNum.readIntelInt();
        int checkSum = inNum.readIntelInt();

        int check = 0;
        check = check ^ magic ^ size ^ infoSize ^ sectnCnt ^ lastSecOff ^
            checkSum;

        if (check != MAGICCHECKNUM) {
            throw new CheckSumException("Check sum error");
        }

        int leftbytes = size - infoSize;
        int offset = 0 + infoSize;
        int checkSection;

        while (sectnCnt > 0) {
            in = new ByteArrayInputStream(data, offset, leftbytes);
            inNum = new IntelDataInputStream(in);

            String tag = inNum.readIntelString(4);
            int sectionSize = inNum.readIntelInt();

            if (sectionSize > leftbytes) {
                throw new CheckSumException("Section size error");
            }

            int sectionHdrSize = inNum.readIntelInt();
            int nextSectionOffset = inNum.readIntelInt();
            int sectionCheckSum = inNum.readIntelInt();

            byte[] tagBytes = tag.getBytes();
            int tagInt = tagBytes[0] + (tagBytes[1] * (1 << 8)) +
                (tagBytes[2] * (1 << 16)) + (tagBytes[3] * (1 << 24));

            checkSection = 0;
            checkSection = checkSection ^ tagInt ^ sectionSize ^
                sectionHdrSize ^ nextSectionOffset ^ sectionCheckSum;

            if (checkSection != MAGICCHECKNUM) {
                throw new CheckSumException("Check sum error");
            }

            /*if(_sectionInfo.containsKey(tag)) {
                    if(_sectionInfo.get(tag).getPropertyType().equals("intArray")) {
                            int[] intData = readIntArraySec(inNum);
                            _data.put(_sectionInfo.get(tag).getName(), intData);
                    } else if(_sectionInfo.get(tag).getPropertyType().equals("floatArray")) {
                            float[] floatData = readFloatArraySec(inNum);
                            _data.put(_sectionInfo.get(tag).getName(), floatData);
                    } else if(_sectionInfo.get(tag).getPropertyType().equals("image")) {
                            byte[] imageData = readImageSec(inNum, data,
                            offset + sectionHdrSize, sectionSize - sectionHdrSize);
                            _data.put(_sectionInfo.get(tag).getName(), imageData);
                    }
                    leftbytes -= sectionSize;
                offset += nextSectionOffset;
            } else {
                    leftbytes -= sectionSize;
                offset += nextSectionOffset;
            } */
            String[] propNamesArr = (String[]) _sectionInfo.keySet()
                                                           .toArray(new String[0]);
            Vector<ServerProperty> parseProps = new Vector<ServerProperty>();

            for (int i = 0; i < propNamesArr.length; i++) {
                String currPropName = propNamesArr[i];
                ServerProperty currProp = _sectionInfo.get(currPropName);

                if (_flatMap.get(currPropName).equals(tag)) {
                    if (currProp.getPropertyType().equals("image")) {
                        byte[] imageData = readImageSec(inNum, data,
                                offset + sectionHdrSize,
                                sectionSize - sectionHdrSize);
                        _data.put(currPropName, imageData);
                        parseProps.clear();

                        break;
                    } else {
                        parseProps.add(currProp);
                    }
                }
            }

            if (!parseProps.isEmpty()) {
                if (parseProps.get(0).getPropertyType()
                                  .equalsIgnoreCase("intArray") ||
                        parseProps.get(0).getPropertyType()
                                      .equalsIgnoreCase("int")) {
                    readIntArraySec(inNum, parseProps,
                        sectionSize - sectionHdrSize);

                    //_data.put(_sectionInfo.get(tag).getName(), intData);
                } else if (parseProps.get(0).getPropertyType()
                                         .equalsIgnoreCase("floatArray") ||
                        parseProps.get(0).getPropertyType()
                                      .equalsIgnoreCase("float")) {
                    readFloatArraySec(inNum, parseProps,
                        sectionSize - sectionHdrSize);
                }
            }

            leftbytes -= sectionSize;
            offset += nextSectionOffset;
            sectnCnt = sectnCnt - 1;
        }
    }

    private void readFloatArraySec(IntelDataInputStream inNum,
        Vector<ServerProperty> parseProps, int size) throws IOException {
        try {
            /*int newsize;
            int elementCount;
            if(parseProps.size() == 1 &&
               parseProps.get(0).getPropertyType().equalsIgnoreCase("floatarray")) {
                    newsize = size;
                    elementCount = 1;
            } else {
                    elementCount = inNum.readIntelInt();
                    newsize = size - 4;
            }*/
            int bytes = 0;
            int counter = 0;

            while (bytes < size) {
                boolean foundElem = false;

                for (int i = 0; i < parseProps.size(); i++) {
                    ServerProperty prop = parseProps.get(i);

                    if (prop.getElementOffset() == bytes) {
                        counter++;
                        foundElem = true;

                        int units = prop.getUnitsInElement();

                        //if(units > 1) {
                        if (prop.getPropertyType().equalsIgnoreCase("floatarray")) {
                            int arrsize = inNum.readIntelInt() * units;
                            float[] res = new float[arrsize];

                            for (int j = 0; j < arrsize; j++) {
                                res[j] = inNum.readIntelFloat();
                                bytes += 4;
                            }

                            //elementCount--;
                            /*float[] res = new float[units];
                            for(int j = 0; j < units; j++) {
                                    res[j] = inNum.readIntelFloat();
                                    bytes += 4;
                            }*/
                            _data.put(prop.getName(), res);
                        } else {
                            float res = inNum.readIntelFloat();
                            bytes += 4;
                            //elementCount--;
                            _data.put(prop.getName(), res);
                        }

                        break;
                    }
                }

                if (!foundElem) {
                    bytes += 4;
                    inNum.readIntelFloat();

                    //elementCount--;
                }

                if (counter >= parseProps.size()) { // || elementCount == 0) {

                    break;
                }
            }

            /*float[] res = new float[size];
            
            for (int i = 0; i < size; i++) {
                    res[i] = inNum.readIntelFloat();
            }
            return res;*/
        } catch (IOException e) {
            throw new IOException("Invalid FloatArray section");
        }
    }

    private void readIntArraySec(IntelDataInputStream inNum,
        Vector<ServerProperty> parseProps, int size) throws IOException {
        try {
            /*int newsize;
            int elementCount;
            if(parseProps.size() == 1 &&
               parseProps.get(0).getPropertyType().equalsIgnoreCase("intarray")) {
                    newsize = size;
                    elementCount = 1;
            } else {
                    elementCount = inNum.readIntelInt();
                    newsize = size - 4;
            }*/
            int bytes = 0;
            int counter = 0;

            while (bytes < size) {
                boolean foundElem = false;

                for (int i = 0; i < parseProps.size(); i++) {
                    ServerProperty prop = parseProps.get(i);

                    if (prop.getElementOffset() == bytes) {
                        counter++;
                        foundElem = true;

                        int units = prop.getUnitsInElement();

                        //if(units > 1) {
                        if (prop.getPropertyType().equalsIgnoreCase("intarray")) {
                            int arrsize = inNum.readIntelInt() * units;
                            int[] res = new int[arrsize];

                            for (int j = 0; j < arrsize; j++) {
                                res[j] = inNum.readIntelInt();
                                bytes += 4;
                            }

                            //elementCount--;
                            /*int[] res = new int[units];
                            for(int j = 0; j < units; j++) {
                                    res[j] = inNum.readIntelInt();
                                    bytes += 4;
                            }*/
                            _data.put(prop.getName(), res);
                        } else {
                            int res = inNum.readIntelInt();
                            bytes += 4;
                            //elementCount--;
                            _data.put(prop.getName(), res);
                        }

                        break;
                    }
                }

                if (!foundElem) {
                    inNum.readIntelInt();
                    bytes += 4;

                    //elementCount--;
                }

                if (counter >= parseProps.size()) { // || elementCount == 0) {

                    break;
                }
            }

            /*int[] res = new int[size];
            
            for (int i = 0; i < size; i++) {
                    res[i] = inNum.readIntelInt();
            }
            return res;*/
        } catch (IOException e) {
            throw new IOException("Invalid FloatArray section");
        }
    }

    private byte[] readImageSec(IntelDataInputStream inNum, byte[] data,
        int offset, int lenght) throws IOException {
        try {
            int width = inNum.readIntelInt();
            int height = inNum.readIntelInt();
            int bitDepth = inNum.readIntelInt();
            byte[] img = new byte[width * height];
            ByteArrayInputStream in = new ByteArrayInputStream(data,
                    offset + 12, lenght - 12);
            readAll(in, img, img.length);

            ByteProcessor bp = new ByteProcessor(width, height);
            bp.setPixels(img);

            Image image = new ImagePlus("img", bp).getImage();
            BufferedImage bu = new BufferedImage(width, height,
                    BufferedImage.TYPE_BYTE_GRAY);
            Graphics2D g2 = bu.createGraphics();
            g2.drawImage(image, 0, 0, null);
            g2.dispose();

            ByteArrayOutputStream bas = new ByteArrayOutputStream();
            ImageIO.write(bu, "bmp", bas);

            byte[] res = bas.toByteArray();

            return res;
        } catch (IOException e) {
            throw new IOException("Invalid Image section");
        }
    }

    private void readAll(InputStream in, byte[] buf, int len)
        throws IOException {
        int read = 0;
        int r;

        while ((read < len) && ((r = in.read(buf, read, len - read)) != -1)) {
            read += r;
        }
    }

    public HashMap<String, Object> getPacketData() {
        return _data;
    }
}
