package com.nts.isip.tcp;


/**
 * Contains useful functions for operating with system properties
 *
 */
public class PropertyUtil {
    public static int _conversionCoef = 1;

    /**
     * Creates a formatted property list string from an array of numbers
     * @param vals Array of numbers to convert to a formatted property string
     * @return A string that can be used, by convention, for a system property list
     */
    public static String MakeProperty(int[] vals) {
        StringBuffer str = new StringBuffer();

        for (int i = 0; i != vals.length; ++i) {
            str.append(vals[i]);
            str.append('|');
        }

        return str.toString();
    }

    public static String ConvertFromRobot(String val) {
        String[] allVals = val.split("\\|");
        StringBuilder strb = new StringBuilder();

        for (int i = 0; i != allVals.length; ++i) {
            float fVal = Float.parseFloat(allVals[i]);
            strb.append(ConvertFromRobot(fVal));

            if (i != (allVals.length - 1)) {
                strb.append('|');
            }
        }

        System.out.println("ConvertFromRobot: " + val + " -> " + strb);

        return strb.toString();
    }

    /**
     * Modifies the provided array to convert from system measurement units
     * to units used in the user interface (micrometers)
     */
    public static void ConvertFromRobot(int[] vals) {
        for (int i = 0; i != vals.length; ++i) {
            vals[i] = (int) ConvertFromRobot(vals[i]);
        }
    }

    /**
     * Converts the provided number from system measurement units into units
     * used in the user interface (micrometers)
     * @param val Number in system measurement units to convert
     * @return Number converted into micrometers
     */
    public static float ConvertFromRobot(float val) {
        return (_conversionCoef == 1) ? val : ((val * _conversionCoef) / 1000);
    }

    /**
     * Converts the provided number from units of the user interface (micrometers)
     * into system units
     * @param val A string representation of the number to convert
     * @return String representation of the converted number
     */
    public static String ConvertToRobot(String val) {
        String[] allVals = val.split("\\|");
        StringBuilder strb = new StringBuilder();

        for (int i = 0; i != allVals.length; ++i) {
            strb.append(ConvertToRobot(Float.parseFloat(allVals[i])));

            if (i != (allVals.length - 1)) {
                strb.append('|');
            }
        }

        System.out.println("ConvertToRobot: " + val + " -> " + strb);

        return strb.toString();
    }

    /**
     * Converts the provided number from units of the user interface (micrometers)
     * into system units
     * @param val Number to convert
     * @return Converted number
     */
    public static float ConvertToRobot(float val) {
        return (val * 1000) / _conversionCoef;
    }

    /**
     * Modifies the provided array to convert from user interface units (micrometers)
     * into system measurement units
     */
    public static void ConvertToRobot(int[] vals) {
        for (int i = 0; i != vals.length; ++i) {
            vals[i] = (int) ConvertToRobot(vals[i]);
        }
    }

    /**
     * Forms a hexadecimal string representation appropriate for sending
     * commands to head
     */
    public static String MakeHex(boolean val) {
        return MakeHex(val ? 1 : 0);
    }

    /**
     * Forms a hexadecimal string representation appropriate for sending
     * commands to head
     */
    public static String MakeHex(int val) {
        StringBuilder res = new StringBuilder();

        if (val < 0x10) {
            res.append('0');
        }

        res.append(Integer.toHexString(val).toUpperCase());

        return res.toString();
    }
}
