package com.nts.isip.tcp.isipgui;

import com.nts.isip.tcp.StreamThread;
import com.nts.isip.tcp.processors.FrameProcessorBase;
import com.nts.isip.tcp.processors.ImageJFrameProcessor;
import com.nts.isip.tcp.processors.MathematicaFrameProcessor;
import com.nts.isip.tcp.processors.ParsingFrameProcessor;
import com.nts.isip.tcp.processors.RematchingFrameProcessor;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.DefaultListModel;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.ListSelectionModel;
import javax.swing.event.ListDataEvent;
import javax.swing.event.ListDataListener;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;


public class CustomProcessorsSetterDialog extends JDialog {
    private String[] _processors;
    private RightList _rightList;
    private StreamThread _streamThr;
    private String _rematchDirName;
    private String _macrosDirName;
    private String _mathDirName;

    public CustomProcessorsSetterDialog(JFrame parent, String[] processors,
        StreamThread streamThr, String rematchDirName, String macrosDirName,
        String mathDirName) {
        super(parent, true);
        _processors = processors;
        _streamThr = streamThr;
        _rematchDirName = rematchDirName;
        _macrosDirName = macrosDirName;
        _mathDirName = mathDirName;
        this.setLocation(200, 200);
        this.init();
    }

    public void init() {
        JPanel pan = new JPanel();
        pan.setLayout(new BorderLayout());

        JList procLeftList = new JList(_processors);
        procLeftList.addMouseListener(new MouseListener() {
                public void mouseClicked(MouseEvent e) {
                    if (e.getClickCount() > 1) {
                        JList leftList = (JList) e.getSource();
                        addElementToList((String) leftList.getSelectedValue());
                    }
                }

                public void mousePressed(MouseEvent e) {
                }

                public void mouseReleased(MouseEvent e) {
                }

                public void mouseEntered(MouseEvent e) {
                }

                public void mouseExited(MouseEvent e) {
                }
            });

        JScrollPane scroll = new JScrollPane();
        scroll.getViewport().add(procLeftList);
        scroll.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED);
        scroll.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
        scroll.setSize(new Dimension(250, 270));
        scroll.setPreferredSize(new Dimension(250, 270));
        pan.add(scroll, BorderLayout.WEST);
        _rightList = new RightList(this);
        pan.add(_rightList, BorderLayout.CENTER);

        JPanel btnPanel = new JPanel();
        btnPanel.setSize(520, 50);
        btnPanel.setPreferredSize(new Dimension(520, 50));

        JButton okButton = new JButton("OK");
        okButton.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent arg0) {
                    String[] processors = _rightList.getProcessors();
                    setProcessors(processors);
                    dispose();
                }
            });

        JButton cancelButton = new JButton("Cancel");
        cancelButton.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent arg0) {
                    dispose();
                }
            });
        btnPanel.setLayout(new BoxLayout(btnPanel, BoxLayout.X_AXIS));
        btnPanel.add(Box.createHorizontalGlue());
        btnPanel.add(okButton);
        btnPanel.add(Box.createRigidArea(new Dimension(10, 0)));
        btnPanel.add(cancelButton);
        btnPanel.add(Box.createRigidArea(new Dimension(280, 0)));
        pan.add(btnPanel, BorderLayout.SOUTH);
        this.setSize(new Dimension(620, 350));
        this.setPreferredSize(new Dimension(620, 350));
        this.add(pan);
    }

    private void addElementToList(String elem) {
        _rightList.addElementToList(elem);
    }

    private void setProcessors(String[] processors) {
        ParsingFrameProcessor proc = new ParsingFrameProcessor();
        FrameProcessorBase linkedProc = null;

        for (int i = 0; i < processors.length; i++) {
            if (processors[i].equals("Parsing")) {
                ParsingFrameProcessor newProc = new ParsingFrameProcessor();

                if (linkedProc == null) {
                    proc.setFrameProcessor(newProc);
                    linkedProc = newProc;
                } else {
                    linkedProc.setFrameProcessor(newProc);
                    linkedProc = newProc;
                }
            } else if (processors[i].startsWith("Rematching")) {
                final String rematchFileName = processors[i].substring(11,
                        processors[i].length() - 1);
                final String rematchFile = _rematchDirName + "/" +
                    rematchFileName;
                RematchingFrameProcessor newProc = new RematchingFrameProcessor(rematchFile);

                if (linkedProc == null) {
                    proc.setFrameProcessor(newProc);
                    linkedProc = newProc;
                } else {
                    linkedProc.setFrameProcessor(newProc);
                    linkedProc = newProc;
                }
            } else if (processors[i].startsWith("Mathematica")) {
                final String mathFileName = processors[i].substring(12,
                        processors[i].length() - 1);
                final String mathFile = _mathDirName + "/" + mathFileName;
                MathematicaFrameProcessor newProc = new MathematicaFrameProcessor(mathFile);

                if (linkedProc == null) {
                    proc.setFrameProcessor(newProc);
                    linkedProc = newProc;
                } else {
                    linkedProc.setFrameProcessor(newProc);
                    linkedProc = newProc;
                }
            } else if (processors[i].startsWith("ImageJ")) {
                final String macrosFileName = processors[i].substring(7,
                        processors[i].length() - 1);
                final String macrosFile = _macrosDirName + "/" +
                    macrosFileName;
                ImageJFrameProcessor newProc = new ImageJFrameProcessor(macrosFile);

                if (linkedProc == null) {
                    proc.setFrameProcessor(newProc);
                    linkedProc = newProc;
                } else {
                    linkedProc.setFrameProcessor(newProc);
                    linkedProc = newProc;
                }
            }
        }

        _streamThr.setFrameProcessor(proc);
    }

    class RightList extends JPanel implements ListSelectionListener {
        private JList list;
        private DefaultListModel listModel;
        private JButton removeButton;
        private JButton upButton;
        private JButton downButton;
        private JButton editButton;
        private ActionListener listener;
        private JDialog parent;

        public RightList(JDialog parent) {
            super(new BorderLayout());
            this.parent = parent;
            this.setSize(new Dimension(370, 270));
            this.setPreferredSize(new Dimension(370, 270));
            this.setBackground(Color.white);
            listModel = new DefaultListModel();
            listModel.addListDataListener(new ListDataListener() {
                    public void intervalAdded(ListDataEvent arg0) {
                    }

                    public void intervalRemoved(ListDataEvent arg0) {
                    }

                    public void contentsChanged(ListDataEvent arg0) {
                        invalidate();
                    }
                });
            list = new JList(listModel);
            list.addListSelectionListener(this);

            JScrollPane scroll = new JScrollPane();
            scroll.getViewport().add(list);
            scroll.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED);
            scroll.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);

            removeButton = new JButton("Remove");
            removeButton.setMinimumSize(new Dimension(100, 25));
            removeButton.setMaximumSize(new Dimension(100, 25));
            removeButton.setPreferredSize(new Dimension(100, 25));
            removeButton.setToolTipText(
                "Move the currently selected list item higher.");
            removeButton.setEnabled(false);
            removeButton.addActionListener(new ActionListener() {
                    public void actionPerformed(ActionEvent arg0) {
                        ListSelectionModel lsm = list.getSelectionModel();
                        int firstSelected = lsm.getMinSelectionIndex();
                        int lastSelected = lsm.getMaxSelectionIndex();
                        listModel.removeRange(firstSelected, lastSelected);

                        int size = listModel.size();

                        if (size == 0) {
                            removeButton.setEnabled(false);
                            upButton.setEnabled(false);
                            downButton.setEnabled(false);
                            editButton.setEnabled(false);
                        } else {
                            if (firstSelected == listModel.getSize()) {
                                firstSelected--;
                            }

                            list.setSelectedIndex(firstSelected);
                        }
                    }
                });

            upButton = new JButton("Up");
            upButton.setMinimumSize(new Dimension(100, 25));
            upButton.setMaximumSize(new Dimension(100, 25));
            upButton.setPreferredSize(new Dimension(100, 25));
            upButton.setToolTipText(
                "Move the currently selected list item higher.");
            upButton.setEnabled(false);
            upButton.addActionListener(new ActionListener() {
                    public void actionPerformed(ActionEvent arg0) {
                        int move = list.getSelectedIndex();

                        if (move > 0) {
                            swap(move, move - 1);
                            list.setSelectedIndex(move - 1);
                            list.ensureIndexIsVisible(move - 1);
                        }
                    }
                });
            downButton = new JButton("Down");
            downButton.setToolTipText(
                "Move the currently selected list item lower.");
            downButton.setMinimumSize(new Dimension(100, 25));
            downButton.setMaximumSize(new Dimension(100, 25));
            downButton.setPreferredSize(new Dimension(100, 25));
            downButton.setEnabled(false);
            downButton.addActionListener(new ActionListener() {
                    public void actionPerformed(ActionEvent arg0) {
                        int move = list.getSelectedIndex();

                        if ((move >= 0) && (move != (listModel.getSize() - 1))) {
                            swap(move, move + 1);
                            list.setSelectedIndex(move + 1);
                            list.ensureIndexIsVisible(move + 1);
                        }
                    }
                });
            editButton = new JButton("Edit script");
            editButton.setMinimumSize(new Dimension(100, 25));
            editButton.setMaximumSize(new Dimension(100, 25));
            editButton.setPreferredSize(new Dimension(100, 25));
            editButton.setEnabled(false);

            if (listener != null) {
                editButton.removeActionListener(listener);
            }

            listener = new ActionListener() {
                        public void actionPerformed(ActionEvent e) {
                            String proc = (String) list.getSelectedValue();

                            if (proc.startsWith("Mathematica")) {
                                final String mathFileName = proc.substring(12,
                                        proc.length() - 1);
                                final String mathFile = _mathDirName + "/" +
                                    mathFileName;
                                createScriptEditor(mathFileName, mathFile);
                            } else {
                                final String macrosFileName = proc.substring(7,
                                        proc.length() - 1);
                                final String macrosFile = _macrosDirName + "/" +
                                    macrosFileName;
                                createScriptEditor(macrosFileName, macrosFile);
                            }
                        }
                    };
            editButton.addActionListener(listener);

            JPanel btnPanel = new JPanel();
            btnPanel.setSize(new Dimension(120, 250));
            btnPanel.setPreferredSize(new Dimension(120, 270));
            btnPanel.setLayout(new BoxLayout(btnPanel, BoxLayout.PAGE_AXIS));
            btnPanel.add(Box.createVerticalGlue());
            btnPanel.add(Box.createHorizontalGlue());
            btnPanel.add(Box.createRigidArea(new Dimension(20, 0)));
            btnPanel.add(removeButton);
            btnPanel.add(Box.createRigidArea(new Dimension(0, 10)));
            btnPanel.add(upButton);
            btnPanel.add(Box.createRigidArea(new Dimension(0, 10)));
            btnPanel.add(downButton);
            btnPanel.add(Box.createRigidArea(new Dimension(0, 10)));
            btnPanel.add(editButton);
            btnPanel.add(Box.createRigidArea(new Dimension(20, 0)));
            btnPanel.add(Box.createHorizontalGlue());
            btnPanel.add(Box.createVerticalGlue());
            this.add(scroll, BorderLayout.CENTER);
            this.add(btnPanel, BorderLayout.EAST);
        }

        private void swap(int i, int j) {
            Object x = listModel.getElementAt(i);
            Object y = listModel.getElementAt(j);
            listModel.set(i, y);
            listModel.set(j, x);
        }

        private void createScriptEditor(String fileName, String file) {
            new ScriptEditor(fileName, file, parent);
        }

        public void valueChanged(ListSelectionEvent e) {
            if (e.getValueIsAdjusting() == false) {
                if (list.getSelectedIndex() == -1) {
                    upButton.setEnabled(false);
                    downButton.setEnabled(false);
                    editButton.setEnabled(false);
                    removeButton.setEnabled(false);
                } else if (list.getSelectedIndices().length > 1) {
                    upButton.setEnabled(false);
                    downButton.setEnabled(false);
                    editButton.setEnabled(false);
                    removeButton.setEnabled(true);
                } else {
                    upButton.setEnabled(true);
                    downButton.setEnabled(true);
                    removeButton.setEnabled(true);

                    String procName = (String) list.getSelectedValue();

                    if ((procName.startsWith("Mathematica")) ||
                            (procName.startsWith("ImageJ"))) {
                        editButton.setEnabled(true);
                    } else {
                        editButton.setEnabled(false);
                    }
                }
            }
        }

        public void addElementToList(String elem) {
            listModel.addElement(elem);
        }

        public String[] getProcessors() {
            String[] processors = new String[listModel.getSize()];

            for (int i = 0; i < processors.length; i++) {
                processors[i] = (String) listModel.getElementAt(i);
            }

            return processors;
        }
    }
}
