package com.nts.isip.tcp.isipgui;

import com.nts.isip.tcp.ClientFrame;
import com.nts.isip.tcp.StreamThread;
import com.nts.isip.tcp.sources.FrameSourceBase;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import java.io.DataInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;

import java.rmi.RemoteException;

import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.Map;
import java.util.Properties;
import java.util.TreeMap;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JFileChooser;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;


public class IcamCommandPanel extends JPanel {
    private FrameSourceBase _icamSource;
    private ClientFrame _parent;
    private Map<Integer, ButtonRepresentation> _allButtons;
    private JTextArea _txtArea;
    private StreamThread _strThread;
    private boolean _cancelCommand;
    private Hashtable<String, String> _commandProps = new Hashtable<String, String>();

    //private StringBuffer _command;
    public IcamCommandPanel(StreamThread strThread, ClientFrame parent) {
        _strThread = strThread;
        _parent = parent;
        init();
    }

    public void init() {
        this.removeAll();
        _allButtons = new TreeMap<Integer, ButtonRepresentation>();

        Properties prop = new Properties();

        try {
            java.io.FileInputStream in = new java.io.FileInputStream(
                    "icam_command.properties");

            try {
                prop.load(in);
            } finally {
                in.close();
            }
        } catch (IOException e1) {
        }

        int maxButtonNumber = 0;
        Enumeration en = prop.keys();

        while (en.hasMoreElements()) {
            String key = (String) en.nextElement();

            ButtonRepresentation button;

            if (key.startsWith("button") && key.endsWith(".label")) {
                button = new ButtonRepresentation();

                int number = 0;
                Pattern p = Pattern.compile(".*?(\\d+).*?");
                Matcher m = p.matcher(key);

                if (m.find()) {
                    number = Integer.valueOf(m.group(1)).intValue();

                    if (number > maxButtonNumber) {
                        maxButtonNumber = number;
                    }
                }

                button.label = prop.getProperty(key);
                button.command = prop.getProperty("button" + number +
                        ".command");
                button.resFile = prop.getProperty("button" + number +
                        ".resultfile");
                button.targetDir = prop.getProperty("button" + number +
                        ".targetDir");

                try {
                    button.synch = Integer.valueOf(prop.getProperty("button" +
                                number + ".synch")).intValue();
                } catch (NumberFormatException e) {
                    button.synch = 0;
                }

                button.sourceFile = prop.getProperty("button" + number +
                        ".sourcefile");

                if (!button.label.equals("")) {
                    //_allButtons.add(button);
                    _allButtons.put(number, button);
                }
            }
        }

        this.setLayout(new BorderLayout());

        JPanel btnPan = new JPanel();
        btnPan.setLayout(new BoxLayout(btnPan, BoxLayout.Y_AXIS));

        Iterator it = _allButtons.keySet().iterator();

        while (it.hasNext()) {
            Object commandNumber = it.next();
            final ButtonRepresentation commandButton = _allButtons.get(commandNumber);
            JButton button = new JButton(commandButton.label);
            button.setSize(new Dimension(170, 25));
            button.setMinimumSize(new Dimension(170, 25));
            button.setMaximumSize(new Dimension(170, 25));
            button.setPreferredSize(new Dimension(170, 25));
            button.addActionListener(new ActionListener() {
                    public void actionPerformed(ActionEvent e) {
                        executeCommand(commandButton);
                    }
                });
            btnPan.add(button);
            //if(commandNumber != (_buttons.size() - 1)) {
            btnPan.add(Box.createRigidArea(new Dimension(0, 5)));

            //}
        }

        btnPan.add(Box.createHorizontalGlue());
        this.add(btnPan, BorderLayout.EAST);

        JPanel resultPan = new JPanel();
        _txtArea = new JTextArea();
        _txtArea.setEditable(false);
        resultPan.add(_txtArea);

        JScrollPane scroll = new JScrollPane();
        scroll.getViewport().add(resultPan);
        scroll.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED);
        scroll.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);

        this.add(scroll, BorderLayout.CENTER);
    }

    private String getPropertyValue(String propVal) {
        if ((propVal != null) && !(propVal.equals(""))) {
            String resVal;

            if (propVal.equals("*")) {
                JFileChooser selectFile = new JFileChooser();
                selectFile.setDialogType(JFileChooser.OPEN_DIALOG);
                selectFile.setDialogTitle("Select file");
                selectFile.setFileSelectionMode(JFileChooser.FILES_ONLY);
                selectFile.setApproveButtonText("OK");

                if (selectFile.showOpenDialog(this) == JFileChooser.APPROVE_OPTION) {
                    resVal = selectFile.getSelectedFile().getPath();
                } else {
                    resVal = "";
                }
            } else if (propVal.equals("?")) {
                JFileChooser selectFile = new JFileChooser();
                selectFile.setDialogType(JFileChooser.OPEN_DIALOG);
                selectFile.setDialogTitle("Select directory");
                selectFile.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
                selectFile.setApproveButtonText("OK");

                if (selectFile.showOpenDialog(this) == JFileChooser.APPROVE_OPTION) {
                    resVal = selectFile.getSelectedFile().getPath();
                } else {
                    resVal = "";
                }
            } else if (propVal.contains("~")) {
                _cancelCommand = false;

                String[] commandLine = propVal.split(" ");
                StringBuffer command = new StringBuffer();

                for (int i = 0; i < commandLine.length; i++) {
                    if (_cancelCommand) {
                        break;
                    }

                    if (commandLine[i].startsWith("~~?")) {
                        String[] paramParts = commandLine[i].split(":");

                        if (paramParts.length >= 3) {
                            String label = paramParts[1];

                            if (label.equals("")) {
                                label = "Parameter";
                            }

                            String param = paramParts[2].substring(0,
                                    paramParts[2].length() - 2);
                            IcamCommandLineDialog paramDlg = new IcamCommandLineDialog(this._parent,
                                    label, param, command, _commandProps, this);
                            paramDlg.setVisible(true);
                        }
                    } else {
                        command.append(commandLine[i] + " ");
                    }
                }

                if (!_cancelCommand) {
                    resVal = command.toString();
                } else {
                    _commandProps.clear();
                    resVal = null;
                }
            } else if (propVal.contains("$")) {
                int firstIdx = Integer.valueOf(propVal.indexOf("$", 0))
                                      .intValue() + 1;
                int secondIdx = Integer.valueOf(propVal.indexOf("$", firstIdx))
                                       .intValue();

                if (secondIdx > firstIdx) {
                    String propName = propVal.substring(firstIdx, secondIdx);

                    if (_commandProps.containsKey(propName)) {
                        resVal = propVal.substring(0, firstIdx - 1) +
                            _commandProps.get(propName) +
                            propVal.substring(secondIdx + 1, propVal.length());
                    } else {
                        resVal = null;
                    }
                } else {
                    resVal = null;
                }
            } else {
                resVal = propVal;
            }

            return resVal;
        } else {
            return null;
        }
    }

    private void executeCommand(ButtonRepresentation button) {
        _icamSource = _strThread.getStreamSource();
        _commandProps.clear();

        if (_icamSource != null) {
            Properties prop = new Properties();

            try {
                java.io.FileInputStream in = new java.io.FileInputStream(
                        "icam_command.properties");

                try {
                    prop.load(in);
                } finally {
                    in.close();
                }
            } catch (IOException e1) {
            }

            int number = 0;
            Enumeration en = prop.keys();

            while (en.hasMoreElements()) {
                String key = (String) en.nextElement();

                if (prop.getProperty(key).equals(button.label)) {
                    Pattern p = Pattern.compile(".*?(\\d+).*?");
                    Matcher m = p.matcher(key);

                    if (m.find()) {
                        number = Integer.valueOf(m.group(1)).intValue();
                        button.command = getPropertyValue(prop.getProperty(
                                    "button" + number + ".command"));
                        button.resFile = getPropertyValue(prop.getProperty(
                                    "button" + number + ".resultfile"));

                        try {
                            button.synch = Integer.valueOf(prop.getProperty(
                                        "button" + number + ".synch")).intValue();
                        } catch (NumberFormatException e) {
                            button.synch = 0;
                        }

                        button.sourceFile = getPropertyValue(prop.getProperty(
                                    "button" + number + ".sourcefile"));
                        button.targetDir = getPropertyValue(prop.getProperty(
                                    "button" + number + ".targetDir"));
                        button.finalFile = getPropertyValue(prop.getProperty(
                                    "button" + number + ".finalFile"));
                        button.finalDestination = getPropertyValue(prop.getProperty(
                                    "button" + number + ".finalDestination"));
                    }

                    break;
                }
            }

            if (button.sourceFile != null) {
                String filename = button.sourceFile;

                if (!filename.equals("")) {
                    File file = new File(filename);
                    FileInputStream is;

                    try {
                        is = new FileInputStream(file);

                        DataInputStream dis = new DataInputStream(is);
                        byte[] bytes = new byte[(int) file.length()];
                        dis.read(bytes);
                        is.close();

                        if (filename.contains("/")) {
                            filename = filename.substring(filename.lastIndexOf(
                                        "/") + 1, filename.length());
                        } else if (filename.contains("\\")) {
                            filename = filename.substring(filename.lastIndexOf(
                                        "\\") + 1, filename.length());
                        }

                        if (button.targetDir != null) {
                            if ((button.targetDir).endsWith("/")) {
                                filename = button.targetDir + filename;
                            } else {
                                filename = button.targetDir + "/" + filename;
                            }
                        }

                        //String errorCode = port.putFile(filename, bytes);
                        try {
                            _icamSource.putFile(filename, bytes);
                        } catch (IOException e) {
                            JOptionPane.showMessageDialog(_parent,
                                "Error while sending file to the server.\n" +
                                "The system told us: " + e.getMessage(),
                                "Error", JOptionPane.ERROR_MESSAGE);
                        }
                    } catch (FileNotFoundException e) {
                        JOptionPane.showMessageDialog(_parent,
                            "Error sending file to the server.\n" +
                            "The system told us: " + e.getMessage(), "Error",
                            JOptionPane.ERROR_MESSAGE);
                    } catch (IOException e) {
                        JOptionPane.showMessageDialog(_parent,
                            "Error sending file to the server.\n" +
                            "The system told us: " + e.getMessage(), "Error",
                            JOptionPane.ERROR_MESSAGE);
                    }
                }
            }

            if (button.command != null) {
                try {
                    //_icamSource.execute(_command.toString(), button.synch);
                    _icamSource.execute(button.command, button.synch);
                } catch (IOException e) {
                    JOptionPane.showMessageDialog(_parent,
                        "Error while executing command on the server.\n" +
                        "The system told us: " + e.getMessage(), "Error",
                        JOptionPane.ERROR_MESSAGE);
                }
            }

            if (button.finalFile != null) {
                String fileName;

                if (button.finalDestination != null) {
                    if ((button.finalDestination).endsWith("/")) {
                        fileName = button.finalDestination + button.finalFile;
                    } else {
                        fileName = button.finalDestination + "/" +
                            button.finalFile;
                    }
                } else {
                    fileName = button.finalFile;
                }

                try {
                    byte[] result = _icamSource.getFile(button.finalFile);

                    if (result != null) {
                        FileOutputStream fout = new FileOutputStream(fileName);
                        fout.write(result);
                        fout.close();
                    }
                } catch (RemoteException e) {
                    JOptionPane.showMessageDialog(_parent,
                        "Error getting the remote file from the server.\n" +
                        "The system told us: " + e.getMessage(), "Error",
                        JOptionPane.ERROR_MESSAGE);
                } catch (FileNotFoundException e) {
                    JOptionPane.showMessageDialog(_parent,
                        "Error getting the remote file from the server.\n" +
                        "The system told us: " + e.getMessage(), "Error",
                        JOptionPane.ERROR_MESSAGE);
                } catch (IOException e) {
                    JOptionPane.showMessageDialog(_parent,
                        "Error getting the remote file from the server.\n" +
                        "The system told us: " + e.getMessage(), "Error",
                        JOptionPane.ERROR_MESSAGE);
                }
            }

            if (button.resFile != null) {
                try {
                    byte[] result = _icamSource.getFile(button.resFile);
                    String resTxt;

                    if (result != null) {
                        resTxt = new String(result);
                    } else {
                        resTxt = "";
                    }

                    _txtArea.setText(resTxt);
                } catch (RemoteException e) {
                    JOptionPane.showMessageDialog(_parent,
                        "Error getting the result file from the server.\n" +
                        "The system told us: " + e.getMessage(), "Error",
                        JOptionPane.ERROR_MESSAGE);
                }
            }
        }
    }

    /*private void executeCommand1(ButtonRepresentation button) {
            //if(_strThread.getStreamSource() instanceof IcamFrameSource) {
            //        _icamSource = (IcamFrameSource)_strThread.getStreamSource();
            //}
            _icamSource = _strThread.getStreamSource();
            if(_icamSource != null) {
                    //ICAMgsoap port = _icamSource.getPort();
                    //if(port == null) {
                    //        String msg = "Error! " +
        //                "The system told us: Disconnected";
                    //        JOptionPane.showMessageDialog(_parent,
                    //                        msg, "Error",
                    //                        JOptionPane.ERROR_MESSAGE);
                    //} else {
                    Properties prop = new Properties();
    
                try {
                    java.io.FileInputStream in = new java.io.FileInputStream(
                            "icam_command.properties");
    
                    try {
                        prop.load(in);
                    } finally {
                        in.close();
                    }
                } catch (IOException e1) {
                }
    
                    int number = 0;
        Enumeration en = prop.keys();
        while (en.hasMoreElements()) {
            String key = (String) en.nextElement();
            if(prop.getProperty(key).equals(button.label)) {
                    Pattern p = Pattern.compile(".*?(\\d+).*?");
                       Matcher m = p.matcher(key);
                if (m.find()) {
                          number = Integer.valueOf(m.group(1)).intValue();
                           button.command = prop.getProperty("button" + number + ".command");
                           button.resFile = prop.getProperty("button" + number + ".resultfile");
                           try {
                                   button.synch = Integer.valueOf(prop.getProperty("button" + number + ".synch")).intValue();
                           } catch (NumberFormatException e) {
                                   button.synch = 0;
                           }
                           button.sourceFile = prop.getProperty("button" + number + ".sourcefile");
                           button.targetDir = prop.getProperty("button" + number + ".targetDir");
                           button.finalFile = prop.getProperty("button" + number + ".finalFile");
                           button.finalDestination = prop.getProperty("button" + number + ".finalDestination");
                }
                break;
            }
        }
    
    
    
                    //try {
                    if(button.sourceFile != null && !(button.sourceFile.equals(""))) {
                            String filename;
                            if(button.sourceFile.equals("?")) {
                                    JFileChooser selectFile = new JFileChooser();
                                selectFile.setDialogType(JFileChooser.OPEN_DIALOG);
                                selectFile.setDialogTitle("Select file");
                                selectFile.setFileSelectionMode(JFileChooser.FILES_ONLY);
                                selectFile.setApproveButtonText("OK");
                                if (selectFile.showOpenDialog(this) == JFileChooser.APPROVE_OPTION) {
                                        filename = selectFile.getSelectedFile().getPath();
                                } else {
                                        filename = "";
                                }
                            } else {
                                    filename = button.sourceFile;
                            }
    
                            if(!filename.equals("")) {
                                    File file = new File(filename);
                                    FileInputStream is;
                                    try {
                                            is = new FileInputStream(file);
                                            DataInputStream dis = new DataInputStream(is);
                                            byte[] bytes = new byte[(int)file.length()];
                                            dis.read(bytes);
                                            is.close();
    
                                            if(filename.contains("/")) {
                                                    filename = filename.substring(filename.lastIndexOf("/") + 1, filename.length());
                                            } else if(filename.contains("\\")) {
                                                            filename = filename.substring(filename.lastIndexOf("\\") + 1, filename.length());
                                            }
                                            if( (button.targetDir).endsWith("/")) {
                                                    filename = button.targetDir + filename;
                                            } else {
                                                    filename = button.targetDir + "/" + filename;
                                            }
                                            //String errorCode = port.putFile(filename, bytes);
                                            try {
                                                    _icamSource.putFile(filename, bytes);
                                            } catch (IOException e) {
                                                    JOptionPane.showMessageDialog(_parent,
                                                                    "Error while sending file to the server.\n" +
                                                                    "The system told us: " + e.getMessage(), "Error",
                                                                    JOptionPane.ERROR_MESSAGE);
                                            }
                                    } catch (FileNotFoundException e) {
                                            JOptionPane.showMessageDialog(_parent,
                                                            "Error sending file to the server.\n" +
                                                            "The system told us: " + e.getMessage(), "Error",
                                                            JOptionPane.ERROR_MESSAGE);
                                    } catch (IOException e) {
                                            JOptionPane.showMessageDialog(_parent,
                                                    "Error sending file to the server.\n" +
                                                            "The system told us: " + e.getMessage(), "Error",
                                                            JOptionPane.ERROR_MESSAGE);
                                    }
                            }
                    }
                    if(button.command != null && !(button.command.equals(""))) {
                            _cancelCommand = false;
                            String[] commandLine = button.command.split(" ");
                            _command = new StringBuffer();
                            for(int i = 0; i < commandLine.length; i++) {
                                    if(_cancelCommand) {
                                            break;
                                    }
                                    if(commandLine[i].startsWith("~~?")) {
                                            String[] paramParts = commandLine[i].split(":");
                                            if(paramParts.length >= 3) {
                                                    String label = paramParts[1];
                                                    String param = paramParts[2].substring(0, paramParts[2].length() - 2);
                                                    IcamCommandLineDialog paramDlg = new IcamCommandLineDialog(this._parent, label, param, _command, this);
                                                    paramDlg.setVisible(true);
                                            }
                                    } else {
                                            _command.append(commandLine[i] + " ");
                                    }
                            }
                            if(!_cancelCommand) {
                                    //System.out.println(_command.toString());
                                    //String errorCode = port.execute(_command.toString(), button.synch);
                                    try {
                                            _icamSource.execute(_command.toString(), button.synch);
                                    } catch (IOException e) {
                                            JOptionPane.showMessageDialog(_parent,
                                                    "Error while executing command on the server.\n" +
                                                    "The system told us: " + e.getMessage(), "Error",
                                                    JOptionPane.ERROR_MESSAGE);
                                    }
                            }
                    }
    
                    if(button.resFile != null && !(button.resFile.equals("")) && (!_cancelCommand) ) {
                            try {
                                    String result = _icamSource.getFile(button.resFile);
                                    _txtArea.setText(result);
                            } catch (RemoteException e) {
                                    JOptionPane.showMessageDialog(_parent,
                                                    "Error getting the result file from the server.\n" +
                                                    "The system told us: " + e.getMessage(),
                                                    "Error",
                                                    JOptionPane.ERROR_MESSAGE);
                            }
                    }
            }
    }*/
    public void setCanceledCommand(boolean canceled) {
        _cancelCommand = canceled;
    }
}


class ButtonRepresentation {
    public String label;
    public String command;
    public String resFile;
    public int synch;
    public String sourceFile;
    public String targetDir;
    public String finalFile;
    public String finalDestination;
}
