package com.nts.isip.tcp.isipgui;

import at.igm.www.iCAMWSDL.client.DirectoryEntry;

import com.nts.isip.tcp.PropertyRepresentation;
import com.nts.isip.tcp.models.IcamPropertyModel;
import com.nts.isip.tcp.sources.FrameSource;
import com.nts.isip.tcp.sources.FrameSourceBase;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Image;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import java.io.BufferedInputStream;
import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.InputStream;

import java.rmi.RemoteException;

import java.util.Enumeration;
import java.util.Properties;
import java.util.Vector;

import javax.imageio.ImageIO;

import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;


public class IcamDirectoryPropertyPanel extends JPanel {
    private IcamPropertyModel _model;
    private PropertyRepresentation[] _properties;
    private boolean _isTransactional;
    private String _dirName;
    private FrameSource _icamFrameSource;
    private JScrollPane _scroll;
    private IcamPropertyTable _table;

    public IcamDirectoryPropertyPanel(PropertyRepresentation[] properties,
        boolean isTransactional, String dirName, FrameSource icamSource) {
        _properties = properties;
        _isTransactional = isTransactional;
        _dirName = dirName;
        _icamFrameSource = icamSource;

        _model = new IcamPropertyModel(_properties, (FrameSourceBase) icamSource);

        this.init();
    }

    public void init() {
        this.setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));

        JPanel pan = new JPanel();
        pan.setLayout(new BoxLayout(pan, BoxLayout.Y_AXIS));

        String[] nameParts = _dirName.split("/");

        if ((_properties.length > 0) ||
                ((nameParts.length >= 2) &&
                nameParts[nameParts.length - 2].equals("channel"))) {
            _table = new IcamPropertyTable(_model, _icamFrameSource);
            _table.getColumnModel().getColumn(0).setMaxWidth(200);
            _table.getColumnModel().getColumn(0).setPreferredWidth(200);
            _table.getColumnModel().getColumn(0).setHeaderValue("Property");
            _table.getColumnModel().getColumn(1).setHeaderValue("Value");
            _table.setRowHeight(20);

            _scroll = new JScrollPane();
            _scroll.getViewport().add(_table);
            _scroll.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
            pan.add(_scroll);

            JPanel btnPanel = new JPanel();
            JButton refresh = new JButton("Refresh");
            refresh.addActionListener(new ActionListener() {
                    public void actionPerformed(ActionEvent e) {
                        //if(_icamFrameSource.getPort() != null) {
                        Vector<PropertyRepresentation> newProperties = new Vector<PropertyRepresentation>();

                        try {
                            DirectoryEntry[] res = _icamFrameSource.getPropertyDirectory(_dirName);

                            if ((res != null) && (res.length > 0)) {
                                for (int i = 0; i < res.length; i++) {
                                    if (res[i].getEntryType().equals("property")) {
                                        String[] nameParts = _dirName.split("/");
                                        PropertyRepresentation prop = _icamFrameSource.getPropertyDefinition(res[i].getEntryName());

                                        if ((nameParts.length >= 2) &&
                                                nameParts[nameParts.length - 2].equals(
                                                    "channel")) {
                                            Properties secProp = new Properties();

                                            try {
                                                java.io.FileInputStream in = new java.io.FileInputStream(
                                                        "isip_client_chart.properties");

                                                try {
                                                    secProp.load(in);
                                                } finally {
                                                    in.close();
                                                }
                                            } catch (IOException ex) {
                                            }

                                            Enumeration en = secProp.keys();

                                            if (!prop.type.equals("image") &&
                                                    !prop.type.equals(
                                                        "histogram")) {
                                                if (prop.type.equalsIgnoreCase(
                                                            "intarray") ||
                                                        prop.type.equalsIgnoreCase(
                                                            "floatarray")) {
                                                    boolean inPropFile = false;

                                                    while (en.hasMoreElements()) {
                                                        String key = (String) en.nextElement();

                                                        if (key.startsWith(
                                                                    "ser") &&
                                                                key.endsWith(
                                                                    ".PKGSECID")) {
                                                            if (secProp.getProperty(
                                                                        key)
                                                                           .equals(prop.info)) {
                                                                inPropFile = true;

                                                                break;
                                                            }
                                                        }
                                                    }

                                                    if (!inPropFile) {
                                                        prop.value = ((FrameSourceBase) _icamFrameSource).getProperty(prop.path,
                                                                prop.type, false);
                                                        newProperties.add(prop);
                                                    }
                                                } else {
                                                    prop.value = ((FrameSourceBase) _icamFrameSource).getProperty(prop.path,
                                                            prop.type, false);
                                                    newProperties.add(prop);
                                                }
                                            }
                                        } else {
                                            prop.value = ((FrameSourceBase) _icamFrameSource).getProperty(prop.path,
                                                    prop.type, false);
                                            newProperties.add(prop);
                                        }
                                    }
                                }
                            }
                        } catch (RemoteException ex) {
                            newProperties.clear();
                        }

                        if (newProperties.isEmpty()) {
                            _model = new IcamPropertyModel(null,
                                    (FrameSourceBase) _icamFrameSource);
                        } else {
                            _properties = newProperties.toArray(new PropertyRepresentation[0]);
                            _model = new IcamPropertyModel(_properties,
                                    (FrameSourceBase) _icamFrameSource);
                        }

                        _scroll.getViewport().removeAll();
                        _table = new IcamPropertyTable(_model, _icamFrameSource);
                        _table.getColumnModel().getColumn(0).setMaxWidth(200);
                        _table.getColumnModel().getColumn(0)
                              .setPreferredWidth(200);
                        _table.getColumnModel().getColumn(0)
                              .setHeaderValue("Property");
                        _table.getColumnModel().getColumn(1)
                              .setHeaderValue("Value");
                        _table.setRowHeight(20);
                        _scroll.getViewport().add(_table);

                        //}
                    }
                });
            btnPanel.add(refresh);

            if (_isTransactional) {
                JButton commit = new JButton("Commit");
                commit.addActionListener(new ActionListener() {
                        public void actionPerformed(ActionEvent e) {
                            try {
                                _icamFrameSource.commitTransaction(_dirName);
                            } catch (RemoteException ex) {
                                String msg = "Could not commit transaction.\n" +
                                    "The system told us: " + ex.getMessage();
                                JOptionPane.showMessageDialog(null, msg,
                                    "Error", JOptionPane.ERROR_MESSAGE);
                            }
                        }
                    });
                btnPanel.add(commit);

                JButton clear = new JButton("Clear");
                clear.addActionListener(new ActionListener() {
                        public void actionPerformed(ActionEvent e) {
                            try {
                                _icamFrameSource.clearTransaction(_dirName);
                            } catch (RemoteException ex) {
                                String msg = "Could not commit transaction.\n" +
                                    "The system told us: " + ex.getMessage();
                                JOptionPane.showMessageDialog(null, msg,
                                    "Error", JOptionPane.ERROR_MESSAGE);
                            }
                        }
                    });
                btnPanel.add(clear);
            }

            JButton pictogram = new JButton("Pictogram");
            pictogram.addActionListener(new ActionListener() {
                    public void actionPerformed(ActionEvent e) {
                        int row = _table.getSelectedRow();

                        if (row >= 0) {
                            byte[] pictogram = _properties[row].pictogram;

                            if ((pictogram != null) && (pictogram.length > 0)) {
                                final JDialog pictogramDlg = new JDialog();
                                pictogramDlg.setLayout(new BorderLayout());
                                pictogramDlg.setLocation(400, 200);

                                ByteArrayInputStream in = new ByteArrayInputStream(pictogram);
                                InputStream is = new BufferedInputStream(in);

                                try {
                                    Image image = ImageIO.read(is);
                                    pictogramDlg.setSize(new Dimension(
                                            image.getWidth(null),
                                            image.getHeight(null) + 70));

                                    PictogramPanel picPan = new PictogramPanel(image);
                                    pictogramDlg.add(picPan, BorderLayout.CENTER);
                                } catch (IOException e1) {
                                    e1.printStackTrace();
                                }

                                JPanel btnPan = new JPanel();
                                JButton ok = new JButton("OK");
                                ok.addActionListener(new ActionListener() {
                                        public void actionPerformed(
                                            ActionEvent e) {
                                            pictogramDlg.dispose();
                                        }
                                    });
                                btnPan.add(ok);
                                pictogramDlg.add(btnPan, BorderLayout.SOUTH);
                                pictogramDlg.setVisible(true);
                            }
                        }
                    }
                });
            btnPanel.add(pictogram);

            pan.add(btnPanel);
            this.add(pan);
        }
    }

    //}
    public IcamPropertyModel getModel() {
        return _model;
    }

    public void setModel(IcamPropertyModel model,
        PropertyRepresentation[] properties) {
        _model = model;
        _properties = properties;
        _scroll.getViewport().removeAll();
        _table = new IcamPropertyTable(_model, _icamFrameSource);
        _table.getColumnModel().getColumn(0).setMaxWidth(200);
        _table.getColumnModel().getColumn(0).setPreferredWidth(200);
        _table.getColumnModel().getColumn(0).setHeaderValue("Property");
        _table.getColumnModel().getColumn(1).setHeaderValue("Value");
        _table.setRowHeight(20);
        _scroll.getViewport().add(_table);
    }

    public String[] getPropertyNames() {
        String[] propNames = new String[_properties.length];

        for (int i = 0; i < _properties.length; i++) {
            propNames[i] = _properties[i].path;
        }

        return propNames;
    }
}


class PictogramPanel extends JPanel {
    private Image _pictogram;

    public PictogramPanel(Image pictogram) {
        _pictogram = pictogram;
    }

    public void paint(Graphics g) {
        if (this._pictogram != null) {
            g.drawImage(this._pictogram, 0, 0, this);
        }
    }
}
