package com.nts.isip.tcp.isipgui;

import com.nts.isip.tcp.ClientFrame;
import com.nts.isip.tcp.ClientView;
import com.nts.isip.tcp.CmdId;
import com.nts.isip.tcp.DirectoryRepresentation;
import com.nts.isip.tcp.HistogramRepresentation;
import com.nts.isip.tcp.ImageRepresentation;
import com.nts.isip.tcp.PainterThread;
import com.nts.isip.tcp.PropertyRepresentation;
import com.nts.isip.tcp.PropertyThread;
import com.nts.isip.tcp.PropertyUtil;
import com.nts.isip.tcp.StreamThread;
import com.nts.isip.tcp.XYSeriesTypeRenderer;
import com.nts.isip.tcp.models.IcamPropertyModel;
import com.nts.isip.tcp.sources.FrameSource;
import com.nts.isip.tcp.sources.FrameSourceBase;

import org.jfree.chart.ChartFactory;
import org.jfree.chart.ChartPanel;
import org.jfree.chart.JFreeChart;
import org.jfree.chart.annotations.XYAnnotation;
import org.jfree.chart.annotations.XYDrawableAnnotation;
import org.jfree.chart.annotations.XYPointerAnnotation;
import org.jfree.chart.labels.StandardXYToolTipGenerator;
import org.jfree.chart.plot.PlotOrientation;
import org.jfree.chart.plot.XYPlot;

import org.jfree.data.xy.XYSeries;
import org.jfree.data.xy.XYSeriesCollection;

import java.awt.BasicStroke;
import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import java.io.IOException;

import java.rmi.RemoteException;

import java.text.DecimalFormat;
import java.text.NumberFormat;

import java.util.ArrayList;
import java.util.Enumeration;
import java.util.Map;
import java.util.Properties;
import java.util.Vector;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.JTabbedPane;
import javax.swing.JTextField;
import javax.swing.SwingConstants;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;


public class IcamPropertiesPanel extends JPanel implements ClientView {
    private JLabel _frameCntLbl;
    private JLabel _frames;
    private Vector<PainterThread> _painter;
    private Vector<DisplayImagePanelTabs> _displayPanels;
    private Vector<ImageDisplayPanel> _allPanels;
    private int _popDataIdx;
    private XYSeriesCollection _serCol;
    private Vector<DataSeries> _2DData = new Vector<DataSeries>();
    private String _templatePropertyName = "";
    private ChartPanel _chartPan;
    private JFreeChart _chart;
    private XYSeriesTypeRenderer _chartRend;
    private boolean _average = true;
    private JTextField _trackPointField;
    private int _numberOfTrackingPoints = 0;
    private ArrayList<Float> _trackingPointList = new ArrayList<Float>();
    private JComboBox _trackComboBox;
    private LightLevelDisplayPanel _lightLevelPanel;
    
    private FrameSource _icamSource;
    private ClientFrame _parent;
    private StreamThread _strThread;
    private Map<Integer, Action> _cmdMap;
    private IcamDirectoryPropertyPanel _propPan;
    private Vector<IcamDirectoryPropertyPanel> _subscribedPropPan = new Vector<IcamDirectoryPropertyPanel>();
    
    private PropertyThread _propThread;

    public IcamPropertiesPanel(StreamThread strThread,
        Map<Integer, Action> cmdMap, ClientFrame parent) {
        _parent = parent;
        _strThread = strThread;
        _cmdMap = cmdMap;
    }

    public void updateIcamPanel(Vector<DirectoryRepresentation> dirs,
        JTabbedPane parentTabPane) {
        _propThread = _icamSource.createPropertyThread();

        Properties prop = new Properties();

        try {
            java.io.FileInputStream in = new java.io.FileInputStream(
                    "isip_client_chart.properties");

            try {
                prop.load(in);
            } finally {
                in.close();
            }
        } catch (IOException e1) {
        }

        try {
            long sleeptime = Long.valueOf(prop.getProperty("eventsleeptime"))
                                 .longValue();

            if (_propThread != null) {
                _propThread.setSleeptime(sleeptime);
            }
        } catch (NumberFormatException e) {
        }

        if (_propThread != null) {
            _propThread.start();
        }

        _icamSource.setPropertyThread(_propThread);

        this.removeAll();
        this.setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));

        JTabbedPane mainPanel = new JTabbedPane();
        mainPanel.setName("/");

        if (dirs != null) {
            for (int i = 0; i < dirs.size(); i++) {
                if (dirs.get(i).dirFullPath.equals("/")) {
                    this.setName(dirs.get(i).dirFullPath);

                    if (parentTabPane != null) {
                        parentTabPane.setTitleAt(0, dirs.get(i).label);
                    }

                    addPanels(dirs.get(i), this, mainPanel);

                    break;
                }
            }
        }
    }

    private void addPanels(DirectoryRepresentation dir, JPanel pan,
        JTabbedPane tbp) {
        String helpStr;

        if (dir.dirFullPath != "/") {
            helpStr = dir.dirFullPath.substring(1, dir.dirFullPath.length() -
                    1);
        } else {
            helpStr = dir.dirFullPath;
        }

        String[] pathParts = helpStr.split("/");

        if (dir.isChannel && (pathParts.length >= 2) &&
                pathParts[pathParts.length - 2].equals("channel")) {
            _propPan = new IcamDirectoryPropertyPanel(dir.properties.toArray(
                        new PropertyRepresentation[0]), dir.isTransactional,
                    dir.dirFullPath, _icamSource);

            for (int i = 0; i < _icamSource.getSubscribed().size(); i++) {
                String remove = _icamSource.getUnsubscribed().get(i);
                _icamSource.getUnsubscribed().add(remove);
                _icamSource.getSubscribed().remove(remove);
            }

            for (int i = 0; i < dir.properties.size(); i++) {
                _icamSource.getSubscribed().add(dir.properties.get(i).path);
            }

            _subscribedPropPan.clear();
            _subscribedPropPan.add(_propPan);

            String channelName = pathParts[pathParts.length - 1];
            _icamSource.setChannel(channelName);

            String[] trackNames = new String[dir.subDirs.size()];

            for (int i = 0; i < dir.subDirs.size(); i++) {
                trackNames[i] = dir.subDirs.get(i).label;
            }

            _trackComboBox = new JComboBox(trackNames);

            if ((trackNames != null) && (trackNames.length > 0)) {
                String[] trackParts = trackNames[0].split("/");
                _icamSource.setTrack(trackParts[trackParts.length - 1]);
            }

            _trackComboBox.addActionListener(new ActionListener() {
                    public void actionPerformed(ActionEvent e) {
                        JComboBox cb = (JComboBox) e.getSource();
                        String[] trackParts = ((String) cb.getSelectedItem()).split(
                                "/");

                        if ((trackParts != null) && (trackParts.length > 0)) {
                            String track = trackParts[trackParts.length - 1];
                            _icamSource.setTrack(track);
                        }
                    }
                });
            initComponents(pan, channelName);
        } else {
            IcamDirectoryPropertyPanel propPan = new IcamDirectoryPropertyPanel(dir.properties.toArray(
                        new PropertyRepresentation[0]), dir.isTransactional,
                    dir.dirFullPath, _icamSource);
            pan.add(propPan);
            
            _subscribedPropPan.add(propPan);

            for (int i = 0; i < dir.subDirs.size(); i++) {
                JPanel subpan = new JPanel();
                subpan.setLayout(new BoxLayout(subpan, BoxLayout.Y_AXIS));

                JTabbedPane tabPanel = new JTabbedPane();
                subpan.setName(dir.subDirs.get(i).dirFullPath);
                addPanels(dir.subDirs.get(i), subpan, tabPanel);
                tbp.add(subpan, dir.subDirs.get(i).label);
            }

            tbp.addChangeListener(new ChangeListener() {
                    public void stateChanged(ChangeEvent e) {
                        JTabbedPane pane = (JTabbedPane) e.getSource();
                        String selectedTab = pane.getSelectedComponent()
                                                 .getName();
                        String name = selectedTab.substring(0,
                                selectedTab.length() - 1);
                        String parentName = name.substring(0,
                                name.lastIndexOf("/")) + "/";
                        Pattern p = Pattern.compile(parentName + "(.*?)/(.*)");

                        for (int i = 0; i < _icamSource.getSubscribed().size();
                                i++) {
                            String remove = _icamSource.getSubscribed().get(i);
                            Matcher m = p.matcher(remove);

                            if (m.matches()) {
                                _icamSource.getUnsubscribed().add(remove);
                                _icamSource.getSubscribed().remove(remove);
                                i--;
                            }
                        }
                        
                        try {
                            Vector<DirectoryRepresentation> newDirs = _icamSource.getDirectoryData(selectedTab);
                            updatePanel(newDirs, selectedTab);
                        } catch (IOException ex) {
                            ex.printStackTrace();

                            String msg = "Could not connect to server.\n" +
                                "The system told us: " + ex.getMessage() +
                                "\n" + "Old data will be shown!";
                            JOptionPane.showMessageDialog(_parent, msg,
                                "Error", JOptionPane.ERROR_MESSAGE);
                        } catch (NullPointerException ex) {
                            ex.printStackTrace();
                        } catch (Exception ex) {
                            ex.printStackTrace();
                        }
                    }
                });

            if (tbp.getComponentCount() > 0) {
                pan.add(tbp);
                tbp.setSelectedComponent(tbp.getComponent(0));

                try {
                    Vector<DirectoryRepresentation> newDirs = _icamSource.getDirectoryData(tbp.getComponent(
                                0).getName());
                    updatePanel(newDirs, tbp.getComponent(0).getName());
                } catch (IOException ex) {
                    ex.printStackTrace();

                    String msg = "Could not connect to server.\n" +
                        "The system told us: " + ex.getMessage() + "\n" +
                        "Old data will be shown!";
                    JOptionPane.showMessageDialog(_parent, msg, "Error",
                        JOptionPane.ERROR_MESSAGE);
                } catch (NullPointerException ex) {
                    ex.printStackTrace();
                } catch (Exception ex) {
                    ex.printStackTrace();
                }
            }
        }

        tbp.setTabLayoutPolicy(JTabbedPane.SCROLL_TAB_LAYOUT);

        if (_propThread != null) {
            _propThread.setPanels(_subscribedPropPan);
        }
    }

    private void updatePanel(Vector<DirectoryRepresentation> newDirs,
        String tabName) {
        for (int i = 0; i < newDirs.size(); i++) {
            if (newDirs.get(i).dirFullPath.equals(tabName)) {
                _subscribedPropPan.clear();
                updatePanelData(newDirs.get(i), tabName, this);

                break;
            }
        }
    }

    private void updatePanelData(DirectoryRepresentation dir, String path,
        JPanel pan) {
        JPanel nextPanel = pan;

        if (nextPanel.getComponentCount() < 2) {
            return;
        }

        IcamDirectoryPropertyPanel subPropPan = (IcamDirectoryPropertyPanel) nextPanel.getComponent(0);
        _subscribedPropPan.add(subPropPan);

        JTabbedPane npan = (JTabbedPane) nextPanel.getComponent(1);

        if (npan.getComponentCount() == 0) {
            return;
        }

        for (int i = 0; i < npan.getComponentCount(); i++) {
            nextPanel = (JPanel) npan.getComponent(i);

            if (nextPanel.getName().equals(path)) {
                nextPanel.removeAll();
                nextPanel.setLayout(new BoxLayout(nextPanel, BoxLayout.Y_AXIS));

                JTabbedPane tbp = new JTabbedPane();
                addPanels(dir, nextPanel, tbp);

                break;
            }

            if (path.startsWith(nextPanel.getName())) {
                updatePanelData(dir, path, nextPanel);

                break;
            }
        }
    }

    private void initComponents(JPanel mainpan, String channel) {
        final String currChannel = channel;
        mainpan.setLayout(new BorderLayout());
        mainpan.add(_trackComboBox, BorderLayout.NORTH);

        JPanel pan = new JPanel();
        pan.setLayout(new BorderLayout());

        JPanel calcStreamPanel = new JPanel();
        calcStreamPanel.setLayout(new BoxLayout(calcStreamPanel,
                BoxLayout.Y_AXIS));

        JPanel imagePanel = new JPanel();
        imagePanel.setLayout(new BoxLayout(imagePanel, BoxLayout.Y_AXIS));
        _frameCntLbl = new JLabel("0");
        _frameCntLbl.setMaximumSize(new Dimension(200, 30));
        _frameCntLbl.setMinimumSize(new Dimension(200, 30));
        _frameCntLbl.setPreferredSize(new Dimension(200, 30));
        _frameCntLbl.setHorizontalTextPosition(SwingConstants.LEADING);
        _frameCntLbl.setAlignmentX(Component.RIGHT_ALIGNMENT);
        imagePanel.add(_frameCntLbl);

        JTabbedPane imageTabbedPane = new JTabbedPane();
        _painter = new Vector<PainterThread>();
        _displayPanels = new Vector<DisplayImagePanelTabs>();

        Properties prop = new Properties();

        try {
            java.io.FileInputStream in = new java.io.FileInputStream(
                    "isip_client_chart.properties");

            try {
                prop.load(in);
            } finally {
                in.close();
            }
        } catch (IOException e1) {
        }

        int numberOfImgTabs = 0;
        int imagesForOneTab = 0;
        Enumeration en1 = prop.keys();

        while (en1.hasMoreElements()) {
            String key = (String) en1.nextElement();

            if (key.endsWith(".tabidx")) {
                int number = 0;
                Pattern p = Pattern.compile(".*?(\\d+).*?");
                Matcher m = p.matcher(key);

                if (m.find()) {
                    number = Integer.valueOf(m.group(1)).intValue();
                }

                try {
                    String serChannel = prop.getProperty("ser" + number +
                            ".channel");
                    String serType = prop.getProperty("ser" + number +
                            ".PKGSECTYPE");

                    if (serType.equals("IMAGE") ||
                            serType.equalsIgnoreCase("HISTOGRAM")) {
                        if ((serChannel != null) && serChannel.equals(channel)) {
                            int tabidx = Integer.valueOf(prop.getProperty("ser" +
                                        number + ".tabidx")).intValue();

                            if (tabidx > numberOfImgTabs) {
                                numberOfImgTabs = tabidx;
                            }
                        }
                    }
                } catch (NumberFormatException e) {
                }
            }
        }

        _allPanels = new Vector<ImageDisplayPanel>();

        PopupImageDialog popupImgDialog;
        ImageDisplayPanel popUpPanel = new ImageDisplayPanel("Popup Image",
                _strThread);
        popupImgDialog = new PopupImageDialog(popUpPanel);
        popUpPanel.setDependVisibleParent(popupImgDialog);

        PainterThread painterThread = new PainterThread(popUpPanel);
        painterThread.setTabIdx(1);
        painterThread.setDataIdx(0);
        painterThread.start();

        _popDataIdx = 0;

        int imgDispPanIdx = 1;
        _painter.add(painterThread);

        if (numberOfImgTabs >= 1) {
            for (int i = 1; i <= numberOfImgTabs; i++) {
                imagesForOneTab = 0;

                JPanel simpleImgPan = new JPanel();
                GridLayout simpleImgLayout = new GridLayout(0, 1);
                simpleImgPan.setLayout(simpleImgLayout);

                Vector<ImageDisplayPanel> displayOneTabPanels = new Vector<ImageDisplayPanel>();

                Enumeration en2 = prop.keys();
                String tabName = "";

                while (en2.hasMoreElements()) {
                    String key = (String) en2.nextElement();

                    if (key.endsWith(".tabidx") &&
                            (Integer.valueOf(prop.getProperty(key)).intValue() == i)) {
                        int number = 0;
                        Pattern p = Pattern.compile(".*?(\\d+).*?");
                        Matcher m = p.matcher(key);

                        if (m.find()) {
                            number = Integer.valueOf(m.group(1)).intValue();
                        }

                        try {
                            String serChannel = prop.getProperty("ser" +
                                    number + ".channel");

                            if ((serChannel != null) &&
                                    serChannel.equals(channel)) {
                                int idx = Integer.valueOf(prop.getProperty(
                                            "ser" + number + ".idx")).intValue() +
                                    1;

                                if (idx > imagesForOneTab) {
                                    imagesForOneTab = idx;
                                }
                            }
                        } catch (NumberFormatException e) {
                        }

                        tabName = prop.getProperty("ser" + number + ".tabName");
                    }
                }

                if (imagesForOneTab > 0) {
                    for (int j = 0; j < imagesForOneTab; j++) {
                        ImageDisplayPanel imgDispPan = new ImageDisplayPanel(
                                " " + (j + 1) + " ", _strThread,
                                popupImgDialog, i, j, this);
                        displayOneTabPanels.add(imgDispPan);
                        imgDispPan.setId(imgDispPanIdx);
                        imgDispPan.setChannel(channel);
                        imgDispPanIdx++;
                        _allPanels.add(imgDispPan);
                        simpleImgPan.add(imgDispPan);

                        DisplayImagePanelTabs displayImgPanel = new DisplayImagePanelTabs();
                        displayImgPanel.pan = imgDispPan;
                        displayImgPanel.tabIdx = i;
                        displayImgPanel.histidx = j;
                        _displayPanels.add(displayImgPanel);

                        PainterThread imgPainterThread = new PainterThread(imgDispPan,
                                i, j, channel);
                        imgPainterThread.start();
                        _painter.add(imgPainterThread);
                    }

                    JScrollPane scroll = new JScrollPane();
                    scroll.getViewport().add(simpleImgPan);
                    scroll.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED);
                    scroll.setMinimumSize(new Dimension(300, 20));
                    imageTabbedPane.add(scroll, tabName);
                }
            }
        }

        if (imagesForOneTab > 0) {
            imagePanel.add(imageTabbedPane);

            ImageLayoutPanel imglayoutPan = new ImageLayoutPanel(_allPanels);
            imagePanel.add(imglayoutPan);
        }

        JTabbedPane matchingTabbedPane = new JTabbedPane();
        JPanel profPan = new JPanel();
        profPan = new JPanel();
        profPan.setLayout(new BoxLayout(profPan, BoxLayout.Y_AXIS));
        profPan.setBorder(BorderFactory.createTitledBorder("Profile data"));

        JPanel seriesPanel = new JPanel();
        seriesPanel.setLayout(new BoxLayout(seriesPanel, BoxLayout.X_AXIS));
        seriesPanel.add(Box.createHorizontalGlue());
        _serCol = new XYSeriesCollection();

        Enumeration en = prop.keys();
        _2DData.clear();
        while (en.hasMoreElements()) {
            String key = (String) en.nextElement();

            if (key.startsWith("ser") && key.endsWith(".idx")) {
                int number = 0;
                Pattern p = Pattern.compile(".*?(\\d+).*?");
                Matcher m = p.matcher(key);

                if (m.find()) {
                    number = Integer.valueOf(m.group(1)).intValue();
                }

                String name;
                String type;
                int idx = 0;
                String color;
                boolean lines;
                final boolean outline;
                boolean choosable;

                if (prop.containsKey("ser" + number + ".PKGSECID")) {
                    String secName = prop.getProperty("ser" + number +
                            ".PKGSECID");

                    if (secName.equals("SELECT")) {
                        if (prop.containsKey("ser" + number + ".propertyName")) {
                            _templatePropertyName = prop.getProperty("ser" +
                                    number + ".propertyName");
                        }
                    }
                }

                if (prop.containsKey("ser" + number + ".name")) {
                    name = prop.getProperty("ser" + number + ".name");
                } else {
                    name = "Series " + number;
                }

                if (prop.containsKey("ser" + number + ".type")) {
                    type = prop.getProperty("ser" + number + ".type");
                } else {
                    type = "points";
                }

                if (prop.containsKey("ser" + number + ".idx")) {
                    try {
                        idx = Integer.parseInt(prop.getProperty("ser" + number +
                                    ".idx"));
                    } catch (NumberFormatException e) {
                        continue;
                    }
                }

                if (prop.containsKey("ser" + number + ".color")) {
                    color = prop.getProperty("ser" + number + ".color");
                } else {
                    color = "black";
                }

                if (prop.containsKey("ser" + number + ".outline")) {
                    outline = Integer.parseInt(prop.getProperty("ser" + number +
                                ".outline")) != 0;
                } else {
                    outline = false;
                }

                if (prop.containsKey("ser" + number + ".lines")) {
                    try {
                        lines = Integer.parseInt(prop.getProperty("ser" +
                                    number + ".lines")) != 0;
                    } catch (NumberFormatException e) {
                        continue;
                    }
                } else {
                    lines = false;
                }

                String dsChannel = "";

                if (prop.containsKey("ser" + number + ".channel")) {
                    dsChannel = prop.getProperty("ser" + number + ".channel");
                }

                if (prop.containsKey("ser" + number + ".chooseLines")) {
                    try {
                        choosable = Integer.parseInt(prop.getProperty("ser" +
                                    number + ".chooseLines")) != 0;
                    } catch (NumberFormatException e) {
                        continue;
                    }
                } else {
                    choosable = false;
                }

                final DataSeries ds = new DataSeries();
                ds.setShow(true);
                ds.setOutline(outline);

                Action action = new AbstractAction(name) {
                        public void actionPerformed(ActionEvent evt) {
                            JCheckBox cb = (JCheckBox) evt.getSource();

                            if (cb.isSelected()) {
                                ds.setShow(true);
                                ds.setOutline(outline);
                            } else {
                                ds.setShow(false);
                                ds.setOutline(false);
                            }
                        }
                    };

                JCheckBox show = new JCheckBox(action);
                show.setSelected(true);

                if (!name.equals("Selected lines") &&
                        !name.equals("Current selected lines") &&
                        dsChannel.equals(channel)) {
                    seriesPanel.add(show);
                }

                ds.setSerData(new XYSeries(name, false));
                ds.setName(name);
                ds.setSerIdx(idx);
                ds.setType(type);
                ds.setColor(getColor(color));
                ds.setLines(lines);
                ds.setChoosable(choosable);
                ds.setChannel(dsChannel);
                if (ds.getChannel().equals(channel)) {
                    _2DData.add(ds);
                    _serCol.addSeries(ds.getSerData());
                }
            }
        }

        seriesPanel.add(Box.createHorizontalGlue());
        _chart = ChartFactory.createScatterPlot("Object profile", "x (mm)",
                "Distance (mm)", _serCol, PlotOrientation.VERTICAL, false,
                false, false);
        _chartPan = new ChartPanel(_chart);
        _chartPan.setLayout(new BorderLayout());

        XYPlot plot = (XYPlot) _chart.getPlot();
        _chartRend = new XYSeriesTypeRenderer(_2DData);
        _chartRend.drawSeries();
        _chartRend.setToolTipGenerator(new StandardXYToolTipGenerator());
        plot.setRenderer(_chartRend);
        _lightLevelPanel = new LightLevelDisplayPanel(_strThread);

        if (prop.containsKey("scale_x_min") && prop.containsKey("scale_x_max")) {
            plot.getDomainAxis()
                .setRange(Integer.parseInt(prop.getProperty("scale_x_min")),
                Integer.parseInt(prop.getProperty("scale_x_max")));
            this._lightLevelPanel.setScale(Integer.parseInt(prop.getProperty(
                        "scale_x_min")),
                Integer.parseInt(prop.getProperty("scale_x_max")));
        } else {
            plot.getDomainAxis().setRange(0, 12500);
        }

        if (prop.containsKey("scale_y_min") && prop.containsKey("scale_y_max")) {
            plot.getRangeAxis()
                .setRange(Integer.parseInt(prop.getProperty("scale_y_min")),
                Integer.parseInt(prop.getProperty("scale_y_max")));
        } else {
            plot.getRangeAxis().setRange(-10000, 0);
        }

        _chartPan.setDisplayToolTips(true);

        JCheckBox average = new JCheckBox();
        average.setSelected(_average);
        average.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    JCheckBox cb = (JCheckBox) e.getSource();

                    if (cb.isSelected()) {
                        _average = true;
                    } else {
                        _average = false;
                    }
                }
            });
        seriesPanel.add(average);
        seriesPanel.add(new JLabel("Avg. T.P."));
        _trackPointField = new JTextField(4);
        _trackPointField.setText(_numberOfTrackingPoints + "");
        seriesPanel.add(_trackPointField);
        seriesPanel.add(new JLabel("T.P. No to avg."));
        _chartPan.add(seriesPanel, BorderLayout.NORTH);
        profPan.add(_chartPan);

        JButton selectLines = new JButton("Choose Lines");
        selectLines.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    DataSeries ds = null;

                    for (int i = 0; i < _2DData.size(); i++) {
                        if (_2DData.get(i).getChoosable() &&
                                _2DData.get(i).getChannel().equals(currChannel)) {
                            ds = _2DData.get(i);

                            break;
                        }
                    }

                    if ((ds != null) && (ds.getSerData().getItemCount() != 0)) {
                        PropertyRepresentation prop;

                        try {
                            prop = _icamSource.getPropertyDefinition(_templatePropertyName);

                            LineChooserDialog lchd = new LineChooserDialog(prop,
                                    ds.getSerData(), _2DData, _strThread,
                                    _icamSource, _parent);
                            lchd.setVisible(true);
                        } catch (RemoteException ex) {
                        }
                    } else {
                        JOptionPane.showMessageDialog(_parent,
                            "No lines to select from!", "Information",
                            JOptionPane.INFORMATION_MESSAGE);
                    }
                }
            });
        profPan.add(selectLines);

        matchingTabbedPane.add(profPan, "Profile");

        //RecordFramePanel recordPanel = new RecordFramePanel(_strThread);
        //matchingTabbedPane.add(recordPanel, "Record");
        matchingTabbedPane.add(_lightLevelPanel, "Laser Intensity");

        matchingTabbedPane.add(_propPan, "Channel Properties");

        calcStreamPanel.add(matchingTabbedPane);
        calcStreamPanel.add(Box.createVerticalGlue());

        calcStreamPanel.setBorder(BorderFactory.createEmptyBorder(0, 10, 10, 10));

        if (imagesForOneTab > 0) {
            JSplitPane center = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT,
                    imagePanel, calcStreamPanel);
            pan.add(center, BorderLayout.CENTER);
        } else {
            pan.add(calcStreamPanel, BorderLayout.CENTER);
        }

        JPanel btnPanel = new JPanel();
        btnPanel.setLayout(new BoxLayout(btnPanel, BoxLayout.X_AXIS));
        btnPanel.add(Box.createHorizontalGlue());
        btnPanel.add(new JButton(_cmdMap.get(CmdId.FIRST_FRAME)));
        btnPanel.add(Box.createRigidArea(new Dimension(10, 0)));
        btnPanel.add(new JButton(_cmdMap.get(CmdId.PREV_FRAME)));
        btnPanel.add(Box.createRigidArea(new Dimension(20, 0)));

        //JButton frameRecorder = new JButton(_cmdMap.get(
        //            CmdId.STARTSTOP_FRAME_RECORDING));
        //btnPanel.add(frameRecorder);
        //btnPanel.add(Box.createRigidArea(new Dimension(10, 0)));
        btnPanel.add(new JButton(_cmdMap.get(CmdId.START_CALC_STREAM)));
        btnPanel.add(Box.createRigidArea(new Dimension(10, 0)));
        btnPanel.add(new JButton(_cmdMap.get(CmdId.STOP_STREAM)));
        btnPanel.add(Box.createRigidArea(new Dimension(10, 0)));
        btnPanel.add(new JButton(_cmdMap.get(CmdId.RELOAD_FRAME)));
        btnPanel.add(Box.createRigidArea(new Dimension(20, 0)));
        btnPanel.add(new JButton(_cmdMap.get(CmdId.NEXT_FRAME)));
        btnPanel.add(Box.createRigidArea(new Dimension(10, 0)));
        btnPanel.add(new JButton(_cmdMap.get(CmdId.LAST_FRAME)));
        btnPanel.add(Box.createRigidArea(new Dimension(10, 0)));
        btnPanel.add(new JButton(_cmdMap.get(CmdId.BY_TIME_FRAME)));
        btnPanel.add(Box.createRigidArea(new Dimension(55, 0)));

        //JButton updBtn = new JButton("Update...");
        //updBtn.addActionListener(new UpdateActionListener(_parent, _strThread));
        //btnPanel.add(updBtn);
        _frames = new JLabel();
        btnPanel.add(Box.createRigidArea(new Dimension(10, 0)));
        btnPanel.add(_frames);
        btnPanel.setBorder(BorderFactory.createEmptyBorder(10, 0, 0, 0));
        btnPanel.add(Box.createHorizontalGlue());
        pan.add(btnPanel, BorderLayout.SOUTH);

        mainpan.add(pan, BorderLayout.CENTER);
    }

    public void setIcamFrameSource(FrameSource icamSource) {
        _icamSource = icamSource;
    }

    public void display2DData(double[][] twoDData) {
        while (_serCol.getSeriesCount() > _2DData.size()) {
            _serCol.removeSeries(_2DData.size());
        }

        _chartRend.removeAnnotations();

        for (int i = 0; i < _2DData.size(); i++) {
            _chart.setNotify(false);
            _2DData.get(i).getSerData().clear();

            double[] data;
            int idx = _2DData.get(i).getSerIdx();

            if ((twoDData != null) && (twoDData.length >= (idx + 1))) {
                data = twoDData[idx];

                if (!_trackPointField.getText().equals("")) {
                    try {
                        _numberOfTrackingPoints = Integer.valueOf(_trackPointField.getText()
                                                                                  .trim())
                                                         .intValue();
                    } catch (NumberFormatException e) {
                        _trackPointField.setText("");
                        _numberOfTrackingPoints = 0;
                    }
                } else {
                    _numberOfTrackingPoints = 0;
                }

                if ((_2DData.get(i).getSerIdx() == 2) &&
                        (_numberOfTrackingPoints != 0) && _average) { //tracking point is at idx 2

                    if ((data != null) && _2DData.get(i).getShow()) {
                        if (data.length > 1) {
                            float x = PropertyUtil.ConvertFromRobot((float) data[0]);
                            float y = PropertyUtil.ConvertFromRobot((float) data[1]);

                            if (_trackingPointList.size() >= (2 * _numberOfTrackingPoints)) {
                                _trackingPointList.remove(0);
                                _trackingPointList.remove(0);
                            }

                            _trackingPointList.add(x);
                            _trackingPointList.add(y);

                            float averageX = 0;
                            float averageY = 0;

                            for (int j = 1; j < _trackingPointList.size();
                                    j += 2) {
                                averageX += _trackingPointList.get(j - 1);
                                averageY += _trackingPointList.get(j);
                            }

                            averageX = averageX / (_trackingPointList.size() / 2);
                            averageY = averageY / (_trackingPointList.size() / 2);

                            _2DData.get(i).getSerData().add(averageX, averageY);
                        }
                    }
                } else {
                    if ((data != null) && _2DData.get(i).getShow()) {
                        for (int j = 1; j < data.length; j += 2) {
                            _2DData.get(i).getSerData()
                                   .add(PropertyUtil.ConvertFromRobot(
                                    (float) data[j - 1]),
                                PropertyUtil.ConvertFromRobot((float) data[j]));
                        }
                    }
                }

                if (_2DData.get(i).getOutline()) {
                    XYSeries serData = _2DData.get(i).getSerData();
                    int numPoints = serData.getItemCount();

                    for (int p = 0; p < numPoints; p++) {
                        final XYSeriesTypeCircleDrawer cd = new XYSeriesTypeCircleDrawer(_2DData.get(
                                    i).getColor(), new BasicStroke(2.0f), null);
                        NumberFormat fmt = DecimalFormat.getInstance();
                        fmt.setMaximumFractionDigits(2);

                        final XYDrawableAnnotation targAnn = new XYDrawableAnnotation((double) serData.getX(
                                    p).doubleValue(),
                                (double) serData.getY(p).doubleValue(), 20, 20,
                                cd);
                        final XYAnnotation bestBid = new XYPointerAnnotation(
                                "Tracking Point[" +
                                fmt.format(serData.getX(p)) + " ; " +
                                fmt.format(serData.getY(p)) + "]",
                                (double) serData.getX(p).doubleValue(),
                                (double) serData.getY(p).doubleValue(),
                                (3.0 * Math.PI) / 4.0);
                        _chartRend.addAnnotation(bestBid);
                        _chartRend.addAnnotation(targAnn);
                    }
                }

                _chart.setNotify(true);
            }
        }
    }

    public void addCurrentSelectedPoints(double[] points, int idx) {
        for (int i = 0; i < _2DData.size(); i++) {
            if (_2DData.get(i).getSerIdx() == idx) {
                _chart.setNotify(false);
                _2DData.get(i).getSerData().clear();

                if ((points != null) && _2DData.get(i).getShow()) {
                    for (int j = 1; j < points.length; j += 2) {
                        _2DData.get(i).getSerData()
                               .add(PropertyUtil.ConvertFromRobot(
                                (float) points[j - 1]),
                            PropertyUtil.ConvertFromRobot((float) points[j]));
                    }
                }

                _chart.setNotify(true);
            }
        }
    }

    public void addSelectedPoints(double[] points, int idx) {
        for (int i = 0; i < _2DData.size(); i++) {
            if (_2DData.get(i).getSerIdx() == idx) {
                _chart.setNotify(false);

                if ((points != null) && _2DData.get(i).getShow()) {
                    for (int j = 1; j < points.length; j += 2) {
                        _2DData.get(i).getSerData()
                               .add(PropertyUtil.ConvertFromRobot(
                                (float) points[j - 1]),
                            PropertyUtil.ConvertFromRobot((float) points[j]));
                    }
                }

                _chart.setNotify(true);
            }
        }
    }

    public void display3DData(double[][] threeDData) {
    }

    public void displayFrame(int frameCnt, String frameName) {
        if (frameName != null) {
            _frameCntLbl.setText(Integer.toString(frameCnt) + " ( " +
                frameName + " )");
        } else {
            _frameCntLbl.setText(Integer.toString(frameCnt));
        }
    }

    public void displayFrameNumbers(int totalFrame, int currentFrame) {
        if ((totalFrame != 0) && (currentFrame != 0)) {
            _frames.setText("Frame " + currentFrame + " of " + totalFrame);
        } else {
            if (currentFrame == 0) {
                _frames.setText("");
            } else {
                _frames.setText("Frame " + currentFrame);
            }
        }
    }

    public void displayHistograms(HistogramRepresentation[] histograms) {
        for (int i = 0; i < histograms.length; i++) {
            for (int j = 0; j < _displayPanels.size(); j++) {
                if ((_displayPanels.get(j).tabIdx == histograms[i].tabIndex) &&
                        (_displayPanels.get(j).histidx == histograms[i].idx)) {
                    _displayPanels.get(j).pan.displayHistograms(histograms[i].histData);
                }
            }
        }
    }

    public void displayImages(ImageRepresentation[] images, double[] pixelData) {
        for (int i = 0; i < _allPanels.size(); i++) {
            if (_allPanels.get(i).getId() == _popDataIdx) {
                _painter.get(0).setTabIdx(_allPanels.get(i).getTabIdx());
                _painter.get(0).setDataIdx(_allPanels.get(i).getDataIdx());
                _painter.get(0).setChannel(_allPanels.get(i).getChannel());
            }
        }

        if (images.length > 0) {
            Vector<Integer> noUsedPaintersIdx = new Vector<Integer>();

            for (int i = 0; i < images.length; i++) {
                for (int j = 0; j < _painter.size(); j++) {
                    if ((_painter.get(j).getTabIdx() == images[i].tabIndex) &&
                            (_painter.get(j).getDataIdx() == images[i].idx) &&
                            (_painter.get(j).getChannel() == images[i].channel)) {
                        _painter.get(j)
                                .setData(images[i].data, images[i].bitDepth,
                            images[i].width, images[i].height, pixelData);
                        noUsedPaintersIdx.add(j);
                    }
                }
            }

            for (int i = 0; i < _painter.size(); i++) {
                if (!noUsedPaintersIdx.contains(i)) {
                    _painter.get(i).setData(null, 0, 0, 0, null);
                }
            }
        } else {
            for (int i = 0; i < _painter.size(); i++) {
                _painter.get(i).setData(null, 0, 0, 0, null);
            }
        }
    }

    public void displayLightLevel(int[] lightLevelData) {
        this._lightLevelPanel.displayLightLevel(lightLevelData);
    }
    
    public JFrame getTopFrame() {
        return _parent;
    }
    
    public Color getColor(String color) {
        if (color.equalsIgnoreCase("red")) {
            return Color.RED;
        } else if (color.equalsIgnoreCase("blue")) {
            return Color.BLUE;
        } else if (color.equalsIgnoreCase("green")) {
            return Color.GREEN;
        } else if (color.equalsIgnoreCase("black")) {
            return Color.BLACK;
        } else if (color.equalsIgnoreCase("orange")) {
            return Color.ORANGE;
        } else if (color.equalsIgnoreCase("yellow")) {
            return Color.YELLOW;
        } else if (color.equalsIgnoreCase("magenta")) {
            return Color.MAGENTA;
        } else {
            return null;
        }
    }

    public void displayChannelProps(PropertyRepresentation[] props) {
        IcamPropertyModel model = new IcamPropertyModel(props,
                (FrameSourceBase) _icamSource);
        _propPan.setModel(model, props);
    }

    public void setPopDataIdx(int popDataIdx) {
        this._popDataIdx = popDataIdx;
    }
}


class DisplayImagePanelTabs {
    protected ImageDisplayPanel pan;
    protected int tabIdx;
    protected int histidx;
}
