package com.nts.isip.tcp.isipgui;

import com.nts.isip.tcp.PropertyRepresentation;
import com.nts.isip.tcp.models.IcamPropertyModel;
import com.nts.isip.tcp.sources.FrameSource;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;

import java.rmi.RemoteException;

import java.text.ParseException;

import java.util.regex.Matcher;
import java.util.regex.Pattern;

import javax.swing.AbstractAction;
import javax.swing.AbstractCellEditor;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.DefaultCellEditor;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JDialog;
import javax.swing.JFormattedTextField;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.KeyStroke;
import javax.swing.table.DefaultTableModel;
import javax.swing.table.TableCellEditor;


public class IcamPropertyTable extends JTable {
    IcamPropertyModel _model;
    FrameSource _icamFrameSource;

    public IcamPropertyTable(IcamPropertyModel model,
        FrameSource icamFrameSource) {
        super(model);
        _model = model;
        _icamFrameSource = icamFrameSource;
    }
    
    public TableCellEditor getCellEditor(int row, int column) {
        if (column == 1) {
            final PropertyRepresentation prop = _model.getCellProperty(row,
                    column);

            if ((prop != null) && prop.type.equals("enumeration")) {
                String[] allowedValues = prop.allowedValues.split(",");
                JComboBox comboBox = new JComboBox(allowedValues);
                
                DefaultCellEditor dce = new DefaultCellEditor(comboBox);

                return (TableCellEditor) dce;
            } else if ((prop != null) && prop.type.equals("dynamiclist")) {
                String dependPropertyName = prop.allowedValues;

                try {
                    PropertyRepresentation dependProperty = _icamFrameSource.getPropertyDefinition(dependPropertyName);

                    if ((dependProperty != null) &&
                            (dependProperty.value != null)) {
                        String[] allowedValues = (dependProperty.value.toString()).split(
                                ",");
                        JComboBox comboBox = new JComboBox(allowedValues);
                        DefaultCellEditor dce = new DefaultCellEditor(comboBox);

                        return (TableCellEditor) dce;
                    } else {
                        return super.getCellEditor(row, column);
                    }
                } catch (RemoteException e) {
                    return super.getCellEditor(row, column);
                }
            } else if ((prop != null) && prop.type.contains("Array")) {
                ButtonCellEditor btnEdit = new ButtonCellEditor(prop);

                return btnEdit;
            } else if ((prop != null) &&
                    (prop.type.equalsIgnoreCase("float") ||
                    prop.type.equalsIgnoreCase("int") ||
                    prop.type.equalsIgnoreCase("String"))) {
                SingleTypeEditor dce = new SingleTypeEditor(prop);

                return dce;
            } else {
                return super.getCellEditor(row, column);
            }
        } else {
            return super.getCellEditor(row, column);
        }
    }
}

class ButtonCellEditor extends AbstractCellEditor implements TableCellEditor {
    JButton _button;
    JTable _table;
    DefaultTableModel _model;
    JDialog _dialog;
    String _value;

    public ButtonCellEditor(PropertyRepresentation prop) {
        if ((prop != null) && (prop.value != null)) {
            _value = prop.value.toString();
        }

        _dialog = new JDialog();
        _dialog.setSize(new Dimension(450, 300));
        _dialog.setLocation(250, 200);
        _dialog.setModal(true);
        _dialog.setDefaultCloseOperation(JDialog.DO_NOTHING_ON_CLOSE);
        _dialog.addWindowListener(new WindowListener() {
			public void windowActivated(WindowEvent e) {}
			public void windowClosed(WindowEvent e) {}
			public void windowClosing(WindowEvent e) {
				_dialog.dispose();
				cancelCellEditing();
			}
			public void windowDeactivated(WindowEvent e) {}
			public void windowDeiconified(WindowEvent e) {}
			public void windowIconified(WindowEvent e) {}
			public void windowOpened(WindowEvent e) {}
        });

        int columns = prop.arrayColumns;

        if (columns == 0) {
            columns = 1;
        }

        int rows;
        String[] values = null;

        if (prop.value != null) {
            String propVal = (String) prop.value;
            values = propVal.split(",");

            if ((values.length % columns) > 0) {
                rows = (values.length / columns) + 1;
            } else {
                rows = values.length / columns;
            }
        } else {
            rows = 0;
        }

        _model = new DefaultTableModel();
        _model.setColumnCount(columns);
        _model.setRowCount(rows);
        _table = new JTable(_model);

        if (values != null) {
            for (int i = 0; i < rows; i++) {
                for (int j = 0; j < columns; j++) {
                    int idx = (columns * i) + j;

                    if (idx < values.length) {
                        _model.setValueAt(values[idx], i, j);
                    } else {
                        break;
                    }
                }
            }
        }

        JPanel pan = new JPanel();
        pan.setLayout(new BorderLayout());
        pan.add(_table, BorderLayout.CENTER);

        JPanel btnPan = new JPanel();
        btnPan.setLayout(new BoxLayout(btnPan, BoxLayout.X_AXIS));
        btnPan.setSize(400, 50);
        btnPan.setPreferredSize(new Dimension(400, 50));

        JButton ok = new JButton("OK");
        ok.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    StringBuffer values = new StringBuffer();

                    //boolean breakAll = false;
                    for (int i = 0; i < _model.getRowCount(); i++) {
                        for (int j = 0; j < _model.getColumnCount(); j++) {
                            if ((_model.getValueAt(i, j) != null) &&
                                    !(_model.getValueAt(i, j)).equals("")) {
                                values.append(_model.getValueAt(i, j));
                                values.append(",");
                            } else {
                                values.append(0);
                                values.append(",");
                            }
                        }
                    }

                    if ((values != null) && (values.length() > 0)) {
                        values.deleteCharAt(values.length() - 1);
                        _value = values.toString();
                    }

                    _dialog.dispose();
                    fireEditingStopped();
                }
            });

        JButton insert = new JButton("Insert row");
        insert.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    _model.addRow(new Object[] {  });
                }
            });

        JButton delete = new JButton("Delete row");
        delete.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    if (_table.getSelectedRow() > -1) {
                        _model.removeRow(_table.getSelectedRow());
                    }
                }
            });
        
        JButton cancel = new JButton("Cancel");
        cancel.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				_dialog.dispose();
				cancelCellEditing();
			}
        });
        
        btnPan.add(Box.createHorizontalGlue());
        btnPan.add(ok);
        btnPan.add(Box.createRigidArea(new Dimension(10, 0)));
        btnPan.add(insert);
        btnPan.add(Box.createRigidArea(new Dimension(10, 0)));
        btnPan.add(delete);
        btnPan.add(Box.createRigidArea(new Dimension(10, 0)));
        btnPan.add(cancel);
        btnPan.add(Box.createHorizontalGlue());

        pan.add(btnPan, BorderLayout.SOUTH);

        JScrollPane scroll = new JScrollPane();
        scroll.getViewport().add(pan);
        scroll.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
        scroll.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED);
        _dialog.getContentPane().add(scroll);
    }

    public Object getCellEditorValue() {
        return _value;
    }

    public Component getTableCellEditorComponent(JTable table, Object value,
        boolean isSelected, int row, int column) {
        JButton button = new JButton((String) value);
        button.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent arg0) {
                    _dialog.setVisible(true);
                }
            });

        return button;
    }
}


class SingleTypeEditor extends DefaultCellEditor {
    JFormattedTextField ftf;
    PropertyRepresentation _prop;

    public SingleTypeEditor(PropertyRepresentation prop) {
        super(new JFormattedTextField());
        _prop = prop;
        ftf = (JFormattedTextField) getComponent();

        ftf.addKeyListener(new KeyListener() {
                public void keyPressed(KeyEvent e) {
                }

                public void keyReleased(KeyEvent e) {
                    if (!_prop.onTypingValidator.equals("")) {
                        Pattern p = Pattern.compile(_prop.onTypingValidator);
                        Matcher m = p.matcher(ftf.getText());

                        if (!m.matches()) {
                            Toolkit.getDefaultToolkit().beep();
                            if (ftf.getText().length() > 0) {
                                ftf.setText(ftf.getText()
                                               .substring(0,
                                        ftf.getText().length() - 1));
                            } else {
                                ftf.setText("");
                            }
                        }
                    }
                }

                public void keyTyped(KeyEvent e) {
                }
            });

        ftf.getInputMap()
           .put(KeyStroke.getKeyStroke(KeyEvent.VK_ENTER, 0), "check");
        ftf.getActionMap().put("check",
            new AbstractAction() {
                public void actionPerformed(ActionEvent e) {
                    if (!_prop.onSetValidator.equals("")) {
                        Pattern p = Pattern.compile(_prop.onSetValidator);
                        Matcher m = p.matcher(ftf.getText());

                        if (!m.matches()) {
                            Toolkit.getDefaultToolkit().beep();
                        } else {
                            try {
                                ftf.commitEdit();
                                ftf.postActionEvent(); //stop editing
                            } catch (java.text.ParseException exc) {
                            }
                        }
                    } else {
                        try {
                            ftf.commitEdit();
                            ftf.postActionEvent();
                        } catch (ParseException e1) {
                        }
                    }
                }
            });
    }

    public Component getTableCellEditorComponent(JTable table, Object value,
        boolean isSelected, int row, int column) {
        JFormattedTextField ftf = (JFormattedTextField) super.getTableCellEditorComponent(table,
                value, isSelected, row, column);
        ftf.setValue(_prop.value);

        return ftf;
    }

    public boolean stopCellEditing() {
        JFormattedTextField ftf = (JFormattedTextField) getComponent();

        if (!_prop.onSetValidator.equals("")) {
            Pattern p = Pattern.compile(_prop.onSetValidator);
            Matcher m = p.matcher(ftf.getText());

            if (!m.matches()) {
                try {
                    ftf.commitEdit();
                } catch (java.text.ParseException exc) {
                }
            } else { //text is invalid

                return false; //don't let the editor go away
            }
        }

        return super.stopCellEditing();
    }
}
