package com.nts.isip.tcp.isipgui;

import com.nts.isip.tcp.ClientView;
import com.nts.isip.tcp.StreamThread;

import org.jfree.chart.ChartFactory;
import org.jfree.chart.ChartPanel;
import org.jfree.chart.JFreeChart;
import org.jfree.chart.plot.CategoryPlot;
import org.jfree.chart.plot.PlotOrientation;

import org.jfree.data.category.DefaultCategoryDataset;

import java.awt.Component;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Image;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JPanel;


public class ImageDisplayPanel extends JPanel {
    private static int maxHistChartLenght = 50;
    ImagePanel _calc;
    JFreeChart _histChart;
    DefaultCategoryDataset _histData;
    ChartPanel _histChartPan;
    StreamThread _strThread;
    PopupImageDialog _popupImgDialog;

    //ClientFrame _parent;
    ClientView _parent;
    Component _dependVisibleParent;
    int _tabIdx;
    int _dataIdx;
    int _panelIdx;
    String _channel;

    public ImageDisplayPanel(String name, StreamThread strThread) {
        _strThread = strThread;
        this.setBorder(BorderFactory.createTitledBorder(name));
        init();
    }

    public ImageDisplayPanel(String name, StreamThread strThread,
        PopupImageDialog popupImgDialog, int tabIdx, int dataIdx,
        ClientView topFrame) {
        _strThread = strThread;
        this.setBorder(BorderFactory.createTitledBorder(name));
        _popupImgDialog = popupImgDialog;
        _tabIdx = tabIdx;
        _dataIdx = dataIdx;
        _parent = topFrame;
        _dependVisibleParent = (Component) topFrame;
        init();
    }

    private void init() {
        _calc = new ImagePanel(_strThread);
        _calc.setPreferredSize(new Dimension(300, 180));
        _calc.setMaximumSize(new Dimension(300, 180));
        _calc.setMinimumSize(new Dimension(300, 180));

        _histData = new DefaultCategoryDataset();
        _histChart = ChartFactory.createBarChart(null, null, null, _histData,
                PlotOrientation.VERTICAL, false, true, true);

        CategoryPlot catPlot = (CategoryPlot) _histChart.getPlot();
        catPlot.getDomainAxis().setVisible(false);
        catPlot.getRangeAxis().setVisible(false);
        _histChartPan = new ChartPanel(_histChart);
        _histChartPan.setPreferredSize(new Dimension(200, 40));
        _histChartPan.setMaximumSize(new Dimension(200, 40));
        _histChartPan.setMinimumSize(new Dimension(200, 40));

        JPanel histBtnPan = new JPanel();
        histBtnPan.setLayout(new BoxLayout(histBtnPan, BoxLayout.X_AXIS));
        histBtnPan.add(_histChartPan);

        JButton btn = new JButton("Zoom");
        histBtnPan.add(Box.createRigidArea(new Dimension(10, 0)));
        histBtnPan.add(btn);
        btn.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    if (_popupImgDialog != null) {
                        _parent.setPopDataIdx(_panelIdx);

                        if (!_popupImgDialog.isVisible()) {
                            _popupImgDialog.setVisible(true);
                        }
                    }
                }
            });

        GridBagLayout layout = new GridBagLayout();
        this.setLayout(layout);

        GridBagConstraints c = new GridBagConstraints();
        c.gridwidth = 1;
        c.gridheight = 5;
        c.fill = GridBagConstraints.BOTH;
        c.weighty = 1;
        c.gridx = 0;
        c.gridy = 0;
        layout.setConstraints(_calc, c);
        this.add(_calc);

        c.gridwidth = 1;
        c.gridx = 0;
        c.gridy = 5;
        c.anchor = GridBagConstraints.EAST;
        c.fill = GridBagConstraints.NONE;
        c.weighty = 0;
        layout.setConstraints(histBtnPan, c);
        this.add(histBtnPan);
    }

    public void setImage(Image img) {
        _calc.setImage(img);
    }

    //public void setPixelProfile(double[] pixelProfile) {
    //	_calc.setPixelProfile(pixelProfile);
    //}
    public void setDrawer(ImageDrawer drawer) {
        _calc.setDrawer(drawer);
    }

    public void setRescale(boolean rescale) {
        _calc.setRescale(rescale);
    }

    public void displayHistograms(int[] histogram) {
        this._histChart.setNotify(false);
        updateHistData(this._histData, histogram);
        this._histChart.setNotify(true);
    }

    private static void updateHistData(DefaultCategoryDataset data, int[] hist) {
        data.clear();

        int pointsCnt = hist.length / 2;
        int step = pointsCnt / maxHistChartLenght;

        if (step == 0) {
            step = 1;
        }

        for (int i = 0; i < pointsCnt; i += step) {
            int avgx = 0;
            int avgy = 0;
            int cnt = 0;

            for (int j = i; (j < (i + step)) && (j < pointsCnt); j++) {
                avgx = avgx + hist[2 * j];
                avgy = avgy + hist[(2 * j) + 1];
                cnt++;
            }

            avgy = avgy / cnt;
            avgx = avgx / cnt;
            data.setValue(avgx, "c", String.valueOf(avgx));
            data.setValue(avgy, "c", String.valueOf(avgy));
        }
    }

    public ChartPanel getHistChartPanel() {
        return _histChartPan;
    }

    public void setHistChartPanel(ChartPanel histChartPanel) {
        _histChartPan = histChartPanel;
    }

    public ImagePanel getImagePanel() {
        return this._calc;
    }

    public int getTabIdx() {
        return this._tabIdx;
    }

    public int getDataIdx() {
        return this._dataIdx;
    }

    public void setId(int id) {
        _panelIdx = id;
    }

    public String getChannel() {
        return _channel;
    }

    public void setChannel(String channel) {
        _channel = channel;
    }

    public int getId() {
        return this._panelIdx;
    }

    public void setDependVisibleParent(Component parent) {
        _dependVisibleParent = parent;
    }

    public Component getDependVisibleParent() {
        return _dependVisibleParent;
    }
}
