package com.nts.isip.tcp.isipgui;

import com.nts.isip.tcp.PropertyRepresentation;
import com.nts.isip.tcp.StreamThread;
import com.nts.isip.tcp.exceptions.NoSuchPropertyException;
import com.nts.isip.tcp.listeners.StreamStateListener;
import com.nts.isip.tcp.sources.FrameSourceBase;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Image;
import java.awt.Point;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.awt.geom.Point2D;

import java.io.IOException;

import java.rmi.RemoteException;

import java.util.Properties;
import java.util.Vector;

import javax.swing.JPanel;


/**
 * Canvas subclass for dynamically displaying images
 * Used to display frame images. The setImage stores the passed image
 * and causes the component to be repainted with it. An ImageDrawer
 * instance can be set to draw additional graphics on top of the image,
 * using the setDrawer method.
 */
public class ImagePanel extends JPanel implements StreamStateListener,
    MouseListener, MouseMotionListener {
    private Image _img = null;
    private ImageDrawer _drawer = null;
    boolean _rescale = false;
    private Point _start = null;
    private Point _end = null;
    private int _vSubSamp = 0;
    private int _hSubSamp = 0;
    private int _hScanDir = 1;
    private int _vScanDir = 1;
    private StreamThread _strThread;

    //private EnumeratingPropertyModel _model;
    private FrameSourceBase _model;
    private Vector<Point> _template;
    private boolean _drawTemplate;
    private Point _firstTmp;
    private Point _secondTmp;
    private float _lineTolerance = 10;
    private float _angleTolerance = 5;
    private float[] _transformCoefs;
    private float _minLineLength = 25;
    private int _startx;
    private int _starty;

    //private double[] _pixelProfile;
    public ImagePanel(StreamThread strThread) {
        _strThread = strThread;
        _strThread.addStateListener(this);
        _model = _strThread.getStreamSource();
        _drawTemplate = false;
        this.addMouseListener(this);
        this.addMouseMotionListener(this);
    }

    public void setImage(Image img) {
        _img = img;
        this.repaint();
    }

    //public void setPixelProfile(double[] pixelProfile) {
    //	_pixelProfile = pixelProfile;
    //}
    public Dimension scaledImg() {
        float coef = (float) this.getWidth() / _img.getWidth(null);
        int width = (int) (_img.getWidth(null) * coef);
        int height = (int) (_img.getHeight(null) * coef);

        if (height > this.getHeight()) {
            coef = (float) this.getHeight() / _img.getHeight(null);
            width = (int) (_img.getWidth(null) * coef);
            height = (int) (_img.getHeight(null) * coef);
        }

        Dimension imgSize = new Dimension(width, height);

        return imgSize;
    }

    public void scaledStartPoint() {
        if ((_img != null) && (_start != null)) {
            float coefX = (float) (_img.getWidth(null) / scaledImg().getWidth());
            float coefY = (float) (_img.getHeight(null) / scaledImg().getHeight());
            _start.x = (int) (_start.x * coefX);
            _start.y = (int) (_start.y * coefY);
        } else {
        }
    }

    public void scaledEndPoint() {
        if ((_end != null) && (_img != null)) {
            float coefX = (float) (_img.getWidth(null) / scaledImg().getWidth());
            float coefY = (float) (_img.getHeight(null) / scaledImg().getHeight());
            _end.x = (int) (_end.x * coefX);
            _end.y = (int) (_end.y * coefY);
        } else {
        }
    }

    public Point scaledPointToNormal(Point p) {
        if (p != null) {
            Point scaledP = new Point();
            float coefX = (float) (_img.getWidth(null) / scaledImg().getWidth());
            float coefY = (float) (_img.getHeight(null) / scaledImg().getHeight());
            scaledP.x = (int) (p.x * coefX);
            scaledP.y = (int) (p.y * coefY);

            return scaledP;
        } else {
            return null;
        }
    }

    public Point normalPointToScaled(Point p) {
        if (p != null) {
            Point normalP = new Point();
            float coefX = (float) (scaledImg().getWidth() / _img.getWidth(null));
            float coefY = (float) (scaledImg().getHeight() / _img.getHeight(null));
            normalP.x = (int) (p.x * coefX);
            normalP.y = (int) (p.y * coefY);

            return normalP;
        } else {
            return null;
        }
    }

    public void paint(Graphics g) {
        if (_img != null) {
            float imgScaledWidth = 1;
            float imgScaledHeight = 1;

            if (_rescale) {
                Dimension imgSize = scaledImg();
                imgScaledWidth = (float) imgSize.getWidth() / _img.getWidth(null);
                imgScaledHeight = (float) imgSize.getHeight() / _img.getHeight(null);
                super.paint(g);
                g.drawImage(_img, 0, 0, imgSize.width, imgSize.height, this);

                /*if(_pixelProfile != null) {
                        for(int i = 0; i < _pixelProfile.length; i += 2) {
                                g.setColor(Color.BLUE);
                                Point p = normalPointToScaled(new Point((int)_pixelProfile[i], (int)_pixelProfile[i+1]));
                                g.drawLine((int)p.getX(), (int)p.getY(), (int)p.getX(), (int)p.getY());
                        }
                }*/
            } else {
                imgScaledWidth = 1;
                imgScaledHeight = 1;
                super.paint(g);
                g.drawImage(_img, 0, 0, this);

                /*if(_pixelProfile != null) {
                        for(int i = 0; i < _pixelProfile.length; i += 2) {
                                g.setColor(Color.BLUE);
                                g.drawLine((int)_pixelProfile[i], (int)_pixelProfile[i + 1], (int)_pixelProfile[i], (int)_pixelProfile[i + 1]);
                        }
                }*/
            }

            if (_drawer != null) {
                _drawer.addDrawing(g, imgScaledWidth, imgScaledHeight);
            }

            if ((_start != null) && (_end != null)) {
                addRectangle(g);
            }

            if (_template != null) {
                for (int i = 0; i < (_template.size() - 1); i++) {
                    if (_rescale) {
                        addLine(g, normalPointToScaled(_template.get(i)),
                            normalPointToScaled(_template.get(i + 1)));
                    } else {
                        addLine(g, _template.get(i), _template.get(i + 1));
                    }
                }
            }
        } else {
            super.paint(g);
        }
    }

    /**
     * Stores the passed ImageDrawer instance and invokes its addDrawing
     * method when painting this component, to draw additional graphics
     * on top of the image displayed.
     */
    public void setDrawer(ImageDrawer drawer) {
        _drawer = drawer;
    }

    public void setRescale(boolean rescale) {
        _rescale = rescale;
        this.repaint();
    }

    public void mousePressed(MouseEvent e) {
        _start = e.getPoint();
        _end = _start;
    }

    public void mouseReleased(MouseEvent e) {
        _end = e.getPoint();

        if (_rescale) {
            scaledStartPoint();
            scaledEndPoint();
        }

        /*try {
            _hScanDir = Integer.valueOf(_model.getProperty("HScanDir").toString())
                               .intValue();
        } catch (NoSuchPropertyException ex) {
        } catch (NullPointerException ex) {
        }
        
        try {
            _vScanDir = Integer.valueOf(_model.getProperty("VScanDir").toString())
                               .intValue();
        } catch (NoSuchPropertyException ex) {
        } catch (NullPointerException ex) {
        }*/
        if (_img != null) {
            if ((_hScanDir == 0) && (_vScanDir == 0)) {
                int startX = _img.getWidth(null) - _end.x;
                int startY = _img.getHeight(null) - _end.y;
                int endX = _img.getWidth(null) - _start.x;
                int endY = _img.getHeight(null) - _start.y;
                _start = new Point(startX, startY);
                _end = new Point(endX, endY);
            }

            if ((_hScanDir == 0) && (_vScanDir == 1)) {
                int startX = _img.getWidth(null) - _end.x;
                int startY = _start.y;
                int endX = _img.getWidth(null) - _start.x;
                int endY = _end.y;
                _start = new Point(startX, startY);
                _end = new Point(endX, endY);
            }

            if ((_hScanDir == 1) && (_vScanDir == 0)) {
                int startX = _start.x;
                int startY = _img.getHeight(null) - _end.y;
                int endX = _end.x;
                int endY = _img.getHeight(null) - _start.y;
                _start = new Point(startX, startY);
                _end = new Point(endX, endY);
            }
        }

        /*try {
            _vSubSamp = Integer.valueOf(_model.getProperty("VSubSamp").toString())
                               .intValue();
        } catch (NoSuchPropertyException ex) {
        } catch (NullPointerException ex) {
        }
        
        try {
            _hSubSamp = Integer.valueOf(_model.getProperty("HSubSamp").toString())
                               .intValue();
        } catch (NoSuchPropertyException ex) {
        } catch (NullPointerException ex) {
        }*/
        _start.x = _start.x * (_hSubSamp + 1);
        _start.y = _start.y * (_vSubSamp + 1);
        _end.x = _end.x * (_hSubSamp + 1);
        _end.y = _end.y * (_vSubSamp + 1);

        if ((_end.x < _start.x) || (_end.y < _start.y)) {
            if (_model != null) {
                //setProperty("WinStartRow", (Integer) 4);
                //setProperty("WinEndRow", (Integer) 1025);
                //setProperty("WinStartCol", (Integer) 28);
                //setProperty("WinEndCol", (Integer) 1211);
            }
        } else {
            if (((_end.y - _start.y) > 64) && ((_end.x - _start.x) > 132)) {
                //setProperty("WinStartRow", (Integer) _start.y);
                //setProperty("WinEndRow", (Integer) _end.y);
                //setProperty("WinStartCol", (Integer) _start.x);
                //setProperty("WinEndCol", (Integer) _end.x);
            }
        }

        _start = null;
        _end = null;
        this.repaint();
    }

    public void mouseDragged(MouseEvent e) {
        _end = e.getPoint();
        addRectangle(this.getGraphics());
        this.repaint();
    }

    public void addRectangle(Graphics g) {
        g.setColor(Color.RED);
        g.drawRect(_start.x, _start.y, _end.x - _start.x, _end.y - _start.y);
    }

    public void mouseMoved(MouseEvent e) {
        if (_drawTemplate) {
            Point currEnd = e.getPoint();

            if (_template.size() >= 1) {
                Point currFirst = _template.get(_template.size() - 1);

                if ((currEnd.x < _img.getWidth(null)) &&
                        (currEnd.y < _img.getHeight(null))) {
                    if (_rescale) {
                        addLine(this.getGraphics(),
                            normalPointToScaled(currFirst), currEnd);
                    } else {
                        addLine(this.getGraphics(), currFirst, currEnd);
                    }

                    this.repaint();
                }
            }
        }
    }

    public void mouseClicked(MouseEvent e) {
        if (_drawTemplate && (_img != null)) {
            _secondTmp = e.getPoint();

            Dimension imgSize;

            if (_rescale) {
                imgSize = scaledImg();
            } else {
                imgSize = new Dimension(_img.getWidth(null),
                        _img.getHeight(null));
            }

            if ((_secondTmp.x < imgSize.getWidth()) &&
                    (_secondTmp.y < imgSize.getHeight())) {
                if (_rescale) {
                    _template.add(scaledPointToNormal(_secondTmp));
                } else {
                    _template.add(_secondTmp);
                }

                addLine(this.getGraphics(), _firstTmp, _secondTmp);
                _firstTmp = _secondTmp;
            }
        }
    }

    public void addLine(Graphics g, Point start, Point end) {
        if ((start != null) && (end != null)) {
            g.setColor(Color.RED);
            g.drawLine(start.x, start.y, end.x, end.y);
        }
    }

    public void mouseEntered(MouseEvent e) {
    }

    public void mouseExited(MouseEvent e) {
    }

    public void setPoints(Point start, Point end) {
        _start = start;
        _end = end;
    }

    public void setDrawTemplate(boolean drawTemplate) {
        _drawTemplate = drawTemplate;

        if (drawTemplate) {
            _template = new Vector<Point>();
        } else {
            //if(_model instanceof IcamFrameSource) {	
            Properties prop = new Properties();

            try {
                java.io.FileInputStream in = new java.io.FileInputStream(
                        "isip_client_chart.properties");

                try {
                    prop.load(in);
                } finally {
                    in.close();
                }
            } catch (IOException e) {
            }

            String lineToleranceProp = prop.getProperty("linetolerance");
            String angleToleranceProp = prop.getProperty("angletolerance");
            String minLineLengthProp = prop.getProperty("minlinelength");
            String transformCoefsProp = prop.getProperty("transformCoefs");
            String windowStartColProp = prop.getProperty("windowstartcol");
            String windowStartRowProp = prop.getProperty("windowstartrow");

            try {
                PropertyRepresentation lineTolerance = (_model).getPropertyDefinition(lineToleranceProp);
                _lineTolerance = Float.valueOf((_model).getProperty(
                            lineTolerance.path, lineTolerance.type, false)
                                                .toString()).floatValue();
            } catch (RemoteException e) {
            } catch (NoSuchPropertyException ex) {
            } catch (NullPointerException ex) {
            }

            try {
                PropertyRepresentation angleTolerance = (_model).getPropertyDefinition(angleToleranceProp);
                _angleTolerance = Float.valueOf((_model).getProperty(
                            angleTolerance.path, angleTolerance.type, false)
                                                 .toString()).floatValue();
            } catch (RemoteException e) {
            } catch (NoSuchPropertyException ex) {
            } catch (NullPointerException ex) {
            }

            try {
                PropertyRepresentation minLineLength = (_model).getPropertyDefinition(minLineLengthProp);
                _minLineLength = Float.valueOf((_model).getProperty(
                            minLineLength.path, minLineLength.type, false)
                                                .toString()).floatValue();
            } catch (RemoteException e) {
            } catch (NoSuchPropertyException ex) {
            } catch (NullPointerException ex) {
            }

            try {
                PropertyRepresentation windowStartCol = (_model).getPropertyDefinition(windowStartColProp);
                _startx = Integer.valueOf((_model).getProperty(
                            windowStartCol.path, windowStartCol.type, false)
                                           .toString()).intValue();
            } catch (RemoteException e) {
            } catch (NoSuchPropertyException ex) {
            } catch (NullPointerException ex) {
            }

            try {
                PropertyRepresentation windowStartRow = (_model).getPropertyDefinition(windowStartRowProp);
                _starty = Integer.valueOf((_model).getProperty(
                            windowStartRow.path, windowStartRow.type, false)
                                           .toString()).intValue();
            } catch (RemoteException e) {
            } catch (NoSuchPropertyException ex) {
            } catch (NullPointerException ex) {
            }

            try {
                PropertyRepresentation transformCoefs = (_model).getPropertyDefinition(transformCoefsProp);
                String strCoefs = (String) ((_model).getProperty(transformCoefs.path,
                        transformCoefs.type, false));
                String[] coefsParts = strCoefs.split(",");
                _transformCoefs = new float[coefsParts.length];

                for (int i = 0; i < coefsParts.length; i++) {
                    _transformCoefs[i] = Float.valueOf(coefsParts[i])
                                              .floatValue();
                }
            } catch (RemoteException e) {
            } catch (NoSuchPropertyException ex) {
            } catch (NullPointerException ex) {
            }

            /*} else {
                    try {
                            _lineTolerance = Double.valueOf(_model.getProperty(
                                                    "template_generator_line_tol").toString())
                                                   .doubleValue();
                    } catch (NoSuchPropertyException ex) {
                    } catch (NullPointerException ex) {
                    }
            
                    try {
                            _angleTolerance = Double.valueOf(_model.getProperty(
                                "template_generator_ang_tol").toString())
                                            .doubleValue();
                    } catch (NoSuchPropertyException ex) {
                    } catch (NullPointerException ex) {
                    }
            
                    try {
                            _minLineLength = Double.valueOf(_model.getProperty(
                                "min_line_length").toString()).doubleValue();
                    } catch (NoSuchPropertyException ex) {
                    } catch (NullPointerException ex) {
                    }
            
                    try {
                            String projectMatrix = _model.getProperty(
                            "coord_transform_cofficients").toString();
                            String[] projectMatrixValues = projectMatrix.split("\\|");
                            _transformCoefs = new double[projectMatrixValues.length];
            
                            for (int i = 0; i != _transformCoefs.length; ++i) {
                                    try {
                                            _transformCoefs[i] = Double.valueOf(projectMatrixValues[i].trim());
                                    } catch (NumberFormatException e) {
                                            e.printStackTrace();
                                            _transformCoefs[i] = 0.;
                                    }
                            }
                    } catch (NoSuchPropertyException ex) {
                    } catch (NullPointerException ex) {
                    }
            
            
                    try {
                            _startx = Integer.parseInt(_model.getProperty("WinStartCol")
                                                     .toString());
                    } catch (NoSuchPropertyException e) {
                    } catch (NullPointerException ex) {
                    }
            
                    try {
                            _starty = Integer.parseInt(_model.getProperty("WinStartRow")
                                                     .toString());
                    } catch (NoSuchPropertyException e) {
                    } catch (NullPointerException ex) {
                    }
            
            }*/
            StringBuffer tmpStr = new StringBuffer();
            StringBuffer linesLength = new StringBuffer();
            StringBuffer linesMinMaxLengths = new StringBuffer();

            if ((_template != null) && (_template.size() > 1)) {
                for (int i = 0; i < (_template.size() - 1); i++) {
                    Point pPx1 = _template.get(i);
                    Point pPx2 = _template.get(i + 1);
                    Point2D.Double pt = transformPoint(pPx1);
                    double x1_mm = pt.x;
                    double y1_mm = pt.y;
                    pt = transformPoint(pPx2);

                    double x2_mm = pt.x;
                    double y2_mm = pt.y;
                    //tmpStr.append(x1_mm + "|" + y1_mm + "|");
                    tmpStr.append(x1_mm + "," + y1_mm + ",");

                    double length = lineLength(x1_mm, y1_mm, x2_mm, y2_mm);
                    //linesLength.append(length + "|");
                    linesLength.append(length + ",");

                    if ((i == 0) || (i == (_template.size() - 2))) {
                        //linesMinMaxLengths.append(_minLineLength + "|");
                        linesMinMaxLengths.append(_minLineLength + ",");
                    } else {
                        double minLength = length - _lineTolerance;

                        if (minLength < 0.0) {
                            //linesMinMaxLengths.append("0|");
                            linesMinMaxLengths.append("0,");
                        } else {
                            //linesMinMaxLengths.append(minLength + "|");
                            linesMinMaxLengths.append(minLength + ",");
                        }
                    }

                    if ((i == 0) || (i == (_template.size() - 2))) {
                        //linesMinMaxLengths.append("-1|");
                        linesMinMaxLengths.append("-1,");
                    } else {
                        //linesMinMaxLengths.append(length + _lineTolerance +
                        //    "|");
                        linesMinMaxLengths.append(length + _lineTolerance +
                            ",");
                    }
                }

                Point pPxLast = _template.get(_template.size() - 1);
                Point2D.Double pt = transformPoint(pPxLast);
                double xLast_mm = pt.x;
                double yLAst_mm = pt.y;
                //tmpStr.append(xLast_mm + "|" + yLAst_mm);
                tmpStr.append(xLast_mm + "," + yLAst_mm);
                linesLength.deleteCharAt(linesLength.length() - 1);
                setProperty("current_template", tmpStr.toString());
                System.out.println("current_template_lineranges=" +
                    linesMinMaxLengths.toString());
                setProperty("current_template_lineranges",
                    linesMinMaxLengths.toString());

                StringBuffer angles = new StringBuffer();

                if (_template.size() > 2) {
                    for (int i = 0; i < (_template.size() - 2); i++) {
                        Point pPx1 = _template.get(i);
                        Point pPx2 = _template.get(i + 1);
                        Point pPx3 = _template.get(i + 2);

                        pt = transformPoint(pPx1);

                        double x1_mm = pt.x;
                        double y1_mm = pt.y;

                        pt = transformPoint(pPx2);

                        double x2_mm = pt.x;
                        double y2_mm = pt.y;

                        pt = transformPoint(pPx3);

                        double x3_mm = pt.x;
                        double y3_mm = pt.y;

                        double lineA = lineLength(x1_mm, y1_mm, x2_mm, y2_mm);
                        double lineB = lineLength(x2_mm, y2_mm, x3_mm, y3_mm);
                        double anglecos = (((x2_mm - x1_mm) * (x3_mm - x2_mm)) +
                            ((y2_mm - y1_mm) * (y3_mm - y2_mm))) / (lineA * lineB);
                        double sign = Math.signum((y1_mm - y2_mm - y2_mm +
                                y3_mm));
                        double angle = sign * (180 / Math.PI * Math.acos(anglecos));
                        //angles.append(Math.round(angle) - _angleTolerance +
                        //    "|");
                        angles.append(Math.round(angle) - _angleTolerance +
                            ",");
                        //angles.append(Math.round(angle) + _angleTolerance +
                        //    "|");
                        angles.append(Math.round(angle) + _angleTolerance +
                            ",");

                        double lineAuntr = lineLength(pPx1.x, pPx1.y, pPx2.x,
                                pPx2.y);
                        double lineBuntr = lineLength(pPx2.x, pPx2.y, pPx3.x,
                                pPx3.y);
                        double anglecosuntr = (((pPx2.x - pPx1.x) * (pPx3.x -
                            pPx2.x)) + ((pPx2.y - pPx1.y) * (pPx3.y - pPx2.y))) / (lineAuntr * lineBuntr);

                        double angleuntr = sign * (180 / Math.PI * Math.acos(anglecosuntr));
                        System.out.println("Angle1=" + Math.round(angle) +
                            " , Angle2=" + Math.round(angleuntr));
                    }

                    angles.deleteCharAt(angles.length() - 1);
                    setProperty("current_template_angleranges",
                        angles.toString());

                    /*if (_model instanceof IsipFrameSource) {
                        try {
                            ((IsipFrameSource) _model).applyProfilePropertie();
                        } catch (IOException e) {
                            e.printStackTrace();
                        } catch (ProtocolError e) {
                            e.printStackTrace();
                        } catch (ServerError e) {
                            e.printStackTrace();
                        }
                    }*/
                }
            }
        }
    }

    public void streamStateChanged(boolean active) {
        _model = _strThread.getStreamSource();
    }

    public void setProperty(String propName, Object val) {
        try {
            //if(_model instanceof IcamFrameSource) {
            Properties properties = new Properties();

            try {
                java.io.FileInputStream in = new java.io.FileInputStream(
                        "isip_client_chart.properties");

                try {
                    properties.load(in);
                } finally {
                    in.close();
                }
            } catch (IOException e) {
            }

            String propPath = properties.getProperty(propName);
            PropertyRepresentation prop = (_model).getPropertyDefinition(propPath);
            (_model).setProperty(prop, val);

            //} else {
            //	_model.setProperty(propName, val.toString());
            //}
        } catch (NoSuchPropertyException ex) {
        } catch (Exception ex) {
        }
    }

    public double lineLength(double startX, double startY, double endX,
        double endY) {
        double deltaX = (endX - startX) * (endX - startX);
        double deltaY = (endY - startY) * (endY - startY);
        double length = Math.sqrt(deltaX + deltaY);
        double power_of_ten = 100.0;

        return Math.round(length * power_of_ten) / power_of_ten;
    }

    public double findAngle(double a, double b, double c) {
        double cosAngle = (((a * a) + (b * b)) - (c * c)) / (2 * a * b);
        double angleRad = Math.acos(cosAngle);
        double angle = (angleRad * 180) / Math.PI;
        double power_of_ten = 100.0;

        return Math.round(angle * power_of_ten) / power_of_ten;
    }

    private Point2D.Double transformPoint(Point p) {
        int x = p.x + _startx;
        int y = p.y + _starty;
        double tx = (_transformCoefs[0] * x * x * x) +
            (_transformCoefs[1] * x * x * y) +
            (_transformCoefs[2] * x * y * y) +
            (_transformCoefs[3] * y * y * y) + (_transformCoefs[4] * x * x) +
            (_transformCoefs[5] * x * y) + (_transformCoefs[6] * y * y) +
            (_transformCoefs[7] * x) + (_transformCoefs[8] * y) +
            _transformCoefs[9];
        double ty = (_transformCoefs[10] * x * x * x) +
            (_transformCoefs[11] * x * x * y) +
            (_transformCoefs[12] * x * y * y) +
            (_transformCoefs[13] * y * y * y) + (_transformCoefs[14] * x * x) +
            (_transformCoefs[15] * x * y) + (_transformCoefs[16] * y * y) +
            (_transformCoefs[17] * x) + (_transformCoefs[18] * y) +
            _transformCoefs[19];
        System.out.println("[" + p.x + "," + p.y + "]->[" + tx + "," + ty +
            "]");

        return new Point2D.Double(tx, ty);
    }
}
