package com.nts.isip.tcp.isipgui;

import com.nts.isip.tcp.PropertyUtil;
import com.nts.isip.tcp.StreamThread;
import com.nts.isip.tcp.exceptions.NoSuchPropertyException;

import org.jfree.chart.ChartFactory;
import org.jfree.chart.ChartPanel;
import org.jfree.chart.JFreeChart;
import org.jfree.chart.labels.StandardXYToolTipGenerator;
import org.jfree.chart.plot.PlotOrientation;
import org.jfree.chart.plot.XYPlot;
import org.jfree.chart.renderer.xy.XYLineAndShapeRenderer;

import org.jfree.data.xy.XYSeries;
import org.jfree.data.xy.XYSeriesCollection;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Point;

import java.util.Arrays;
import java.util.Comparator;

import javax.swing.JPanel;


public class LightLevelDisplayPanel extends JPanel {
    private static int maxHistChartLenght = 50;
    JFreeChart _histChart;
    XYSeries _histData;
    XYSeries _histDataMax;
    XYSeries _histDataMin;
    XYSeries _histDataavg;
    StreamThread _strThread;
    XYSeriesCollection _serCol;
    ChartPanel _histChartPan;
    XYPlot _plot;

    public LightLevelDisplayPanel(final StreamThread strThread) {
        this.setLayout(new BorderLayout());
        _serCol = new XYSeriesCollection();
        _histDataMax = new XYSeries("Max");
        _histDataavg = new XYSeries("avg");
        _histData = new XYSeries("subs");
        _histDataMin = new XYSeries("Min");
        _serCol.addSeries(_histDataMax);
        _serCol.addSeries(_histDataMin);
        _serCol.addSeries(_histDataavg);

        _histChart = ChartFactory.createScatterPlot("Laser Intensity",
                "X Location", "Intensity", _serCol, PlotOrientation.VERTICAL,
                true, true, true);
        _histChartPan = new ChartPanel(_histChart);
        _histChartPan.setLayout(new BorderLayout());

        _strThread = strThread;

        _plot = (XYPlot) _histChart.getPlot();

        XYLineAndShapeRenderer renderer = (XYLineAndShapeRenderer) _plot.getRenderer();
        renderer.setSeriesLinesVisible(0, true);
        renderer.setSeriesLinesVisible(1, true);
        renderer.setSeriesLinesVisible(2, true);
        renderer.setSeriesLinesVisible(3, true);
        renderer.setSeriesPaint(0, Color.red);
        renderer.setSeriesPaint(1, Color.red);
        renderer.setSeriesPaint(2, Color.blue);
        renderer.setSeriesPaint(3, Color.black);
        renderer.setToolTipGenerator(new StandardXYToolTipGenerator());
        renderer.setShape(new java.awt.Rectangle.Double(-0.1, -0.1, 0.1, 0.1));

        _plot.setRenderer(renderer);
        _histChartPan.setDisplayToolTips(false);
        this.add(_histChartPan, BorderLayout.CENTER);

        //JPanel btnPanel = new JPanel();
        /*JButton laserOnBtn = new JButton("Laser ON");
        laserOnBtn.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    if (strThread.getStreamSource() instanceof IsipFrameSource) {
                        ((IsipFrameSource) strThread.getStreamSource()).setProperty("LaserOnOff",
                            "1");
                    }
                }
            });
        btnPanel.add(laserOnBtn);*/

        //final JTextField itxt = new JTextField(5);

        /*if (strThread.getStreamSource() instanceof IsipFrameSource) {
            itxt.setText(strThread.getStreamSource()
                                  .getProperty("IntegrationTime").toString());
        }*/

        /*itxt.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    if (strThread.getStreamSource() instanceof IsipFrameSource) {
                        ((IsipFrameSource) strThread.getStreamSource()).setProperty("IntegrationTime",
                            itxt.getText());
                    }
                }
            });
        btnPanel.add(new JLabel("Integration Time:"));
        btnPanel.add(itxt);
        this.add(btnPanel, BorderLayout.SOUTH);*/
    }

    public void displayLightLevel(int[] histogram) {
        this._histChart.setNotify(false);
        updateHistData(this._histData, this._histDataMax, this._histDataavg,
            this._histDataMin, histogram, this._strThread, this._histChartPan);
        this._histChart.setNotify(true);
    }

    private static void updateHistData(XYSeries data, XYSeries dataMax,
        XYSeries dataavg, XYSeries dataMin, int[] hist, StreamThread strThread,
        ChartPanel pan) {
        data.clear();
        dataMax.clear();
        dataMin.clear();
        dataavg.clear();

        int dec = 1;
        int ptsCnt = (hist.length / 2) / dec;
        Point[] pts = new Point[ptsCnt];

        for (int i = 0; i != ptsCnt; ++i) {
            pts[i] = new Point(hist[i * dec * 2], hist[(i * dec * 2) + 1]);
        }

        Arrays.sort(pts,
            new Comparator<Point>() {
                public int compare(Point p1, Point p2) {
                    return p1.x - p2.x;
                }
            });

        try {
            /*int minY = Integer.valueOf(strThread.getStreamSource()
                                                .getProperty("min_target_intensity")
                                                .toString()).intValue();
            int maxY = Integer.valueOf(strThread.getStreamSource()
                                                .getProperty("max_target_intensity")
                                                .toString()).intValue();*/

            //int startx = hist[0];
            //int endx = hist[hist.length - 2];

            /*for (int i = startx; i < endx; i++) {
                dataMax.add(i, maxY);
                dataMin.add(i, minY);
            }*/
        } catch (NoSuchPropertyException e) {
        } catch (NullPointerException e) {
        } catch (RuntimeException e) {
        }

        try {
            int step = ptsCnt / maxHistChartLenght;

            if (step == 0) {
                step = 2;
            }

            for (int i = 0; i < ptsCnt; i += step) {
                float maxVal = pts[i].y;

                float minVal = pts[i].y;
                float avgInt = 0;
                float avgx = 0;
                int cnt = 0;

                for (int j = i; (j < (i + step)) && (j < ptsCnt); ++j) {
                    if (pts[j].y > maxVal) {
                        maxVal = pts[j].y;
                    }

                    if (pts[j].y < minVal) {
                        minVal = pts[j].y;
                    }

                    avgInt = avgInt + pts[j].y;
                    avgx += pts[j].x;
                    cnt++;
                }

                avgInt = avgInt / cnt;
                avgx /= cnt;
                dataavg.add(PropertyUtil.ConvertFromRobot(avgx), avgInt);
                data.add(PropertyUtil.ConvertFromRobot(pts[i].x), pts[i].y);
            }
        } catch (Exception ex) {
            ex.printStackTrace();
        }
    }

    public ChartPanel getHistChartPanel() {
        return _histChartPan;
    }

    public void setHistChartPanel(ChartPanel histChartPanel) {
        _histChartPan = histChartPanel;
    }

    public void setScale(int min, int max) {
        XYPlot plot = (XYPlot) this._histChart.getPlot();
        plot.getDomainAxis().setRange(min, max);
        plot.getRangeAxis().setRange(-50, 700);
    }
}
