package com.nts.isip.tcp.isipgui;

import com.nts.isip.tcp.ClientFrame;
import com.nts.isip.tcp.PropertyRepresentation;
import com.nts.isip.tcp.StreamThread;
import com.nts.isip.tcp.sources.FrameSource;
import com.nts.isip.tcp.sources.FrameSourceBase;

import org.jfree.data.xy.XYSeries;

import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;

import java.io.IOException;

import java.util.Vector;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JList;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;


public class LineChooserDialog extends JDialog {
    private XYSeries _ser;
    private PropertyRepresentation _prop;
    private int _selectedLineIdx;
    private Vector<DataSeries> _2DData;
    private StreamThread _strThread;
    private double[] _points;

    //private String _type;
    private FrameSource _icamSource;
    private ClientFrame _parent;

    public LineChooserDialog(PropertyRepresentation prop, XYSeries ser,
        Vector<DataSeries> twoDData, StreamThread strThread,
        FrameSource icamSource, ClientFrame parent) {
        super(parent, "Line Chooser", true);
        this.setLocation(350, 200);
        this.setPreferredSize(new Dimension(200, 300));

        _ser = ser;
        _prop = prop;
        _2DData = twoDData;
        //_type = type;
        _icamSource = icamSource;
        _parent = parent;

        boolean hasIdx = false;

        for (int i = 0; i < _2DData.size(); i++) {
            if (_2DData.get(i).getName().equals("Selected lines")) {
                _selectedLineIdx = _2DData.get(i).getSerIdx();
                hasIdx = true;
            }
        }

        if (!hasIdx) {
            _selectedLineIdx = -1;
        }

        _strThread = strThread;
        init();
    }

    public void init() {
        this.setDefaultCloseOperation(JDialog.DO_NOTHING_ON_CLOSE);
        this.addWindowListener(new WindowListener() {
                public void windowActivated(WindowEvent e) {
                }

                public void windowClosed(WindowEvent e) {
                }

                public void windowClosing(WindowEvent e) {
                    if (_selectedLineIdx >= 0) {
                        _strThread.getView()
                                  .addCurrentSelectedPoints(null,
                            _selectedLineIdx);
                    }

                    dispose();
                }

                public void windowDeactivated(WindowEvent e) {
                }

                public void windowDeiconified(WindowEvent e) {
                }

                public void windowIconified(WindowEvent e) {
                }

                public void windowOpened(WindowEvent e) {
                }
            });

        JPanel dataPan = new JPanel();
        dataPan.setLayout(new BoxLayout(dataPan, BoxLayout.Y_AXIS));

        String[] lines = new String[_ser.getItemCount() / 2];
        int selectedIdx = 0;

        for (int i = 0; i < (_ser.getItemCount() - 1); i += 2) {
            String line = "(" + _ser.getDataItem(i).getX() + "," +
                _ser.getDataItem(i).getY() + ")" + " - " + "(" +
                _ser.getDataItem(i + 1).getX() + "," +
                _ser.getDataItem(i + 1).getY() + ")";

            //if(!_type.equals("line")) {
            lines[selectedIdx] = line;
            //} else {
            //	if (( selectedIdx % 2) == 0) {
            //		lines[selectedIdx] = line;
            //	}
            //}
            selectedIdx++;
        }

        JList lineList = new JList(lines);

        lineList.addListSelectionListener(new ListSelectionListener() {
                public void valueChanged(ListSelectionEvent e) {
                    //if(_currSelectedLineIdx >= 0) {
                    //	_strThread.getView().addCurrentSelectedPoints(null, _currSelectedLineIdx);
                    //}
                    JList list = (JList) e.getSource();
                    int[] lineIdx = list.getSelectedIndices();
                    Vector<Double> chosenPoints = new Vector<Double>();

                    for (int i = 0; i < lineIdx.length; i++) {
                        chosenPoints.add(_ser.getDataItem(lineIdx[i] * 2).getX()
                                             .doubleValue());
                        chosenPoints.add(_ser.getDataItem(lineIdx[i] * 2).getY()
                                             .doubleValue());
                        chosenPoints.add(_ser.getDataItem((lineIdx[i] * 2) + 1)
                                             .getX().doubleValue());
                        chosenPoints.add(_ser.getDataItem((lineIdx[i] * 2) + 1)
                                             .getY().doubleValue());
                    }

                    Double[] dp = chosenPoints.toArray(new Double[0]);
                    _points = new double[dp.length];

                    for (int i = 0; i < _points.length; i++) {
                        _points[i] = (double) dp[i];
                    }

                    if (_selectedLineIdx >= 0) {
                        _strThread.getView()
                                  .addCurrentSelectedPoints(_points,
                            _selectedLineIdx);
                    }
                }
            });

        JScrollPane scroll = new JScrollPane();
        scroll.getViewport().add(lineList);
        scroll.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED);
        dataPan.add(scroll);

        JPanel btnPan = new JPanel();
        btnPan.setLayout(new BoxLayout(btnPan, BoxLayout.X_AXIS));
        btnPan.setBorder(BorderFactory.createEmptyBorder(10, 10, 10, 10));
        btnPan.add(Box.createHorizontalGlue());

        JButton ok = new JButton("OK");
        ok.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    if (_points != null) {
                        if ((_prop != null) && (_prop.type != null) &&
                                _prop.type.equalsIgnoreCase("intArray")) {
                            int[] templatePoints = new int[_points.length];

                            for (int i = 0; i < _points.length; i++) {
                                templatePoints[i] = (int) _points[i];
                            }

                            try {
                                ((FrameSourceBase) _icamSource).setProperty(_prop,
                                    templatePoints);
                            } catch (IOException ex) {
                                JOptionPane.showMessageDialog(_parent,
                                    "Could not set property " + _prop.path +
                                    ".\n The system told us " +
                                    ex.getMessage(), "Error",
                                    JOptionPane.ERROR_MESSAGE);
                            }
                        }

                        if ((_prop != null) && (_prop.type != null) &&
                                _prop.type.equalsIgnoreCase("floatArray")) {
                            float[] templatePoints = new float[_points.length];

                            for (int i = 0; i < _points.length; i++) {
                                templatePoints[i] = (float) _points[i];
                            }

                            try {
                                ((FrameSourceBase) _icamSource).setProperty(_prop,
                                    templatePoints);
                            } catch (IOException ex) {
                                JOptionPane.showMessageDialog(_parent,
                                    "Could not set property " + _prop.path +
                                    ".\n The system told us " +
                                    ex.getMessage(), "Error",
                                    JOptionPane.ERROR_MESSAGE);
                            }
                        }
                    }

                    if (_selectedLineIdx >= 0) {
                        _strThread.getView()
                                  .addCurrentSelectedPoints(null,
                            _selectedLineIdx);
                    }

                    dispose();
                }
            });
        btnPan.add(ok);
        btnPan.add(Box.createRigidArea(new Dimension(10, 0)));

        JButton cancel = new JButton("Cancel");
        cancel.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    if (_selectedLineIdx >= 0) {
                        _strThread.getView()
                                  .addCurrentSelectedPoints(null,
                            _selectedLineIdx);
                    }

                    dispose();
                }
            });
        btnPan.add(cancel);
        btnPan.add(Box.createHorizontalGlue());

        JButton remove = new JButton("Remove");
        remove.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent arg0) {
                }
            });

        dataPan.add(Box.createRigidArea(new Dimension(0, 10)));
        dataPan.add(btnPan);
        this.getContentPane().add(dataPan);
        this.pack();
    }
}
