package com.nts.isip.tcp.isipgui;

import com.nts.isip.tcp.StreamThread;
import com.nts.isip.tcp.processors.ImageJFrameProcessor;
import com.nts.isip.tcp.processors.MathematicaFrameProcessor;
import com.nts.isip.tcp.processors.ParsingFrameProcessor;
import com.nts.isip.tcp.processors.RematchingFrameProcessor;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JFrame;


public class ProcessorTypeSetter extends JComboBox implements ActionListener {
    private JFrame _frame;
    private String[] _processors;
    private StreamThread _streamThr;
    private String _rematchDirName;
    private String _macrosDirName;
    private String _mathDirName;
    private JButton _editScript;
    private ActionListener _listener;

    public ProcessorTypeSetter(JFrame frame, StreamThread streamThr,
        String[] processors, String rematchDirName, String macrosDirName,
        String mathDirName, JButton editScript) {
        super(processors);
        _frame = frame;
        _processors = processors;
        _streamThr = streamThr;
        _rematchDirName = rematchDirName;
        _macrosDirName = macrosDirName;
        _mathDirName = mathDirName;
        _editScript = editScript;
        addActionListener(this);
    }

    public void actionPerformed(ActionEvent e) {
        System.out.println("Setter..");

        JComboBox cb = (JComboBox) e.getSource();
        String procName = (String) cb.getSelectedItem();

        if (procName.equals("Parsing")) {
            _editScript.setEnabled(false);
            _streamThr.setFrameProcessor(new ParsingFrameProcessor());
        } else if (procName.startsWith("Rematching")) {
            _editScript.setEnabled(false);

            String rematchFile = _rematchDirName + "/" +
                procName.substring(11, procName.length() - 1);
            RematchingFrameProcessor proc = new RematchingFrameProcessor(rematchFile);
            _streamThr.setFrameProcessor(proc);
        } else if (procName.startsWith("Mathematica")) {
            final String mathFileName = procName.substring(12,
                    procName.length() - 1);
            final String mathFile = _mathDirName + "/" + mathFileName;

            _editScript.setEnabled(true);

            if (_listener != null) {
                _editScript.removeActionListener(_listener);
            }

            _listener = new ActionListener() {
                        public void actionPerformed(ActionEvent e) {
                            new ScriptEditor(mathFileName, mathFile);
                        }
                    };
            _editScript.addActionListener(_listener);

            MathematicaFrameProcessor proc = new MathematicaFrameProcessor(mathFile);
            _streamThr.setFrameProcessor(proc);
        } else if (procName.startsWith("ImageJ")) {
            final String macrosFileName = procName.substring(7,
                    procName.length() - 1);
            final String macrosFile = _macrosDirName + "/" + macrosFileName;

            if (_listener != null) {
                _editScript.removeActionListener(_listener);
            }

            _listener = new ActionListener() {
                        public void actionPerformed(ActionEvent e) {
                            new ScriptEditor(macrosFileName, macrosFile);
                        }
                    };
            _editScript.setEnabled(true);
            _editScript.addActionListener(_listener);

            ImageJFrameProcessor proc = new ImageJFrameProcessor(macrosFile);
            _streamThr.setFrameProcessor(proc);
        } else if (procName.equals("Custom...")) {
            _editScript.setEnabled(false);

            String[] procs = new String[_processors.length - 1];
            System.arraycopy(_processors, 0, procs, 0, _processors.length - 1);

            CustomProcessorsSetterDialog customProcs = new CustomProcessorsSetterDialog(_frame,
                    procs, _streamThr, _rematchDirName, _macrosDirName,
                    _mathDirName);
            customProcs.setVisible(true);
        }
    }
}
