package com.nts.isip.tcp.isipgui;

import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;

import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextPane;
import javax.swing.JToolBar;


public class ScriptEditor extends JDialog {
    private JTextPane _text;
    private String _filePath;
    private boolean _saved = true;
    private String _title;

    public ScriptEditor(String title, String filePath) {
        _title = title;
        this.setTitle(title);
        _filePath = filePath;
        this.init();
    }

    public ScriptEditor(String title, String filePath, JDialog parent) {
        super(parent, false);
        _title = title;
        this.setTitle(title);
        _filePath = filePath;
        this.init();
    }

    public void init() {
        JPanel pan = new JPanel();
        JToolBar toolBar = new JToolBar();
        pan.setLayout(new BorderLayout());

        JButton save = new JButton("Save");
        save.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    save(_filePath, _title);
                }
            });
        toolBar.add(save);
        pan.add(toolBar, BorderLayout.NORTH);

        _text = new JTextPane();
        _text.setEditable(true);
        _text.addKeyListener(new KeyListener() {
                public void keyTyped(KeyEvent arg0) {
                    setUnSavedTitle(_title + "*");
                }

                public void keyPressed(KeyEvent arg0) {
                    setUnSavedTitle(_title + "*");
                }

                public void keyReleased(KeyEvent arg0) {
                    setUnSavedTitle(_title + "*");
                }
            });

        try {
            BufferedReader in = new BufferedReader(new FileReader(_filePath));
            String line = new String();
            StringBuffer content = new StringBuffer();

            while ((line = in.readLine()) != null) {
                content.append(line);
                content.append("\n");
            }

            _text.setText(content.toString());
        } catch (IOException e) {
            JOptionPane.showMessageDialog(this,
                "File error.\n" + "The System told us" + e.getMessage(),
                "Error", JOptionPane.ERROR_MESSAGE);
        }

        JScrollPane scroll = new JScrollPane();
        scroll.getViewport().add(_text);
        scroll.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED);
        pan.add(scroll, BorderLayout.CENTER);

        this.getContentPane().add(pan);
        this.pack();
        this.setSize(500, 600);
        this.setVisible(true);
        this.setLocationRelativeTo(null);
        this.setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);
        this.addWindowListener(new WindowListener() {
                public void windowOpened(WindowEvent arg0) {
                }

                public void windowClosed(WindowEvent arg0) {
                }

                public void windowIconified(WindowEvent arg0) {
                }

                public void windowDeiconified(WindowEvent arg0) {
                }

                public void windowActivated(WindowEvent arg0) {
                }

                public void windowDeactivated(WindowEvent arg0) {
                }

                public void windowClosing(WindowEvent arg0) {
                    if (!_saved) {
                        int saving = JOptionPane.showConfirmDialog(null,
                                "Do you want to save the changes?",
                                "Select an Option",
                                JOptionPane.YES_NO_CANCEL_OPTION,
                                JOptionPane.PLAIN_MESSAGE);

                        if (saving == JOptionPane.YES_OPTION) {
                            save(_filePath, _title);
                            disposeOnClose();
                        } else if (saving == JOptionPane.NO_OPTION) {
                            disposeOnClose();
                        } else if (saving == JOptionPane.CANCEL_OPTION) {
                        }
                    } else {
                        disposeOnClose();
                    }
                }
            });
    }

    private void save(String filePath, String title) {
        String newContent = _text.getText();

        try {
            BufferedWriter out = new BufferedWriter(new FileWriter(filePath));
            out.write(newContent);
            out.close();

            this.setTitle(title);
            _saved = true;
        } catch (IOException e) {
            JOptionPane.showMessageDialog(this,
                "File error.\n" + "The System told us" + e.getMessage(),
                "Error", JOptionPane.ERROR_MESSAGE);
        }
    }

    private void setUnSavedTitle(String title) {
        this.setTitle(title);
        _saved = false;
    }

    private void disposeOnClose() {
        this.dispose();
    }
}
