package com.nts.isip.tcp.isipgui;

import org.jfree.ui.Drawable;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.Stroke;
import java.awt.geom.Ellipse2D;
import java.awt.geom.Line2D;
import java.awt.geom.Rectangle2D;


public class XYSeriesTypeCircleDrawer implements Drawable {
    /** The outline color. */
    private Color outlineColor;

    /** The outline stroke. */
    private Stroke outlineStroke;

    /** The fill color. */
    private Color fillColor;

    /**
     * Creates a new instance.
     * @param outlineColor  the outline color.
     * @param outlineStroke  the outline stroke.
     * @param fillColor  the fill color.
     */
    public XYSeriesTypeCircleDrawer(Color outlineColor, Stroke outlineStroke,
        Color fillColor) {
        this.outlineColor = outlineColor;
        this.outlineStroke = outlineStroke;
        this.fillColor = fillColor;
    }

    /**
     * Draws the circle with two lines
     * @param g2  the graphics device.
     * @param area  the area in which to draw.
     */
    public void draw(Graphics2D g2, Rectangle2D area) {
        Ellipse2D ellipse = new Ellipse2D.Double(area.getX(), area.getY(),
                area.getWidth(), area.getHeight());

        if (this.fillColor != null) {
            g2.setPaint(this.fillColor);
            g2.fill(ellipse);
        }

        if ((this.outlineColor != null) && (this.outlineStroke != null)) {
            g2.setPaint(this.outlineColor);
            g2.setStroke(this.outlineStroke);
            g2.draw(ellipse);
        }

        g2.setPaint(Color.black);
        g2.setStroke(new BasicStroke(1.5f));

        Line2D line1 = new Line2D.Double(area.getCenterX(), area.getMinY(),
                area.getCenterX(), area.getMaxY());
        Line2D line2 = new Line2D.Double(area.getMinX(), area.getCenterY(),
                area.getMaxX(), area.getCenterY());
        g2.draw(line1);
        g2.draw(line2);
    }
}
