package com.nts.isip.tcp.models;

import com.nts.isip.tcp.PropertyRepresentation;
import com.nts.isip.tcp.exceptions.NoSuchPropertyException;

import java.io.IOException;

import javax.swing.JOptionPane;
import javax.swing.table.DefaultTableModel;


public class IcamPropertyModel extends DefaultTableModel {
    private PropertyRepresentation[] _properties;
    private EnumeratingPropertyModel _propModel;

    public IcamPropertyModel(PropertyRepresentation[] properties,
        EnumeratingPropertyModel icamFrameSource) {
        _properties = properties;
        _propModel = icamFrameSource;
    }

    public int getRowCount() {
        if (_properties != null) {
            return _properties.length;
        } else {
            return 0;
        }
    }

    public int getColumnCount() {
        return 2;
    }

    public boolean isCellEditable(int r, int c) {
        if (c == 1) {
            if (_properties[r].isReadOnly) {
                return false;
            } else {
                if (_properties[r].isAdminMode) {
                    if (_propModel.isInAdminMode()) {
                        return true;
                    } else {
                        return false;
                    }
                } else {
                    return true;
                }
            }
        } else {
            return false;
        }
    }

    public PropertyRepresentation getCellProperty(int r, int c) {
        if (_properties != null) {
            if (c == 0) {
                return null;
            } else {
                if (_properties[r] != null) {
                    return _properties[r];
                } else {
                    return null;
                }
            }
        }

        return null;
    }

    public Object getValueAt(int r, int c) {
        if (_properties != null) {
            if (c == 0) {
                return _properties[r].label;
            } else {
                if (_properties[r] != null) {
                    return _properties[r].value;
                } else {
                    return null;
                }
            }
        } else {
            return null;
        }
    }

    public void setValueAt(Object val, int r, int c) {
        if ((_properties != null) && (c == 1)) {
            try {
                _propModel.setProperty(_properties[r], val);
                _properties[r].value = _propModel.getProperty(_properties[r].path,
                        _properties[r].type, false);
                this.fireTableCellUpdated(r, c);
            } catch (NoSuchPropertyException e) {
                e.printStackTrace();
            } catch (NullPointerException e) {
                e.printStackTrace();
            } catch (IOException e) {
                String msg = "Could not connect to server.\n" +
                    "The system told us: " + e.getMessage();
                JOptionPane.showMessageDialog(null, msg, "Error",
                    JOptionPane.ERROR_MESSAGE);
            }
        }
    }
    
    public void changeProperty(String propName, Object val) {
        for (int i = 0; i < _properties.length; i++) {
            if (propName.equals(_properties[i].path)) {
                _properties[i].value = val;
                System.out.println(propName);
                System.out.println(val);
                this.fireTableCellUpdated(i, 1);

                break;
            }
        }
    }

    public void setProperties(PropertyRepresentation[] properties) {
        _properties = properties;
    }
}
