package com.nts.isip.tcp.models;

import com.nts.isip.tcp.PropertyRepresentation;
import com.nts.isip.tcp.exceptions.NoSuchPropertyException;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

import java.io.IOException;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;

import java.util.Vector;

import javax.swing.JOptionPane;


/**
 * A base class for property models.
 * A property model has a number of properties. A property constitutes
 * a getXXX, and optionally a setXXX method, where XXX is the name of
 * the property. The getProperty and setProperty methods will delegate
 * to the appropriate getter or setter. There are also indexed properties.
 * An indexed property is the same as a normal property, except that its
 * type is an array. The getIndexedProperty and setIndexedProperty methods
 * can be used to read and modify specific elements of indexed properties.
 */
public class PropertyModel {
    private Vector<PropertyChangeListener> _listeners = new Vector<PropertyChangeListener>();

    public void addPropertyChangeListener(PropertyChangeListener l) {
        _listeners.add(l);
    }

    public void removePropertyChangeListener(PropertyChangeListener l) {
        _listeners.remove(l);
    }

    public Object getProperty(String name, String type, boolean points) {
        //if(_port == null) {
        //	return null;
        //}
        Object result = null;
        String methodName = "getProperty";
        Object[] arglist = new Object[1];
        arglist[0] = new String(name);

        if (type.equalsIgnoreCase("intArray")) {
            if (points) {
                methodName += "PointsIntArray";
            } else {
                methodName += "IntArray";
            }
        } else if (type.equalsIgnoreCase("floatArray")) {
            if (points) {
                methodName += "PointsFloatArray";
            } else {
                methodName += "FloatArray";
            }
        } else if (type.equalsIgnoreCase("image")) {
            methodName += "Image";
        } else if (type.equalsIgnoreCase("stringArray")) {
            methodName += "StringArray";
        } else if (type.equalsIgnoreCase("float")) {
            methodName += "ValueFloat";
        } else if (type.equalsIgnoreCase("int")) {
            methodName += "ValueInt";
        } else if (type.equalsIgnoreCase("String") ||
                type.equalsIgnoreCase("enumeration") ||
                type.equalsIgnoreCase("dynamiclist") ||
                type.equalsIgnoreCase("dynamic")) {
            methodName += "ValueString";
        } else {
            //System.out.println("The type of the property " + name + " is: " +
            //    type);
            return null;
        }

        Class thisClass = this.getClass();

        try {
            Class c = Class.forName("java.lang.String");
            Method getMethod = thisClass.getMethod(methodName, c);
            result = getMethod.invoke(this, arglist);
            //System.out.println("Value of property " + name + " is: " + result);
        } catch (NoSuchMethodException e) {
            throw new NoSuchPropertyException(e);
        } catch (InvocationTargetException e) {
            if (e.getCause() instanceof RuntimeException) {
                throw (RuntimeException) e.getCause();
            } else {
                e.printStackTrace();
            }
        } catch (IllegalAccessException e) {
            throw new NoSuchPropertyException(e);
        } catch (ClassNotFoundException e) {
            e.printStackTrace();
        }

        return result;
    }

    public void setProperty(PropertyRepresentation prop, Object val)
        throws IOException {
        //if(_port == null) {
        //	throw new IOException("Disconnected");
        //}
    	//System.out.println(prop.path + " setting value to: " + val );
        String methodName = "setProperty";
        Class thisClass = this.getClass();

        if (prop.type.equalsIgnoreCase("intArray")) {
            methodName += "IntArray";
        } else if (prop.type.equalsIgnoreCase("floatArray")) {
            methodName += "FloatArray";

            //} else if(prop.type.equals("image")) {
            //	methodName += "Image";
        } else if (prop.type.equalsIgnoreCase("stringArray")) {
            methodName += "StringArray";
        } else if (prop.type.equalsIgnoreCase("float")) {
            methodName += "ValueFloat";
        } else if (prop.type.equalsIgnoreCase("int")) {
            methodName += "ValueInt";
        } else if (prop.type.equalsIgnoreCase("String") ||
                prop.type.equalsIgnoreCase("enumeration") ||
                prop.type.equalsIgnoreCase("dynamiclist")) {
            methodName += "ValueString";
        } else {
            JOptionPane.showMessageDialog(null, "Unknown property type!",
                "Information", JOptionPane.INFORMATION_MESSAGE);

            return;
        }

        Class[] parsType = { prop.getClass(), Object.class };

        try {
            Method setMethod = thisClass.getMethod(methodName, parsType);
            Object[] pars = { prop, val };
            setMethod.invoke(this, pars);
        } catch (NoSuchMethodException e) {
            throw new NoSuchPropertyException(e);
        } catch (InvocationTargetException e) {
            if (e.getCause() instanceof RuntimeException) {
                throw (RuntimeException) e.getCause();
            } else {
                e.printStackTrace();
            }
        } catch (IllegalAccessException e) {
            throw new NoSuchPropertyException(e);
        }
    }

    /*public Object getProperty(String prop) {
        Object result = null;
        String methodName = "get" + prop;
        Class thisClass = this.getClass();
    
        try {
            Method getMethod = thisClass.getMethod(methodName, (Class) null);
            result = getMethod.invoke(this, (Object[]) null);
        } catch (NoSuchMethodException e) {
            throw new NoSuchPropertyException(e);
        } catch (InvocationTargetException e) {
            if (e.getCause() instanceof RuntimeException) {
                throw (RuntimeException) e.getCause();
            } else {
                e.printStackTrace();
            }
        } catch (IllegalAccessException e) {
            throw new NoSuchPropertyException(e);
        }
    
        return result;
    }
    
    public Object getIndexedProperty(String prop, int idx) {
        Object array = getProperty(prop);
    
        return Array.get(array, idx);
    }*/

    /*public void setProperty(String prop, Object val) {
        String methodName = "set" + prop;
        Class thisClass = this.getClass();
        Class paramClass;
    
        if (val instanceof Boolean) {
            paramClass = boolean.class;
        } else if (val instanceof Integer) {
            paramClass = int.class;
        } else if (val instanceof Float) {
            paramClass = float.class;
        } else if (val instanceof Double) {
            paramClass = double.class;
        } else {
            paramClass = val.getClass();
        }
    
        Class[] parsType = { paramClass };
        Method setMethod;
    
        try {
            setMethod = thisClass.getMethod(methodName, parsType);
    
            Object[] pars = { val };
            setMethod.invoke(this, pars);
        } catch (NoSuchMethodException e) {
            throw new NoSuchPropertyException(e);
        } catch (IllegalAccessException e) {
            throw new NoSuchPropertyException(e);
        } catch (InvocationTargetException e) {
            if (e.getCause() instanceof RuntimeException) {
                throw (RuntimeException) e.getCause();
            } else {
                e.printStackTrace();
            }
        }
    }
    
    public void setIndexedProperty(String prop, int idx, Object val) {
        Object array = getProperty(prop);
        Array.set(array, idx, val);
        setProperty(prop, array);
    }*/
    protected void firePropertyChangeEvent(String name, Object oldVal,
        Object newVal) {
        PropertyChangeEvent e = new PropertyChangeEvent(this, name, oldVal,
                newVal);

        for (int i = 0; i != _listeners.size(); ++i) {
            _listeners.elementAt(i).propertyChange(e);
        }
    }
}
