package com.nts.isip.tcp.processors;

import com.nts.isip.tcp.HistogramRepresentation;
import com.nts.isip.tcp.ImageRepresentation;
import com.nts.isip.tcp.PropertyRepresentation;
import com.nts.isip.tcp.exceptions.CheckSumException;
import com.nts.isip.tcp.models.EnumeratingPropertyModel;

import java.io.IOException;

import java.util.Vector;


public abstract class FrameProcessorBase extends EnumeratingPropertyModel
    implements FrameProcessor {
    FrameProcessor _linkedProcessor;

    //private Vector<byte[]> _fullPackage = new Vector<byte[]>();
    //private Vector<Integer> _status = new Vector<Integer>();
    private Vector<ImageRepresentation> _images = new Vector<ImageRepresentation>();
    private Vector<HistogramRepresentation> _histograms = new Vector<HistogramRepresentation>();
    private int[] _lightLevel;
    private Vector<double[]> _2DData = new Vector<double[]>();
    private Vector<double[]> _3DData = new Vector<double[]>();
    private Vector<Integer> _frameNumber = new Vector<Integer>();

    //private Vector<Long> _timeStamp = new Vector<Long>();
    //private Vector<Double> _numericData = new Vector<Double>();
    //private Vector<String> _numericDataNames = new Vector<String>();
    private PropertyRepresentation[] _channelProps;
    private boolean _adminMode;
    private double[] _pixelProfile;

    //Any additional read-only properties the processor provides
    public String[] getExtraReadPropertyList() {
        return new String[0];
    }

    //Any additional read-write properties the processor provides
    public String[] getExtraWritePropertyList() {
        return new String[0];
    }

    public void setFrameProcessor(FrameProcessor fp) {
        _linkedProcessor = fp;
    }

    /*public byte[] getFullpackage() {
        if (_linkedProcessor != null) {
            return _linkedProcessor.getFullpackage();
        } else {
            _fullPackage = getBaseFullPackage();
    
            if (_fullPackage.isEmpty()) {
                return null;
            }
    
            return (byte[]) _fullPackage.toArray()[0];
        }
    }*/

    /*public int getStatus() {
        if (_linkedProcessor != null) {
            return _linkedProcessor.getStatus();
        } else {
            _status = getBaseStatus();
    
            if (_status.isEmpty()) {
                return 0;
            }
    
            return (Integer) _status.toArray()[0];
        }
    }*/
    public ImageRepresentation[] getImages() {
        if (_linkedProcessor != null) {
            return _linkedProcessor.getImages();
        } else {
            _images = getBaseImages();

            if (_images.isEmpty()) {
                return null;
            }

            return (ImageRepresentation[]) _images.toArray(new ImageRepresentation[0]);
        }
    }

    public HistogramRepresentation[] getHistograms() {
        if (_linkedProcessor != null) {
            return _linkedProcessor.getHistograms();
        } else {
            _histograms = getBaseHistograms();

            if (_histograms.isEmpty()) {
                return null;
            }

            return (HistogramRepresentation[]) _histograms.toArray(new HistogramRepresentation[0]);
        }
    }

    public int[] getLightLevel() {
        if (_linkedProcessor != null) {
            return _linkedProcessor.getLightLevel();
        } else {
            _lightLevel = getBaseLightLevel();

            return _lightLevel;
        }
    }

    public PropertyRepresentation[] getChannelProps() {
        if (_linkedProcessor != null) {
            return _linkedProcessor.getChannelProps();
        } else {
            _channelProps = getBaseChannelProps();

            return _channelProps;
        }
    }

    public double[][] get2DData() throws IOException, CheckSumException {
        if (_linkedProcessor != null) {
            return _linkedProcessor.get2DData();
        } else {
            _2DData = getBase2DData();

            if (_2DData.isEmpty()) {
                return null;
            }

            return (double[][]) _2DData.toArray(new double[0][]);
        }
    }

    public double[][] get3DData() throws IOException, CheckSumException {
        if (_linkedProcessor != null) {
            return _linkedProcessor.get3DData();
        } else {
            _3DData = getBase3DData();

            if (_3DData.isEmpty()) {
                return null;
            }

            return (double[][]) _3DData.toArray(new double[0][]);
        }
    }

    public int getFrameNumber() {
        if (_linkedProcessor != null) {
            return _linkedProcessor.getFrameNumber();
        } else {
            _frameNumber = getBaseFrameNumber();

            if (_frameNumber.isEmpty()) {
                return 0;
            }

            return (Integer) _frameNumber.toArray()[0];
        }
    }

    public double[] getPixelProfile() {
        if (_linkedProcessor != null) {
            return _linkedProcessor.getPixelProfile();
        } else {
            _pixelProfile = getBasePixelProfile();

            return _pixelProfile;
        }
    }

    public boolean isInAdminMode() {
        if (_linkedProcessor != null) {
            return _linkedProcessor.isInAdminMode();
        } else {
            _adminMode = getBaseAdminMode();

            return _adminMode;
        }
    }

    /*public long getTimeStamp() {
        if (_linkedProcessor != null) {
            return _linkedProcessor.getTimeStamp();
        } else {
            _timeStamp = getBaseTimeStamp();
    
            if (_timeStamp.isEmpty()) {
                return 0;
            }
    
            return (Long) _timeStamp.toArray()[0];
        }
    }*/

    /*public String[] getPropertyNames() {
        if (_linkedProcessor != null) {
            String[] prop1 = getPropertyNamesImpl();
            String[] prop2 = ((FrameProcessorBase) _linkedProcessor).getPropertyNames();
            int size1;
    
            try {
                size1 = prop1.length;
            } catch (NullPointerException e) {
                size1 = 0;
            }
    
            int size2;
    
            try {
                size2 = prop2.length;
            } catch (NullPointerException e) {
                size2 = 0;
            }
    
            String[] resProp = new String[size1 + size2];
    
            for (int i = 0; i < size1; i++) {
                resProp[i] = prop1[i];
            }
    
            for (int i = 0; i < size2; i++) {
                resProp[size1 + i] = prop2[i];
            }
    
            return resProp;
        } else {
            return getPropertyNamesImpl();
        }
    }*/

    /*public double[] getNumericData() {
        if (_linkedProcessor != null) {
            return _linkedProcessor.getNumericData();
        } else {
            _numericData = getBaseNumericData();
    
            if (_numericData.isEmpty()) {
                return null;
            }
    
            Double[] data = (Double[]) _numericData.toArray(new Double[0]);
            double[] rData = new double[data.length];
    
            for (int i = 0; i < data.length; i++) {
                rData[i] = (double) data[i];
            }
    
            return rData;
        }
    }*/

    /*public String[] getNumericDataNames() {
        if (_linkedProcessor != null) {
            return _linkedProcessor.getNumericDataNames();
        } else {
            _numericDataNames = getBaseNumericDataNames();
    
            if (_numericDataNames.isEmpty()) {
                String[] names = new String[] {  };
    
                return names;
            }
    
            return (String[]) _numericDataNames.toArray(new String[0]);
        }
    }*/

    //protected abstract String[] getPropertyNamesImpl();

    /*private Vector<Integer> getBaseStatus() {
        if (_linkedProcessor != null) {
            return ((FrameProcessorBase) _linkedProcessor).getBaseStatus();
        } else {
            return _status;
        }
    }*/

    /*private Vector<byte[]> getBaseFullPackage() {
        if (_linkedProcessor != null) {
            return ((FrameProcessorBase) _linkedProcessor).getBaseFullPackage();
        } else {
            return _fullPackage;
        }
    }*/
    private Vector<ImageRepresentation> getBaseImages() {
        if (_linkedProcessor != null) {
            return ((FrameProcessorBase) _linkedProcessor).getBaseImages();
        } else {
            return _images;
        }
    }

    private Vector<HistogramRepresentation> getBaseHistograms() {
        if (_linkedProcessor != null) {
            return ((FrameProcessorBase) _linkedProcessor).getBaseHistograms();
        } else {
            return _histograms;
        }
    }

    private int[] getBaseLightLevel() {
        if (_linkedProcessor != null) {
            return ((FrameProcessorBase) _linkedProcessor).getBaseLightLevel();
        } else {
            return _lightLevel;
        }
    }

    private PropertyRepresentation[] getBaseChannelProps() {
        if (_linkedProcessor != null) {
            return ((FrameProcessorBase) _linkedProcessor).getBaseChannelProps();
        } else {
            return _channelProps;
        }
    }

    private Vector<double[]> getBase2DData() {
        if (_linkedProcessor != null) {
            return ((FrameProcessorBase) _linkedProcessor).getBase2DData();
        } else {
            return _2DData;
        }
    }

    private Vector<double[]> getBase3DData() {
        if (_linkedProcessor != null) {
            return ((FrameProcessorBase) _linkedProcessor).getBase3DData();
        } else {
            return _3DData;
        }
    }

    private Vector<Integer> getBaseFrameNumber() {
        if (_linkedProcessor != null) {
            return ((FrameProcessorBase) _linkedProcessor).getBaseFrameNumber();
        } else {
            return _frameNumber;
        }
    }

    private double[] getBasePixelProfile() {
        if (_linkedProcessor != null) {
            return ((FrameProcessorBase) _linkedProcessor).getBasePixelProfile();
        } else {
            return _pixelProfile;
        }
    }

    private boolean getBaseAdminMode() {
        if (_linkedProcessor != null) {
            return ((FrameProcessorBase) _linkedProcessor).getBaseAdminMode();
        } else {
            return _adminMode;
        }
    }

    /*private Vector<Long> getBaseTimeStamp() {
        if (_linkedProcessor != null) {
            return ((FrameProcessorBase) _linkedProcessor).getBaseTimeStamp();
        } else {
            return _timeStamp;
        }
    }*/

    /*private Vector<Double> getBaseNumericData() {
        if (_linkedProcessor != null) {
            return ((FrameProcessorBase) _linkedProcessor).getBaseNumericData();
        } else {
            return _numericData;
        }
    }*/

    /*private Vector<String> getBaseNumericDataNames() {
        if (_linkedProcessor != null) {
            return ((FrameProcessorBase) _linkedProcessor).getBaseNumericDataNames();
        } else {
            return _numericDataNames;
        }
    }*/

    /*public Object getProperty(String prop) {
        try {
            String[] propertyNames = getPropertyNamesImpl();
            boolean hasProperty = false;
    
            if (propertyNames != null) {
                for (int i = 0; i < propertyNames.length; i++) {
                    if (prop.equals(propertyNames[i])) {
                        hasProperty = true;
    
                        break;
                    }
                }
            }
    
            if (hasProperty) {
                return super.getProperty(prop);
            } else {
                if (_linkedProcessor != null) {
                    return ((FrameProcessorBase) _linkedProcessor).getProperty(prop);
                } else {
                    return null;
                }
            }
        } catch (NoSuchPropertyException e) {
            if (_linkedProcessor != null) {
                return ((FrameProcessorBase) _linkedProcessor).getProperty(prop);
            } else {
                return null;
            }
        }
    }*/

    /*public void setProperty(String prop, Object val) {
        try {
            String[] propertyNames = getPropertyNamesImpl();
            boolean hasProperty = false;
    
            if (propertyNames != null) {
                for (int i = 0; i < propertyNames.length; i++) {
                    if (prop.equals(propertyNames[i])) {
                        hasProperty = true;
    
                        break;
                    }
                }
            }
    
            if (hasProperty) {
                super.setProperty(prop, val);
            } else {
                if (_linkedProcessor != null) {
                    ((FrameProcessorBase) _linkedProcessor).setProperty(prop,
                        val);
                }
            }
        } catch (NoSuchPropertyException e) {
            if (_linkedProcessor != null) {
                ((FrameProcessorBase) _linkedProcessor).setProperty(prop, val);
            }
        }
    }*/

    /*public void addFullPackageResult(byte[] fullPackage) {
        getBaseFullPackage().setSize(1);
        getBaseFullPackage().setElementAt(fullPackage, 0);
    }*/

    /*public void addStatusResults(int status) {
        getBaseStatus().setSize(1);
        getBaseStatus().setElementAt(status, 0);
    }*/
    public void addImagesResults(ImageRepresentation[] images) {
        if ((images != null) && (images.length > 0)) {
            if (images.length > _images.size()) {
                getBaseImages().setSize(images.length);
            }

            for (int i = 0; i < images.length; i++) {
                getBaseImages().setElementAt(images[i], i);
            }
        } else {
            getBaseImages().clear();
        }
    }

    public void addHistogramsResults(HistogramRepresentation[] histograms) {
        if (histograms != null) {
            if (histograms.length > getBaseHistograms().size()) {
                getBaseHistograms().setSize(histograms.length);
            }

            for (int i = 0; i < histograms.length; i++) {
                if (histograms[i] != null) {
                    getBaseHistograms().setElementAt(histograms[i], i);
                }
            }
        }
    }

    public void addLightLevelResults(int[] lightLevel) {
        if (lightLevel != null) {
            _lightLevel = lightLevel;
        }
    }

    public void addChannelPropsResults(PropertyRepresentation[] channelProps) {
        if (channelProps != null) {
            _channelProps = channelProps;
        }
    }

    public void add2DResults(double[][] new2DData) {
        if (new2DData != null) {
            if (new2DData.length > getBase2DData().size()) {
                getBase2DData().setSize(new2DData.length);
            }

            for (int i = 0; i < new2DData.length; i++) {
                getBase2DData().setElementAt(new2DData[i], i);
            }
        }
    }

    public void add3DResults(double[][] new3DData) {
        if (new3DData != null) {
            if (new3DData.length > getBase3DData().size()) {
                getBase3DData().setSize(new3DData.length);
            }

            for (int i = 0; i < new3DData.length; i++) {
                if (new3DData[i] != null) {
                    getBase3DData().setElementAt(new3DData[i], i);
                }
            }
        }
    }

    public void addFrameNumberResults(int frameNumber) {
        getBaseFrameNumber().setSize(1);
        getBaseFrameNumber().add(0, frameNumber);
    }

    public void addPixelProfileResults(double[] pixelProfile) {
        _pixelProfile = pixelProfile;
    }

    public void addAdminMode(boolean adminMode) {
        _adminMode = adminMode;
    }

    /*public void addTimeStampResults(long timeStamp) {
        getBaseTimeStamp().setSize(1);
        getBaseTimeStamp().add(0, timeStamp);
    }*/

    /*public void addNumericDataResults(double[] numericData) {
        if (numericData != null) {
            if (numericData.length > getBaseNumericData().size()) {
                getBaseNumericData().setSize(numericData.length);
            }
    
            for (int i = 0; i < numericData.length; i++) {
                if (numericData[i] != Double.NaN) {
                    getBaseNumericData().setElementAt(numericData[i], i);
                }
            }
        }
    }*/

    /*public void addNumericDataNamesResults(String[] numericDataNames) {
        if (numericDataNames != null) {
            if (numericDataNames.length > getBaseNumericDataNames().size()) {
                getBaseNumericDataNames().setSize(numericDataNames.length);
            }
    
            for (int i = 0; i < numericDataNames.length; i++) {
                getBaseNumericDataNames().setElementAt(numericDataNames[i], i);
            }
        }
    }*/
}
