package com.nts.isip.tcp.processors;

import com.nts.isip.tcp.HistogramRepresentation;
import com.nts.isip.tcp.ImageRepresentation;
import com.nts.isip.tcp.PropertyRepresentation;
import com.nts.isip.tcp.exceptions.CheckSumException;
import com.nts.isip.tcp.sources.FrameSourceBase;

import com.ntsbg.isip.ij.IJUtil;

import ij.IJ;
import ij.ImagePlus;
import ij.ImageStack;
import ij.WindowManager;

import java.io.IOException;


public class ImageJFrameProcessor extends FrameProcessorBase {
    //private byte[] _fullPackage;
    //private int _status;
    private ImageRepresentation[] _images;
    private HistogramRepresentation[] _histograms;
    private int[] _lightLevel;
    private double[][] _2DData;
    private double[][] _3DData;

    //private double[] _numericData;
    private int _frameCnt;
    private double[] _pixelProfile;
    private boolean _adminMode;

    //private long _timestamp;
    private PropertyRepresentation[] _channelProps;

    //    private PackageParameters _params = new PackageParameters();
    private String _macrosFileName;
    private String _propsFileName;

    public ImageJFrameProcessor(String macrosFileName) {
        _macrosFileName = macrosFileName;
        _propsFileName = _macrosFileName.replace(".txt", ".props");

        try {
            java.io.FileInputStream in = new java.io.FileInputStream(_propsFileName);

            try {
                IJUtil.GetConfig().clear();
                IJUtil.GetConfig().load(in);
            } finally {
                in.close();
            }
        } catch (java.io.IOException e) {
            IJUtil.GetConfig().clear();
        }

        //getNumericDataNamesImpl();
    }

    public void processFrame(FrameProcessor fp)
        throws IOException, CheckSumException {
        FrameSourceBase processor = (FrameSourceBase) fp;
        _images = processor.getImages();

        if (_images != null) {
            int n = 0;

            while ((n < _images.length) && (_images[n] == null)) {
                n++;
            }

            if (n < _images.length) {
                ImageStack stack = new ImageStack(_images[n].width,
                        _images[n].height);

                for (int i = 0; i < _images.length; i++) {
                    byte[] data;

                    if (_images[i] != null) {
                        if (_images[i].bitDepth <= 8) {
                            data = new byte[_images[i].data.length];
                            System.arraycopy(_images[i].data, 0, data, 0,
                                data.length);
                        } else {
                            data = new byte[_images[i].data.length / 2];

                            for (int j = 0; j < data.length; j++) {
                                int d1 = _images[i].data[j * 2] & 0xff;
                                int d2 = _images[i].data[(j * 2) + 1] & 0xff;
                                short val = (short) ((d2 << 8) | d1);
                                data[j] = (byte) (val >> (_images[i].bitDepth -
                                    8));
                            }
                        }

                        stack.addSlice("Images", data);
                    }
                }

                if (stack.getSize() > 0) {
                    ImagePlus isipImages = new ImagePlus("Isip Images", stack);
                    WindowManager.setTempCurrentImage(isipImages);

                    System.out.println("Running macro file");
                    IJ.runMacroFile(_macrosFileName);
                }

                _2DData = IJUtil.Get2DData();
                _3DData = IJUtil.Get3DData();
                //_status = processor.getStatus();
                _frameCnt = processor.getFrameNumber();
                //_timestamp = processor.getTimeStamp();
                //_fullPackage = processor.getFullpackage();
                _lightLevel = processor.getLightLevel();
                _channelProps = processor.getChannelProps();
                _adminMode = processor.isInAdminMode();
                _pixelProfile = processor.getPixelProfile();
            }
        }

        try {
            _histograms = processor.getHistograms();
        } catch (NullPointerException e) {
            _histograms = null;
        }

        //        try {
        //            _params = (PackageParameters) processor.getProperty(
        //                    "PackageParameters");
        //        } catch (NoSuchPropertyException e) {
        //        } catch (NullPointerException e) {
        //        }

        //addFullPackageResult(_fullPackage);
        //addStatusResults(_status);
        addImagesResults(_images);
        addHistogramsResults(_histograms);
        addLightLevelResults(_lightLevel);
        addChannelPropsResults(_channelProps);
        add2DResults(_2DData);
        add3DResults(_3DData);
        addFrameNumberResults(_frameCnt);
        addPixelProfileResults(_pixelProfile);
        addAdminMode(_adminMode);

        //addTimeStampResults(_timestamp);
        //addNumericDataResults(_numericData);
        if (_linkedProcessor != null) {
            _linkedProcessor.processFrame(processor);
        }
    }

    //    public PackageParameters getPackageParameters() {
    //        return _params;
    //    }
    /*protected String[] getPropertyNamesImpl() {
        StringBuffer names = new StringBuffer("");
    
        if (!IJUtil.GetConfig().isEmpty()) {
            Enumeration en = IJUtil.GetConfig().propertyNames();
    
            while (en.hasMoreElements()) {
                names.append(en.nextElement().toString());
                names.append(",");
            }
    
            String[] propNames = names.toString().split(",");
    
            return propNames;
        } else {
            return null;
        }
    }*/

    /*public Object getProperty(String propName) {
        return IJUtil.GetConfig().getProperty(propName);
    }
    
    public void setProperty(String propName, Object val) {
        if (val instanceof Integer) {
            val = Integer.toString((Integer) val);
        }
    
        IJUtil.setProperty(propName, (String) val);
    }*/

    /*protected String[] getNumericDataNamesImpl() {
        String[] numericDataNames = {  };
        addNumericDataNamesResults(numericDataNames);
    
        return numericDataNames;
    }*/

    /*public String[] getProfilePropertyNames() {
        return null;
    }*/
    public int[] getLightLevel() {
        return _lightLevel;
    }

    public PropertyRepresentation[] getChannelProps() {
        return _channelProps;
    }

    public double[] getPixelProfile() {
        return _pixelProfile;
    }

    public boolean isInAdminMode() {
        return _adminMode;
    }
}
