package com.nts.isip.tcp.processors;

import com.nts.isip.tcp.HistogramRepresentation;
import com.nts.isip.tcp.ImageRepresentation;
import com.nts.isip.tcp.PropertyRepresentation;
import com.nts.isip.tcp.exceptions.CheckSumException;
import com.nts.isip.tcp.sources.FrameSourceBase;

import com.wolfram.jlink.KernelLink;
import com.wolfram.jlink.MathLinkException;
import com.wolfram.jlink.MathLinkFactory;

import java.io.BufferedReader;
import java.io.FileReader;
import java.io.IOException;

import java.util.Enumeration;
import java.util.Properties;


public class MathematicaFrameProcessor extends FrameProcessorBase {
    //private byte[] _fullPackage;
    //private int _status;
    private ImageRepresentation[] _images;
    private HistogramRepresentation[] _histograms;
    private int[] _lightLevel;
    private double[][] _2DData;
    private double[][] _3DData;
    private int _frameCnt;

    //private long _timestamp;
    //private double[] _numericData;
    //private String[] _numericDataNames;
    private PropertyRepresentation[] _channelProps;
    private double[] _pixelProfile;
    private boolean _adminMode;

    //    private PackageParameters _params = new PackageParameters();
    private KernelLink _ml = null;
    private String _mathPacket;
    private Properties _props = new Properties();
    private String _mathlink;

    public MathematicaFrameProcessor(String mathPacket) {
        _mathPacket = mathPacket;

        String propsFileName = _mathPacket.replace(".m", ".properties");

        try {
            java.io.FileInputStream in = new java.io.FileInputStream(propsFileName);

            try {
                _props.clear();
                _props.load(in);
            } finally {
                in.close();
            }
        } catch (java.io.IOException e) {
            _props.clear();
        }

        try {
            BufferedReader in = new BufferedReader(new FileReader(
                        "mathlink.txt"));
            String str;
            StringBuffer res = new StringBuffer();

            while ((str = in.readLine()) != null) {
                res.append(str);
            }

            _mathlink = res.toString();
            in.close();
        } catch (IOException e) {
            _mathlink = "-linkmode launch -linkname 'math -mathlink'";
        }

        //getNumericDataNamesImpl();
    }

    public void processFrame(FrameProcessor fp)
        throws IOException, CheckSumException {
        FrameSourceBase processor = (FrameSourceBase) fp;
        ImageRepresentation[] images = processor.getImages();

        if (images != null) {
            StringBuffer processImg = new StringBuffer("ProcessImg[{ ");

            for (int size = 0; size < (images.length - 1); size++) {
                if (images[size] != null) {
                    byte[] img = images[size].data;
                    int imgWidth = images[size].width;
                    int imgHeight = images[size].height;

                    processImg.append(recordImage(img, imgWidth, imgHeight));
                    processImg.append(",");
                }
            }

            if (images[images.length - 1] != null) {
                byte[] img = images[images.length - 1].data;
                int imgWidth = images[images.length - 1].width;
                int imgHeight = images[images.length - 1].height;
                processImg.append(recordImage(img, imgWidth, imgHeight));
            } else {
                processImg.deleteCharAt(processImg.length() - 1);
            }

            processImg.append("}]");

            try {
                _ml = MathLinkFactory.createKernelLink(_mathlink);
            } catch (MathLinkException e) {
                e.printStackTrace();
                System.out.println("Fatal error opening link: " +
                    e.getMessage());

                return;
            }

            try {
                _ml.discardAnswer();

                _ml.evaluate("<<" + _mathPacket);
                _ml.discardAnswer();

                Enumeration en = _props.keys();

                while (en.hasMoreElements()) {
                    String name = (String) en.nextElement();
                    String val = _props.getProperty(name);
                    _ml.evaluate("Arguments[\"" + name + "\"]=" + val);
                    _ml.discardAnswer();
                }

                _ml.evaluate(processImg.toString());
                _ml.discardAnswer();

                _ml.evaluate("Result1D");
                _ml.waitForAnswer();
                //_numericData = _ml.getDoubleArray1();
                _ml.evaluate("ResultImage");
                _ml.waitForAnswer();

                Object result = _ml.getArray(KernelLink.TYPE_DOUBLE, 3);

                double[][][] imgRes = (double[][][]) result;

                _images = new ImageRepresentation[imgRes.length];

                for (int i = 0; i < imgRes.length; i++) {
                    _images[i] = new ImageRepresentation();

                    int height = imgRes[i].length;
                    int width = imgRes[i][0].length;
                    _images[i].width = width;
                    _images[i].height = height;
                    _images[i].bitDepth = 8;
                    _images[i].tabIndex = i;

                    byte[] data = new byte[width * height];

                    for (int x = 0; x < width; x++) {
                        for (int y = 0; y < height; y++) {
                            data[(y * width) + x] = (byte) imgRes[i][y][x];
                        }
                    }

                    _images[i].data = data;
                }

                _ml.evaluate("Result2D");
                _ml.waitForAnswer();

                Object res = _ml.getArray(KernelLink.TYPE_DOUBLE, 2);

                _2DData = (double[][]) res;
            } catch (MathLinkException e) {
                System.out.println("MathLinkException occurred: " +
                    e.getMessage());
                e.printStackTrace();
            } finally {
                _ml.close();
            }
        } else {
        }

        try {
            //_status = processor.getStatus();
            _frameCnt = processor.getFrameNumber();
            //_timestamp = processor.getTimeStamp();
            //_fullPackage = processor.getFullpackage();
            _lightLevel = processor.getLightLevel();
            _channelProps = processor.getChannelProps();
            _adminMode = processor.isInAdminMode();
            _pixelProfile = processor.getPixelProfile();
        } catch (NullPointerException e) {
        }

        try {
            _histograms = processor.getHistograms();
        } catch (NullPointerException e) {
            _histograms = null;
        }

        //        try {
        //            _params = (PackageParameters) processor.getProperty(
        //                    "PackageParameters");
        //        } catch (NoSuchPropertyException e) {
        //        } catch (NullPointerException e) {
        //        }

        //addFullPackageResult(_fullPackage);
        //addStatusResults(_status);
        addImagesResults(_images);
        addHistogramsResults(_histograms);
        addLightLevelResults(_lightLevel);
        addChannelPropsResults(_channelProps);
        add2DResults(_2DData);
        add3DResults(_3DData);
        addFrameNumberResults(_frameCnt);
        addPixelProfileResults(_pixelProfile);
        addAdminMode(_adminMode);

        //addTimeStampResults(_timestamp);
        //addNumericDataResults(_numericData);
        if (_linkedProcessor != null) {
            _linkedProcessor.processFrame(processor);
        }
    }

    /*protected String[] getPropertyNamesImpl() {
        String[] propertyNames = new String[_props.size()];
        Enumeration en = _props.keys();
        int i = 0;
    
        while (en.hasMoreElements()) {
            propertyNames[i] = (String) en.nextElement();
            i++;
        }
    
        return propertyNames;
    }*/

    /*public Object getProperty(String prop) {
        return _props.getProperty(prop);
    }
    
    public void setProperty(String prop, Object val) {
        _props.setProperty(prop, (String) val);
    }*/

    /*protected String[] getNumericDataNamesImpl() {
        try {
            _ml = MathLinkFactory.createKernelLink(_mathlink);
        } catch (MathLinkException e) {
            e.printStackTrace();
            System.out.println("Fatal error opening link: " + e.getMessage());
    
            return null;
        }
    
        try {
            _ml.discardAnswer();
            _ml.evaluate("<<" + _mathPacket);
            _ml.discardAnswer();
    
            _ml.evaluate("Result1DNames");
            _ml.waitForAnswer();
            _numericDataNames = _ml.getStringArray1();
        } catch (MathLinkException e) {
            System.out.println("MathLinkException occurred: " + e.getMessage());
            e.printStackTrace();
        } finally {
            _ml.close();
        }
    
        addNumericDataNamesResults(_numericDataNames);
    
        return _numericDataNames;
    }*/
    private String recordImage(byte[] img, int width, int height) {
        StringBuffer res = new StringBuffer();
        res.append("{");

        for (int i = 0; i < (height - 1); i++) {
            res.append("{");

            for (int j = 0; j < (width - 1); j++) {
                res.append(img[(i * width) + j]);
                res.append(",");
            }

            res.append(img[((i * width) + width) - 1]);
            res.append("},");
        }

        res.append("{");

        for (int j = 0; j < (width - 1); j++) {
            res.append(img[((height - 1) * width) + j]);
            res.append(",");
        }

        res.append(img[(((height - 1) * width) + width) - 1]);
        res.append("}}");

        return res.toString();
    }

    /*public String[] getProfilePropertyNames() {
        return null;
    }*/
    public int[] getLightLevel() {
        return _lightLevel;
    }

    public PropertyRepresentation[] getChannelProps() {
        return _channelProps;
    }

    public double[] getPixelProfile() {
        return _pixelProfile;
    }

    public boolean isInAdminMode() {
        return _adminMode;
    }
}
