package com.nts.isip.tcp.processors;

import com.nts.isip.tcp.HistogramRepresentation;
import com.nts.isip.tcp.ImageRepresentation;
import com.nts.isip.tcp.NativeProcessingAlgorithm;
import com.nts.isip.tcp.PropertyRepresentation;
import com.nts.isip.tcp.exceptions.CheckSumException;
import com.nts.isip.tcp.sources.FrameSourceBase;

import java.io.File;
import java.io.IOException;

import java.util.Properties;


public class RematchingFrameProcessor extends FrameProcessorBase {
    //private byte[] _fullPackage;
    //private int _status;
    private ImageRepresentation[] _images;
    private HistogramRepresentation[] _histograms;
    private int[] _lightLevel;
    private double[][] _2DData;
    private double[][] _3DData;
    private int _frameCnt;

    //private long _timestamp;
    //private double[] _numericData;
    private PropertyRepresentation[] _channelProps;
    private double[] _pixelProfile;
    private boolean _adminMode;

    //    private PackageParameters _params = new PackageParameters();
    private Properties _props = new Properties();
    private boolean _override = false;
    private NativeProcessingAlgorithm _npa;

    public RematchingFrameProcessor(String nativeLibFileName) {
        File file = new File(nativeLibFileName);
        _npa = new NativeProcessingAlgorithm(file.getAbsolutePath());

        String propFileName = null;

        if (nativeLibFileName.endsWith(".dll")) {
            propFileName = nativeLibFileName.replace(".dll", ".properties");
        }

        if (nativeLibFileName.endsWith(".so")) {
            propFileName = nativeLibFileName.replace(".so", ".properties");
        }

        try {
            java.io.FileInputStream in = new java.io.FileInputStream(propFileName);

            try {
                _props.clear();
                _props.load(in);
            } finally {
                if (in != null) {
                    in.close();
                }
            }
        } catch (java.io.IOException e) {
            _props.clear();
        }

        //getPropertyNamesImpl();
        //getNumericDataNamesImpl();
    }

    public void processFrame(FrameProcessor fp)
        throws IOException, CheckSumException {
        FrameSourceBase processor = (FrameSourceBase) fp;

        if (!_override) {
            //            try {
            //                _params = (PackageParameters) processor.getProperty(
            //                        "PackageParameters");
            //            } catch (NoSuchPropertyException e) {
            //            }
        }

        _npa.setProperties(_props);

        ImageRepresentation[] img = processor.getImages();

        if (img != null) {
            ImageRepresentation[] nativeImg = _npa.processImages(img);

            if (nativeImg != null) {
                _images = new ImageRepresentation[img.length +
                    nativeImg.length];
                System.arraycopy(img, 0, _images, 0, img.length);
                System.arraycopy(nativeImg, 0, _images, img.length,
                    nativeImg.length);
            } else {
                _images = img;
            }
        }

        _2DData = _npa.get2DData();
        _3DData = _npa.get3DData();

        //_numericData = _npa.getNumericData();
        try {
            //_status = processor.getStatus();
            _frameCnt = processor.getFrameNumber();
            //_timestamp = processor.getTimeStamp();
            _histograms = processor.getHistograms();
            //_fullPackage = processor.getFullpackage();
            _lightLevel = processor.getLightLevel();
            _channelProps = processor.getChannelProps();
            _pixelProfile = processor.getPixelProfile();
            _adminMode = processor.isInAdminMode();
        } catch (NullPointerException e) {
        }

        //        try {
        //            _params = (PackageParameters) processor.getProperty(
        //                    "PackageParameters");
        //        } catch (NoSuchPropertyException e) {
        //        } catch (NullPointerException e) {
        //        }

        //addFullPackageResult(_fullPackage);
        //addStatusResults(_status);
        addImagesResults(_images);
        addHistogramsResults(_histograms);
        addLightLevelResults(_lightLevel);
        addChannelPropsResults(_channelProps);
        add2DResults(_2DData);
        add3DResults(_3DData);
        addFrameNumberResults(_frameCnt);
        addPixelProfileResults(_pixelProfile);
        addAdminMode(_adminMode);

        //addTimeStampResults(_timestamp);
        //addNumericDataResults(_numericData);
        if (_linkedProcessor != null) {
            _linkedProcessor.processFrame(processor);
        }
    }

    public void setParametersOverride(boolean override) {
        _override = override;
    }

    //    public PackageParameters getPackageParameters() {
    //        return _params;
    //    }
    /*public Object getProperty(String propName) {
        return _props.getProperty(propName);
    }
    
    public void setProperty(String propName, Object val) {
        _props.setProperty(propName, (String) val);
    }*/

    /*protected String[] getPropertyNamesImpl() {
        Enumeration en = _props.keys();
        Vector<String> keys = new Vector<String>();
    
        while (en.hasMoreElements()) {
            keys.add((String) en.nextElement());
        }
    
        return keys.toArray(new String[0]);
    }*/

    /*protected String[] getNumericDataNamesImpl() {
        String[] numericDataNames = {  };
        addNumericDataNamesResults(numericDataNames);
    
        return numericDataNames;
    }*/

    /*public String[] getProfilePropertyNames() {
        return null;
    }*/
    public int[] getLightLevel() {
        return _lightLevel;
    }

    public PropertyRepresentation[] getChannelProps() {
        return _channelProps;
    }

    public double[] getPixelProfile() {
        return _pixelProfile;
    }

    public boolean isInAdminMode() {
        return _adminMode;
    }
}
