package com.nts.isip.tcp.sources;

import at.igm.www.iCAMWSDL.client.ArrayOfFloats;
import at.igm.www.iCAMWSDL.client.ArrayOfInts;
import at.igm.www.iCAMWSDL.client.ArrayOfStrings;
import at.igm.www.iCAMWSDL.client.DirectoryEntry;
import at.igm.www.iCAMWSDL.client.ICAMWEBServiceLocator;
import at.igm.www.iCAMWSDL.client.ICAMgsoap;
import at.igm.www.iCAMWSDL.client.holders.ArrayOfEntriesHolder;
import at.igm.www.iCAMWSDL.client.holders.ArrayOfFloatsHolder;
import at.igm.www.iCAMWSDL.client.holders.ArrayOfIntsHolder;
import at.igm.www.iCAMWSDL.client.holders.ArrayOfStringsHolder;
import at.igm.www.iCAMWSDL.client.holders.DirectoryHolder;
import at.igm.www.iCAMWSDL.client.holders.PropertyHolder;

import com.nts.isip.tcp.DirectoryRepresentation;
import com.nts.isip.tcp.HistogramRepresentation;
import com.nts.isip.tcp.ImageRepresentation;
import com.nts.isip.tcp.PropertyRepresentation;
import com.nts.isip.tcp.PropertyThread;
import com.nts.isip.tcp.exceptions.CheckSumException;
import com.nts.isip.tcp.exceptions.FrameSourceException;
import com.nts.isip.tcp.exceptions.NoSuchPropertyException;
import com.nts.isip.tcp.processors.FrameProcessor;

import java.awt.Image;
import java.awt.image.PixelGrabber;

import java.io.BufferedInputStream;
import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.InputStream;

import java.rmi.RemoteException;

import java.text.SimpleDateFormat;

import java.util.Date;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Properties;
import java.util.Vector;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import javax.imageio.ImageIO;

import javax.swing.JOptionPane;

import javax.xml.rpc.holders.ByteArrayHolder;
import javax.xml.rpc.holders.FloatHolder;
import javax.xml.rpc.holders.IntHolder;
import javax.xml.rpc.holders.StringHolder;


public class IcamFrameSource extends FrameSourceBase {
    private static final String VALIDRESERROR = "S_OK";
    private final static int TWODDATAARRAYS = 30;
    
    private ICAMgsoap _port;
    private String _url;
    private Vector<DirectoryRepresentation> _directories = new Vector<DirectoryRepresentation>();
    private Hashtable<String, PropertyRepresentation> _properties = new Hashtable<String, PropertyRepresentation>();
    private Vector<String> _subscribed = new Vector<String>();
    private Vector<String> _unsubscribeProp = new Vector<String>();
    private PropertyThread _propThread;
    private boolean _hasFrameAvailable = false;
    
    private String _channel;
    private String _track;
    private Hashtable<String, SectionProperties> _secProps;
    private Hashtable<String, ICAMgsoap> _channelPorts;
    
    private Vector<ImageRepresentation> _images;
    private Vector<HistogramRepresentation> _histograms;
    private Vector<PropertyRepresentation> _channelProps;
    private int[] _lightLevel;
    private double[][] _2DData;
    private double[][] _3DData;
    private double[] _pixelProfile;
    
    private int _frameCnt;

    public IcamFrameSource(ICAMgsoap port, String url) throws IOException, Exception {
        _port = port;
        _url = url;
        _hasFrameAvailable = true;
        _frameCnt = 0;
        _channelPorts = new Hashtable<String, ICAMgsoap>();
        
        Properties prop = new Properties();
        try {
            java.io.FileInputStream in = new java.io.FileInputStream(
                    "isip_client_chart.properties");

            try {
                prop.load(in);
            } finally {
                in.close();
            }
        } catch (IOException e) {
        }

        String propName = prop.getProperty("timeSynch");
        SimpleDateFormat formatter = new SimpleDateFormat(
                "yyyy/MM/dd HH:mm:ss Z");
        String s = formatter.format(new Date());

        if (propName != null) {
            PropertyRepresentation timeSynch = this.getPropertyDefinition(propName);

            if ((timeSynch != null) && (timeSynch.path != null)) {
                timeSynch.isReadOnly = true;
                setProperty(timeSynch, s);
            }
        }
    }

    public void retrieveDirectories(DirectoryRepresentation dir)
        throws IOException, Exception {
        DirectoryEntry[] res = {  };
        ArrayOfEntriesHolder listing = new ArrayOfEntriesHolder();
        StringHolder strError = new StringHolder();

        if (_port == null) {
            throw new IOException("Disconnected");
        }

        _port.getPropertyDirectory(dir.dirFullPath, listing, strError);

        if (strError.value.equals(VALIDRESERROR)) {
            res = listing.value.getItem();

            if ((res != null) && (res.length > 0)) {
                for (int i = 0; i < res.length; i++) {
                    if (res[i].getEntryType().equalsIgnoreCase("directory")) {
                        String[] nameParts = dir.dirFullPath.split("/");
                        
                        if (!((nameParts.length >= 3) &&
                                nameParts[nameParts.length - 3].equalsIgnoreCase(
                                    "channel"))) {
                            DirectoryRepresentation subDir = getDirectoryDefinition(res[i].getEntryName());
                            
                            if( nameParts.length >= 1 && nameParts[nameParts.length - 1].equalsIgnoreCase("channel")) {
                            	if(subDir != null) {
                            		if(!_channelPorts.containsKey(subDir.dirFullPath)) {
                            			try {
                            				ICAMWEBServiceLocator service = new ICAMWEBServiceLocator();
                            				service.setiCAMWebserviceEndpointAddress(_url);
                            				service.setMaintainSession(true);
                            				ICAMgsoap port = service.getiCAMWebservice();
                            				_channelPorts.put(subDir.dirFullPath, port);
                            			} catch (javax.xml.rpc.ServiceException e) {
                            			}
                            		}
                            	}
                            }
                            
                            if (subDir != null) {
                                dir.subDirs.add(subDir);
                            }
                        }
                    } else {
                        if (!dir.isChannel) {
                            PropertyRepresentation prop = getPropertyDefinition(res[i].getEntryName());

                            if ((prop != null) && (prop.path != null) &&
                                    (prop.type != null)) {
                                prop.value = getProperty(prop.path, prop.type,
                                        false);
                                dir.properties.add(prop);
                            }
                        }
                    }
                }
            }
        } else {
            System.out.println(strError.value);
        }

        boolean replacedElem = false;

        for (int i = 0; i < _directories.size(); i++) {
            if (_directories.get(i).dirFullPath.equals(dir.dirFullPath)) {
                _directories.setElementAt(dir, i);
                replacedElem = true;

                break;
            }
        }

        if (!replacedElem) {
            _directories.add(dir);
        }

        if (dir.properties.size() > 0) {
            if (_unsubscribeProp.size() > 0) {
                String[] propNames = _unsubscribeProp.toArray(new String[0]);
                _port.unsubscribeForEvents(new ArrayOfStrings(propNames));
                _unsubscribeProp.clear();
            }

            //String[] currentSubscribed = new String[dir.properties.size()];

            for (int i = 0; i < dir.properties.size(); i++) {
                _subscribed.add(dir.properties.get(i).path);
                //currentSubscribed[i] = dir.properties.get(i).path;
            }
            
            String[] currentSubscribed = new String[_subscribed.size()];
            for(int i = 0; i < currentSubscribed.length; i++) {
            	currentSubscribed[i] = _subscribed.get(i);
            }
            
            ArrayOfStrings arrstr = new ArrayOfStrings(currentSubscribed);
            _port.subscribeForEvents(arrstr);
        }
    }
    
    public Vector<DirectoryRepresentation> getDirectoryData(String path)
        throws Exception {
        DirectoryRepresentation dir = getDirectoryDefinition(path);

        if (dir != null) {
            retrieveDirectories(dir);
        }

        return _directories;
    }
    
    public PropertyRepresentation getPropertyDefinition(String entryName)
        throws RemoteException {
        if (_properties.containsKey(entryName)) {
            return _properties.get(entryName);
        }

        PropertyRepresentation prop = new PropertyRepresentation();
        PropertyHolder propHolder = new PropertyHolder();
        StringHolder propErrHolder = new StringHolder();
        _port.getPropertyDefinition(entryName, propHolder, propErrHolder);

        if (propErrHolder.value.equals(VALIDRESERROR)) {
            prop.path = entryName;
            prop.label = propHolder.value.getLabel();

            if (prop.label.equals("")) {
                String[] propNameParts = (prop.path).split("/");
                prop.label = propNameParts[propNameParts.length - 1];
            }

            prop.minvalue = propHolder.value.getMinValue();
            prop.maxvalue = propHolder.value.getMaxValue();
            prop.type = propHolder.value.getPropertyType();
            prop.group = propHolder.value.getPropertyGroup();
            prop.allowedValues = propHolder.value.getPropertyAllowedValues();
            prop.isReadOnly = propHolder.value.isIsReadOnly();
            prop.isAdminMode = propHolder.value.isIsAdmin();
            prop.info = propHolder.value.getInfo();
            prop.arrayColumns = propHolder.value.getArrayColumns();
            prop.onSetValidator = propHolder.value.getOnSetValidator();
            prop.onTypingValidator = propHolder.value.getOnTypingValidator();
            prop.pictogram = propHolder.value.getPictogram();
        }

        _properties.put(entryName, prop);

        return prop;
    }

    public DirectoryRepresentation getDirectoryDefinition(String entryName)
        throws RemoteException, IOException {
        if (_port == null) {
            throw new IOException("Disconnected");
        }

        DirectoryRepresentation dir = new DirectoryRepresentation();
        StringHolder _propDirError = new StringHolder();
        DirectoryHolder _dirHolder = new DirectoryHolder();
        _port.getDirectoryDefinition(entryName, _dirHolder, _propDirError);

        if (!_propDirError.equals(VALIDRESERROR)) {
            if (_dirHolder.value != null) {
                dir.isTransactional = _dirHolder.value.isIsTransaction();
            } else {
                dir.isTransactional = false;
            }

            if (entryName.endsWith("/")) {
                dir.dirFullPath = entryName;
            } else {
                dir.dirFullPath = entryName + "/";
            }

            if (_dirHolder.value != null) {
                dir.label = _dirHolder.value.getDirectoryLabel();
            } else {
                dir.label = dir.dirFullPath;
            }

            dir.isChannel = false;

            String[] nameParts = dir.dirFullPath.split("/");

            for (int i = 0; i < (nameParts.length - 1); i++) {
                if (nameParts[i].equalsIgnoreCase("channel")) {
                    dir.isChannel = true;
                }
            }
        } else {
            dir = null;
            System.out.println(_propDirError.value);
        }

        return dir;
    }

    public boolean isInAdminMode() {
        if (_port == null) {
            return false;
        }

        StringHolder res = new StringHolder();
        StringHolder errStr = new StringHolder();

        try {
            _port.getPropertyValueString("/mode/admin/", res, errStr);

            if (errStr.value.equals(VALIDRESERROR)) {
                if (res.value.equalsIgnoreCase("on")) {
                    return true;
                } else {
                    return false;
                }
            } else {
                return false;
            }
        } catch (RemoteException e) {
        }

        return false;
    }

    public Object getPropertyIntArray(String name) throws RuntimeException {
        ArrayOfIntsHolder arrIntHolder = new ArrayOfIntsHolder();
        StringHolder strHolder = new StringHolder();

        try {
        	String[] propNameParts = name.split("/");
        	if(propNameParts.length >= 3 && 
        	   propNameParts[propNameParts.length - 3].equalsIgnoreCase("channel") &&
        	   _channelPorts.containsKey("/channel/" + _channel + "/") ) {
        		_channelPorts.get("/channel/" + _channel + "/").getPropertyIntArray(name, arrIntHolder, strHolder);
        	} else {
        		_port.getPropertyIntArray(name, arrIntHolder, strHolder);
        	}

            if (strHolder.value.equals(VALIDRESERROR) &&
                    (arrIntHolder.value != null)) {
                StringBuffer res = new StringBuffer();

                if (arrIntHolder.value.getItem() != null) {
                    for (int i = 0; i < arrIntHolder.value.getItem().length;
                            i++) {
                        res.append(arrIntHolder.value.getItem(i));
                        res.append(",");
                    }

                    res.deleteCharAt(res.length() - 1);
                }

                return res.toString();
            } else {
                return null;
            }
        } catch (RemoteException e) {
            throw (RuntimeException) e.getCause();
        }
    }

    public Object getPropertyPointsIntArray(String name)
        throws RuntimeException {
        ArrayOfIntsHolder arrIntHolder = new ArrayOfIntsHolder();
        StringHolder strHolder = new StringHolder();

        try {
        	String[] propNameParts = name.split("/");
        	if(propNameParts.length >= 3 && 
        	   propNameParts[propNameParts.length - 3].equalsIgnoreCase("channel") &&
               _channelPorts.containsKey("/channel/" + _channel + "/") ) {
             	_channelPorts.get("/channel/" + _channel + "/").getPropertyIntArray(name, arrIntHolder, strHolder);
            } else {
            	_port.getPropertyIntArray(name, arrIntHolder, strHolder);
            }

            if (strHolder.value.equals(VALIDRESERROR) &&
                    (arrIntHolder.value != null)) {
                if (arrIntHolder.value.getItem() != null) {
                    int[] resInt = arrIntHolder.value.getItem();
                    double[] res = new double[resInt.length];

                    for (int i = 0; i < resInt.length; i++) {
                        res[i] = (double) resInt[i];
                    }

                    return res;
                } else {
                    return null;
                }
            } else {
                return null;
            }
        } catch (RemoteException e) {
            throw (RuntimeException) e.getCause();
        }
    }

    public Object getPropertyFloatArray(String name) throws RuntimeException {
        ArrayOfFloatsHolder arrFloatHolder = new ArrayOfFloatsHolder();
        StringHolder strHolder = new StringHolder();

        try {
        	String[] propNameParts = name.split("/");
        	if(propNameParts.length >= 3 && 
        	   propNameParts[propNameParts.length - 3].equalsIgnoreCase("channel") &&
        	   _channelPorts.containsKey("/channel/" + _channel + "/") ) {
        		_channelPorts.get("/channel/" + _channel + "/").getPropertyFloatArray(name, arrFloatHolder, strHolder);
        	} else {
        		_port.getPropertyFloatArray(name, arrFloatHolder, strHolder);
        	}

            if (strHolder.value.equals(VALIDRESERROR) &&
                    (arrFloatHolder != null)) {
                StringBuffer res = new StringBuffer();

                if (arrFloatHolder.value != null) {
                    if (arrFloatHolder.value.getItem() != null) {
                        for (int i = 0;
                                i < arrFloatHolder.value.getItem().length;
                                i++) {
                            res.append(arrFloatHolder.value.getItem(i));
                            res.append(",");
                        }

                        res.deleteCharAt(res.length() - 1);
                    }
                }

                return res.toString();
            } else {
                return null;
            }
        } catch (RemoteException e) {
            throw (RuntimeException) e.getCause();
        }
    }

    public Object getPropertyPointsFloatArray(String name)
        throws RuntimeException {
        ArrayOfFloatsHolder arrFloatHolder = new ArrayOfFloatsHolder();
        StringHolder strHolder = new StringHolder();

        try {
        	String[] propNameParts = name.split("/");
        	if(propNameParts.length >= 3 && 
        	   propNameParts[propNameParts.length - 3].equalsIgnoreCase("channel") &&
        	   _channelPorts.containsKey("/channel/" + _channel + "/") ) {
        		_channelPorts.get("/channel/" + _channel + "/").getPropertyFloatArray(name, arrFloatHolder, strHolder);
        	} else {
        		_port.getPropertyFloatArray(name, arrFloatHolder, strHolder);
        	}

            if (strHolder.value.equals(VALIDRESERROR) &&
                    (arrFloatHolder != null)) {
                if (arrFloatHolder.value != null) {
                    if (arrFloatHolder.value.getItem() != null) {
                        float[] resFloat = arrFloatHolder.value.getItem();
                        double[] res = new double[resFloat.length];

                        for (int i = 0; i < resFloat.length; i++) {
                            res[i] = (double) resFloat[i];
                        }

                        return res;
                    } else {
                        return null;
                    }
                } else {
                    return null;
                }
            } else {
                return null;
            }
        } catch (RemoteException e) {
            throw (RuntimeException) e.getCause();
        }
    }

    public Object getPropertyImage(String name) throws RuntimeException {
        ByteArrayHolder byteArrHolder = new ByteArrayHolder();
        StringHolder strHolder = new StringHolder();

        try {
        	if(_channelPorts.containsKey("/channel/" + _channel + "/")) {
        		_channelPorts.get("/channel/" + _channel + "/").getPropertyImage("/channel/" + _channel + "/" + _track, name,
                        byteArrHolder, strHolder);
        	} else {
        		_port.getPropertyImage("/channel/" + _channel + "/" + _track, name,
        				byteArrHolder, strHolder);
        	}

            if (strHolder.value.equals(VALIDRESERROR)) {
                return byteArrHolder.value;
            } else {
                return null;
            }
        } catch (RemoteException e) {
            throw (RuntimeException) e.getCause();
        }
    }

    public Object getPropertyStringArray(String name) throws RuntimeException {
        ArrayOfStringsHolder arrStringHolder = new ArrayOfStringsHolder();
        StringHolder strHolder = new StringHolder();

        try {
        	String[] propNameParts = name.split("/");
        	if(propNameParts.length >= 3 && 
        	   propNameParts[propNameParts.length - 3].equalsIgnoreCase("channel") &&
        	   _channelPorts.containsKey("/channel/" + _channel + "/") ) {
        		_channelPorts.get("/channel/" + _channel + "/").getPropertyStringArray(name, arrStringHolder, strHolder);
        	} else {
        		_port.getPropertyStringArray(name, arrStringHolder, strHolder);
        	}

            if (strHolder.value.equals(VALIDRESERROR) &&
                    (arrStringHolder.value != null)) {
                StringBuffer res = new StringBuffer();

                if (arrStringHolder.value.getItem() != null) {
                    for (int i = 0; i < arrStringHolder.value.getItem().length;
                            i++) {
                        res.append(arrStringHolder.value.getItem(i));
                        res.append(",");
                    }

                    res.deleteCharAt(res.length() - 1);
                }

                return res.toString();
            } else {
                return null;
            }
        } catch (RemoteException e) {
            throw (RuntimeException) e.getCause();
        }
    }

    public Object getPropertyValueFloat(String name) throws RuntimeException {
        FloatHolder floatHolder = new FloatHolder();
        StringHolder strHolder = new StringHolder();

        try {
        	String[] propNameParts = name.split("/");
        	if(propNameParts.length >= 3 && 
        	   propNameParts[propNameParts.length - 3].equalsIgnoreCase("channel") &&
        	   _channelPorts.containsKey("/channel/" + _channel + "/") ) {
        		_channelPorts.get("/channel/" + _channel + "/").getPropertyValueFloat(name, floatHolder, strHolder);
        	} else {
        		_port.getPropertyValueFloat(name, floatHolder, strHolder);
        	}

            if (strHolder.value.equals(VALIDRESERROR)) {
                return floatHolder.value;
            } else {
                return null;
            }
        } catch (RemoteException e) {
            throw (RuntimeException) e.getCause();
        }
    }

    public Object getPropertyValueInt(String name) throws RuntimeException {
        IntHolder intHolder = new IntHolder();
        StringHolder strHolder = new StringHolder();

        try {
        	String[] propNameParts = name.split("/");
        	if(propNameParts.length >= 3 && 
        	   propNameParts[propNameParts.length - 3].equalsIgnoreCase("channel") &&
        	   _channelPorts.containsKey("/channel/" + _channel + "/") ) {
        		_channelPorts.get("/channel/" + _channel + "/").getPropertyValueInt(name, intHolder, strHolder);
        	} else {
        		_port.getPropertyValueInt(name, intHolder, strHolder);
        	}

            if (strHolder.value.equals(VALIDRESERROR)) {
                return intHolder.value;
            } else {
                return null;
            }
        } catch (RemoteException e) {
            throw (RuntimeException) e.getCause();
        }
    }

    public Object getPropertyValueString(String name) throws RuntimeException {
        StringHolder stringHolder = new StringHolder();
        StringHolder strHolder = new StringHolder();

        try {
        	String[] propNameParts = name.split("/");
        	if(propNameParts.length >= 3 && 
        	   propNameParts[propNameParts.length - 3].equalsIgnoreCase("channel") &&
        	   _channelPorts.containsKey("/channel/" + _channel + "/") ) {
        		_channelPorts.get("/channel/" + _channel + "/").getPropertyValueString(name, stringHolder, strHolder);
        	} else {
        		_port.getPropertyValueString(name, stringHolder, strHolder);
        	}

            if (strHolder.value.equals(VALIDRESERROR)) {
                return stringHolder.value;
            } else {
                return null;
            }
        } catch (RemoteException e) {
            throw (RuntimeException) e.getCause();
        }
    }

    public void setPropertyIntArray(PropertyRepresentation prop, Object value)
        throws RuntimeException {
        try {
            if (value != null) {
                String[] valParts = (value.toString()).split(",");
                int[] newVals = new int[valParts.length];

                for (int i = 0; i < valParts.length; i++) {
                    newVals[i] = Integer.valueOf(valParts[i].trim());
                }

                try {
                	String res;
                	String[] propNameParts = (prop.path).split("/");
                	if(propNameParts.length >= 3 && 
                	   propNameParts[propNameParts.length - 3].equalsIgnoreCase("channel") &&
                	   _channelPorts.containsKey("/channel/" + _channel + "/") ) {
                		res = _channelPorts.get("/channel/" + _channel + "/").setPropertyIntArray(prop.path,
                				new ArrayOfInts(newVals));
                	} else {
                		res = _port.setPropertyIntArray(prop.path,
                				new ArrayOfInts(newVals));
                	}

                    if (!res.equals(VALIDRESERROR)) {
                        JOptionPane.showMessageDialog(null, res, "Error",
                            JOptionPane.ERROR_MESSAGE);
                    }
                } catch (RemoteException e) {
                    throw (RuntimeException) e.getCause();
                }
            }
        } catch (NumberFormatException e) {
            JOptionPane.showMessageDialog(null, "Wrong property value type!",
                "Error", JOptionPane.ERROR_MESSAGE);
        }
    }

    public void setPropertyFloatArray(PropertyRepresentation prop, Object value)
        throws RuntimeException {
        try {
            if (value != null) {
                String[] valParts = (value.toString()).split(",");
                float[] newVals = new float[valParts.length];

                for (int i = 0; i < valParts.length; i++) {
                    newVals[i] = Float.valueOf(valParts[i].trim());
                }

                try {
                	String res;
                	String[] propNameParts = (prop.path).split("/");
                	if(propNameParts.length >= 3 && 
                	   propNameParts[propNameParts.length - 3].equalsIgnoreCase("channel") &&
                	   _channelPorts.containsKey("/channel/" + _channel + "/") ) {
                		res = _channelPorts.get("/channel/" + _channel + "/").setPropertyFloatArray(prop.path,
                                new ArrayOfFloats(newVals));
                	} else {
                		res = _port.setPropertyFloatArray(prop.path,
                            new ArrayOfFloats(newVals));
                	}

                    if (!res.equals(VALIDRESERROR)) {
                        JOptionPane.showMessageDialog(null, res, "Error",
                            JOptionPane.ERROR_MESSAGE);
                    }
                } catch (RemoteException e) {
                    throw (RuntimeException) e.getCause();
                }
            }
        } catch (NumberFormatException e) {
            JOptionPane.showMessageDialog(null, "Wrong property value type!",
                "Error", JOptionPane.ERROR_MESSAGE);
        }
    }

    public void setPropertyStringArray(PropertyRepresentation prop, Object value)
        throws RuntimeException {
        try {
            if (value != null) {
            	String res;
            	String[] propNameParts = (prop.path).split("/");
            	if(propNameParts.length >= 3 && 
            	   propNameParts[propNameParts.length - 3].equalsIgnoreCase("channel") &&
            	   _channelPorts.containsKey("/channel/" + _channel + "/") ) {
            		res = _channelPorts.get("/channel/" + _channel + "/").setPropertyStringArray(prop.path,
                            new ArrayOfStrings((value.toString()).split(",")));
            	} else {
            		res = _port.setPropertyStringArray(prop.path,
                        new ArrayOfStrings((value.toString()).split(",")));
            	}

                if (!res.equals(VALIDRESERROR)) {
                    JOptionPane.showMessageDialog(null, res, "Error",
                        JOptionPane.ERROR_MESSAGE);
                }
            }
        } catch (RemoteException e) {
            throw (RuntimeException) e.getCause();
        }
    }

    public void setPropertyValueFloat(PropertyRepresentation prop, Object value)
        throws RuntimeException {
        if (value != null) {
            try {
                float newVal = Float.valueOf(value.toString()).floatValue();

                if (prop.minvalue != prop.maxvalue) {
                    if (newVal > prop.maxvalue) {
                        JOptionPane.showMessageDialog(null,
                            "Wrong value! " + "The maximum allowed value is " +
                            prop.maxvalue, "Error", JOptionPane.ERROR_MESSAGE);
                    } else if (newVal < prop.minvalue) {
                        JOptionPane.showMessageDialog(null,
                            "Wrong value! " + "The minimum allowed value is " +
                            prop.minvalue, "Error", JOptionPane.ERROR_MESSAGE);
                    } else {
                        try {
                        	String res;
                        	String[] propNameParts = (prop.path).split("/");
                        	if(propNameParts.length >= 3 && 
                        	   propNameParts[propNameParts.length - 3].equalsIgnoreCase("channel") &&
                        	   _channelPorts.containsKey("/channel/" + _channel + "/") ) {
                        		res = _channelPorts.get("/channel/" + _channel + "/").setPropertyValueFloat(prop.path,
                                        newVal);
                        	} else {
                        		res = _port.setPropertyValueFloat(prop.path,
                                    newVal);
                        	}

                            if (!res.equals(VALIDRESERROR)) {
                                JOptionPane.showMessageDialog(null, res,
                                    "Error", JOptionPane.ERROR_MESSAGE);
                            }
                        } catch (RemoteException e) {
                            throw (RuntimeException) e.getCause();
                        }
                    }
                } else {
                    try {
                    	String res;
                    	String[] propNameParts = (prop.path).split("/");
                    	if(propNameParts.length >= 3 && 
                    	   propNameParts[propNameParts.length - 3].equalsIgnoreCase("channel") &&
                    	   _channelPorts.containsKey("/channel/" + _channel + "/") ) {
                    		res = _channelPorts.get("/channel/" + _channel + "/").setPropertyValueFloat(prop.path,
                                    newVal);
                    	} else {
                    		res = _port.setPropertyValueFloat(prop.path,
                                newVal);
                    	}

                        if (!res.equals(VALIDRESERROR)) {
                            JOptionPane.showMessageDialog(null, res, "Error",
                                JOptionPane.ERROR_MESSAGE);
                        }
                    } catch (RemoteException e) {
                        throw (RuntimeException) e.getCause();
                    }
                }
            } catch (NumberFormatException e) {
                JOptionPane.showMessageDialog(null,
                    "Wrong property value type!", "Error",
                    JOptionPane.ERROR_MESSAGE);
            }
        }
    }

    public void setPropertyValueInt(PropertyRepresentation prop, Object value)
        throws RuntimeException {
        if (value != null) {
            try {
                int newVal = Integer.valueOf(value.toString()).intValue();

                if (prop.minvalue != prop.maxvalue) {
                    if (newVal > prop.maxvalue) {
                        JOptionPane.showMessageDialog(null,
                            "Wrong value! " + "The maximum allowed value is " +
                            prop.maxvalue, "Error", JOptionPane.ERROR_MESSAGE);
                    } else if (newVal < prop.minvalue) {
                        JOptionPane.showMessageDialog(null,
                            "Wrong value! " + "The minimum allowed value is " +
                            prop.minvalue, "Error", JOptionPane.ERROR_MESSAGE);
                    } else {
                    	String res;
                    	String[] propNameParts = (prop.path).split("/");
                    	if(propNameParts.length >= 3 && 
                    	   propNameParts[propNameParts.length - 3].equalsIgnoreCase("channel") &&
                    	   _channelPorts.containsKey("/channel/" + _channel + "/") ) {
                    		res = _channelPorts.get("/channel/" + _channel + "/").setPropertyValueInt(prop.path, newVal);
                    	} else {
                    		res = _port.setPropertyValueInt(prop.path, newVal);
                    	}

                        if (!res.equals(VALIDRESERROR)) {
                            JOptionPane.showMessageDialog(null, res, "Error",
                                JOptionPane.ERROR_MESSAGE);
                        }
                    }
                } else {
                	String res;
                	String[] propNameParts = (prop.path).split("/");
                	if(propNameParts.length >= 3 && 
                	   propNameParts[propNameParts.length - 3].equalsIgnoreCase("channel") &&
                	   _channelPorts.containsKey("/channel/" + _channel + "/") ) {
                		res = _channelPorts.get("/channel/" + _channel + "/").setPropertyValueInt(prop.path, newVal);
                	} else {
                		res = _port.setPropertyValueInt(prop.path, newVal);
                	}

                    if (!res.equals(VALIDRESERROR)) {
                        JOptionPane.showMessageDialog(null, res, "Error",
                            JOptionPane.ERROR_MESSAGE);
                    }
                }
            } catch (RemoteException e) {
                throw (RuntimeException) e.getCause();
            } catch (NumberFormatException e) {
                JOptionPane.showMessageDialog(null,
                    "Wrong property value type!", "Error",
                    JOptionPane.ERROR_MESSAGE);
            }
        }
    }

    public void setPropertyValueString(PropertyRepresentation prop, Object value)
        throws RuntimeException {
        if (value != null) {
            try {
            	String res;
            	String[] propNameParts = (prop.path).split("/");
            	if(propNameParts.length >= 3 && 
            	   propNameParts[propNameParts.length - 3].equalsIgnoreCase("channel") &&
            	   _channelPorts.containsKey("/channel/" + _channel + "/") ) {
            		res = _channelPorts.get("/channel/" + _channel + "/").setPropertyValueString(prop.path,
                            (String) value);
            	} else {
            		res = _port.setPropertyValueString(prop.path,
            				(String) value);
            	}

                if (!res.equals(VALIDRESERROR)) {
                    JOptionPane.showMessageDialog(null, res, "Error",
                        JOptionPane.ERROR_MESSAGE);
                }
            } catch (RemoteException e) {
                throw (RuntimeException) e.getCause();
            }
        }
    }

    public void commitTransaction(String propName) throws RemoteException {
        _port.commitPropertyTransaction(propName);
    }

    public void clearTransaction(String propName) throws RemoteException {
        _port.clearPropertyTransaction(propName);
    }

    public void setPropertyThread(PropertyThread propThread) {
        _propThread = propThread;
    }

    public void disconnect() throws IOException {
        if ((_port != null) && (_subscribed != null)) {
            _port.unsubscribeForEvents(new ArrayOfStrings(_subscribed.toArray(
                        new String[0])));
            _subscribed.clear();
        }

        _port = null;
        
        if(_channelPorts != null && !_channelPorts.isEmpty()) {
        	Enumeration en = _channelPorts.keys();
        	while(en.hasMoreElements()) {
        		String key = (String)en.nextElement();
        		_channelPorts.remove(key);
        	}
        }
        
        _hasFrameAvailable = false;
        _propThread.stopEvents();
    }
    
    public void setChannel(String channel) {
        _channel = channel;
    }

    public void setTrack(String track) {
        _track = track;
    }
    
    private void readData(String channelName, String selectType, int direction)
        throws IOException, CheckSumException {
        if (_port == null) {
            throw new IOException("Disconnected");
        }
        
        _images = new Vector<ImageRepresentation>();
        _histograms = new Vector<HistogramRepresentation>();
        _channelProps = new Vector<PropertyRepresentation>();
        _2DData = new double[TWODDATAARRAYS][];
        _frameCnt = 0;
        _secProps = new Hashtable<String, SectionProperties>();
        _pixelProfile = null;

        Properties secProp = new Properties();

        try {
            java.io.FileInputStream in = new java.io.FileInputStream(
                    "isip_client_chart.properties");

            try {
                secProp.load(in);
            } finally {
                in.close();
            }
        } catch (IOException e) {
        }

        Enumeration en = secProp.keys();

        while (en.hasMoreElements()) {
            String key = (String) en.nextElement();

            if (key.startsWith("ser") && key.endsWith(".PKGSECID")) {
                int number = 0;
                Pattern p = Pattern.compile(".*?(\\d+).*?");
                Matcher m = p.matcher(key);

                if (m.find()) {
                    number = Integer.valueOf(m.group(1)).intValue();
                }

                String secId = secProp.getProperty("ser" + number +
                        ".PKGSECID");
                SectionProperties secProps = new SectionProperties();
                secProps.pkgSecType = secProp.getProperty("ser" + number +
                        ".PKGSECTYPE");
                secProps.pkgTabIdx = Integer.valueOf(secProp.getProperty("ser" +
                            number + ".tabidx")).intValue();
                secProps.channel = secProp.getProperty("ser" + number +
                        ".channel");

                try {
                    secProps.idx = Integer.valueOf(secProp.getProperty("ser" +
                                number + ".idx").trim()).intValue();
                } catch (NumberFormatException e) {
                    secProps.idx = -1;
                }

                secProps.color = secProp.getProperty("ser" + number + ".color");

                try {
                    secProps.lines = Integer.valueOf(secProp.getProperty("ser" +
                                number + ".lines").trim()).intValue();
                } catch (NumberFormatException e) {
                    secProps.lines = 0;
                }

                secProps.name = secProp.getProperty("ser" + number + ".name");
                secProps.type = secProp.getProperty("ser" + number + ".type");
                _secProps.put(secId, secProps);
            }
        }

        IntHolder intHolder = new IntHolder();
        StringHolder strHolder = new StringHolder();
        
        if(_channelPorts.containsKey("/channel/" + _channel + "/")) {
        	_channelPorts.get("/channel/" + _channel + "/").selectPacket(channelName, selectType, direction, intHolder,
        			strHolder);
        } else {
        	_port.selectPacket(channelName, selectType, direction, intHolder,
        			strHolder);
        }

        if (strHolder.value.equals(VALIDRESERROR)) {
            try {
                int xoffset = 0;
                int yoffset = 0;
                int xdownsample = 0;
                int ydownsample = 0;

                if ((secProp.getProperty("windowstartcol") != null) &&
                        !secProp.getProperty("windowstartcol").equals("")) {
                    PropertyRepresentation prop = getPropertyDefinition(secProp.getProperty(
                                "windowstartcol"));
                    prop.value = getProperty(prop.path, prop.type, false);
                    xoffset = (Integer) (prop.value);
                }

                if ((secProp.getProperty("windowstartrow") != null) &&
                        !secProp.getProperty("windowstartrow").equals("")) {
                    PropertyRepresentation prop = getPropertyDefinition(secProp.getProperty(
                                "windowstartrow"));
                    prop.value = getProperty(prop.path, prop.type, false);
                    yoffset = (Integer) (prop.value);
                }

                if ((secProp.getProperty("downsamplecol") != null) &&
                        !secProp.getProperty("downsamplecol").equals("")) {
                    PropertyRepresentation prop = getPropertyDefinition(secProp.getProperty(
                                "downsamplecol"));
                    prop.value = getProperty(prop.path, prop.type, false);
                    xdownsample = (Integer) (prop.value);
                }

                if ((secProp.getProperty("downsamplerow") != null) &&
                        !secProp.getProperty("downsamplerow").equals("")) {
                    PropertyRepresentation prop = getPropertyDefinition(secProp.getProperty(
                                "downsamplerow"));
                    prop.value = getProperty(prop.path, prop.type, false);
                    ydownsample = (Integer) (prop.value);
                }

                if ((secProp.getProperty("pixelProfile") != null) &&
                        !secProp.getProperty("pixelProfile").equals("")) {
                    PropertyRepresentation prop = getPropertyDefinition(secProp.getProperty(
                                "pixelProfile"));
                    prop.value = getProperty(prop.path, prop.type, false);
                    _pixelProfile = (double[]) prop.value;

                    for (int i = 0; i < _pixelProfile.length; i += 2) {
                        double shiftX = _pixelProfile[i] - xoffset;
                        double shiftY = _pixelProfile[i + 1] - yoffset;

                        if (xdownsample != 0) {
                            _pixelProfile[i] = shiftX / xdownsample;
                        } else {
                            _pixelProfile[i] = shiftX;
                        }

                        if (ydownsample != 0) {
                            _pixelProfile[i + 1] = shiftY / ydownsample;
                        } else {
                            _pixelProfile[i + 1] = shiftY;
                        }
                    }
                }
            } catch (RemoteException e) {
                _pixelProfile = null;
            } catch (NoSuchPropertyException ex) {
                _pixelProfile = null;
            } catch (NullPointerException ex) {
                _pixelProfile = null;
            }

            _frameCnt = intHolder.value;

            ArrayOfEntriesHolder arrHolder = new ArrayOfEntriesHolder();
            StringHolder propStrHolder = new StringHolder();
            if(_channelPorts.containsKey("/channel/" + _channel + "/")) {
            	_channelPorts.get("/channel/" + _channel + "/").getPropertyDirectory("/channel/" + _channel, arrHolder,
                        propStrHolder);
            } else {
            	_port.getPropertyDirectory("/channel/" + _channel, arrHolder,
            			propStrHolder);
            }
            
            if (propStrHolder.value.equals(VALIDRESERROR)) {
                if ((arrHolder.value != null) &&
                        (arrHolder.value.getItem() != null)) {
                    for (int i = 0; i < arrHolder.value.getItem().length;
                            i++) {
                        if (arrHolder.value.getItem(i).getEntryType()
                                               .equalsIgnoreCase("property")) {
                            PropertyRepresentation prop = getPropertyDefinition(arrHolder.value.getItem(
                                        i).getEntryName());

                            if (prop.type.equalsIgnoreCase("image")) {
                                prop.value = getProperty(prop.path, prop.type,
                                        false);

                                if (prop.value != null) {
                                    if (_secProps.containsKey(prop.info) &&
                                            _secProps.get(prop.info).pkgSecType.equals(
                                                "IMAGE")) {
                                        ImageRepresentation img = new ImageRepresentation();
                                        byte[] imgByteData = (byte[]) (prop.value);
                                        ByteArrayInputStream in = new ByteArrayInputStream(imgByteData);
                                        InputStream is = new BufferedInputStream(in);
                                        Image image = ImageIO.read(is);
                                        PixelGrabber pixelGrabber = new PixelGrabber(image,
                                                0, 0, image.getWidth(null),
                                                image.getHeight(null), false);

                                        try {
                                            pixelGrabber.grabPixels();

                                            byte[] imgData;

                                            if (pixelGrabber.getPixels() instanceof byte[]) {
                                                imgData = (byte[]) pixelGrabber.getPixels();
                                            } else {
                                                int[] intData = (int[]) pixelGrabber.getPixels();
                                                imgData = new byte[intData.length];

                                                for (int j = 0;
                                                        j < intData.length;
                                                        j++) {
                                                    imgData[j] = (byte) intData[j];
                                                }
                                            }

                                            img.width = image.getWidth(null);
                                            img.height = image.getHeight(null);
                                            img.data = imgData;
                                            img.bitDepth = 0;
                                            img.tabIndex = _secProps.get(prop.info).pkgTabIdx;
                                            img.idx = _secProps.get(prop.info).idx;
                                            img.channel = _channel;
                                        } catch (InterruptedException e) {
                                            img.width = 0;
                                            img.height = 0;
                                            img.bitDepth = 0;
                                            img.tabIndex = 0;
                                            img.idx = 0;
                                            img.data = null;
                                            img.channel = _channel;
                                        }

                                        _images.add(img);
                                    }
                                }
                            } else if (prop.type.equalsIgnoreCase("histogram")) {
                                prop.value = getProperty(prop.path, prop.type,
                                        false);

                                if (prop.value != null) {
                                    /*if(_secProps.containsKey(prop.info) &&
                                               _secProps.get(prop.info).pkgSecType.equals("HISTOGRAM")) {
                                            HistogramRepresentation hist = new HistogramRepresentation();
                                            hist.histData = (int[])prop.value;
                                            hist.tabIndex = _secProps.get(prop.info).pkgTabIdx;
                                            hist.idx = _secProps.get(prop.info).idx;
                                            hist.lenght = hist.histData.length;
                                            _histograms.add(hist);
                                    }*/
                                }
                            } else if (prop.type.equalsIgnoreCase("intarray") ||
                                    prop.type.equalsIgnoreCase("floatarray")) {
                                if (_secProps.containsKey(prop.info) &&
                                        _secProps.get(prop.info).pkgSecType.equals(
                                            "POINTS") &&
                                        _secProps.get(prop.info).channel.equals(
                                            _channel)) {
                                    prop.value = getProperty(prop.path,
                                            prop.type, true);

                                    if (prop.value != null) {
                                        double[] dPointsData = (double[]) prop.value;
                                        _2DData[_secProps.get(prop.info).idx] = dPointsData;
                                    }
                                } else if (_secProps.containsKey(prop.info) &&
                                        _secProps.get(prop.info).pkgSecType.equalsIgnoreCase(
                                            "intensity") &&
                                        _secProps.get(prop.info).channel.equals(
                                            _channel)) {
                                    prop.value = getProperty(prop.path,
                                            prop.type, true);

                                    if (prop.value != null) {
                                        double[] dll = (double[]) prop.value;
                                        _lightLevel = new int[dll.length];

                                        for (int j = 0; j < dll.length; j++) {
                                            _lightLevel[j] = (int) dll[j];
                                        }
                                    }
                                }
                            } else {
                                prop.value = getProperty(prop.path, prop.type,
                                        true);
                                _channelProps.add(prop);
                            }
                        }
                    }
                }
            } else {
                System.out.println(propStrHolder.value);
            }
        } else {
            throw new IOException(strHolder.value);
        }
    }

    public Vector<String> getSubscribed() {
        return _subscribed;
    }

    public Vector<String> getUnsubscribed() {
        return _unsubscribeProp;
    }

    public void getFirstFrame() throws IOException, CheckSumException {
        //relative: -1 - previouse
        //			1 - next
        //			2 - last
        //			-2 - first
        //frame: frame number
        //timestamp: timestamp
        readData("/channel/" + _channel + "/" + _track, "relative", -2);
    }

    public void getPrevFrame() throws IOException, CheckSumException {
        readData("/channel/" + _channel + "/" + _track, "relative", -1);
    }

    public void getNextFrame() throws IOException, CheckSumException {
        readData("/channel/" + _channel + "/" + _track, "relative", 1);
    }

    public void getLastFrame() throws IOException, CheckSumException {
    	int idx=(Integer.parseInt(_track)==0 ? 3 : 2);
        readData("/channel/" + _channel + "/" + _track, "relative", idx);
    }

    public void reloadFrame() throws IOException, CheckSumException {
        readData("/channel/" + _channel + "/" + _track, "frame", _frameCnt);
    }

    public void getFrameByTime(int timestamp)
        throws IOException, CheckSumException {
        readData("/channel/" + _channel + "/" + _track, "timestamp", timestamp);
    }

    public int getFrameNumber() {
        return _frameCnt;
    }

    public boolean getFrameAvailable() {
        return _hasFrameAvailable;
    }

    public void putFrame(FrameSource frameSource) {
    }

    public void stopWriting() {
    }

    public double[][] get2DData() throws IOException, CheckSumException {
        return this._2DData;
    }

    public double[][] get3DData() throws IOException, CheckSumException {
        return this._3DData;
    }

    public HistogramRepresentation[] getHistograms() {
        return (HistogramRepresentation[]) _histograms.toArray(new HistogramRepresentation[0]);
    }

    public ImageRepresentation[] getImages() {
        return (ImageRepresentation[]) _images.toArray(new ImageRepresentation[0]);
    }

    public int[] getLightLevel() {
        return _lightLevel;
    }

    public void processFrame(FrameProcessor fp)
        throws IOException, CheckSumException {
    }

    public void systemStore() throws IOException {
        if (_port == null) {
            throw new IOException("Disconnected");
        } else {
            _port.storeSystemSettings();
        }
    }

    public void templateStore(String key)
        throws IOException, FrameSourceException {
        if (_port == null) {
            throw new IOException("Disconnected");
        } else {
            String res = _port.storeUserTemplateSettings(key);

            if (res.equals("Job already exists - delete first if needed")) {
                throw new FrameSourceException(res);
            }
        }
    }

    public String deleteTemplate(String key) throws IOException {
        if (_port == null) {
            throw new IOException("Disconnected");
        } else {
            String res = _port.deleteUserTemplate(key);

            return res;
        }
    }

    public PropertyRepresentation[] getChannelProps() {
        return _channelProps.toArray(new PropertyRepresentation[0]);
    }

    public int getTotalFrames() {
        return 0;
    }

    public String getFrameName() {
        return null;
    }
    
    public void endRecording() {
        //_recordStarted = false;
    }

    public void startRecording(int recordFrames) {
        //_recordStarted = true;
    }

    public double[] getPixelProfile() {
        return _pixelProfile;
    }

    public void execute(String command, int synch) throws IOException {
        if (_port == null) {
            throw new IOException("Disconnected");
        } else {
            String errorCode = _port.execute(command, synch);

            if (!errorCode.equals(VALIDRESERROR)) {
                throw new IOException(errorCode);
            }
        }
    }

    public byte[] getFile(String filename) throws RemoteException {
        ByteArrayHolder byteArrHolder = new ByteArrayHolder();
        StringHolder strHolder = new StringHolder();
        _port.getFile(filename, byteArrHolder, strHolder);

        if (strHolder.value.equals(VALIDRESERROR)) {
            if (byteArrHolder.value != null) {
                return byteArrHolder.value;
            } else {
                return null;
            }
        } else {
            throw new RemoteException(strHolder.value);
        }
    }

    public void putFile(String filename, byte[] bytes)
        throws IOException {
        if (_port == null) {
            throw new IOException("Disconnected");
        } else {
            String errorCode = _port.putFile(filename, bytes);

            if (!errorCode.equals(VALIDRESERROR)) {
                throw new IOException(errorCode);
            }
        }
    }

    public PropertyThread createPropertyThread() {
        if (_port != null) {
            return new PropertyThread(_port);
        } else {
            return null;
        }
    }

    public DirectoryEntry[] getPropertyDirectory(String dirName)
        throws RemoteException {
        if (_port == null) {
            throw new RemoteException("Disconnected");
        } else {
            DirectoryEntry[] res = {  };
            ArrayOfEntriesHolder listing = new ArrayOfEntriesHolder();
            StringHolder strError = new StringHolder();
            _port.getPropertyDirectory(dirName, listing, strError);

            if (strError.value.equals("S_OK")) {
                if (listing.value == null) {
                    res = null;
                } else {
                    res = listing.value.getItem();
                }
            } else {
                res = null;
            }

            return res;
        }
    }
}

class SectionProperties {
    public String pkgSecType;
    public int pkgTabIdx;
    public int idx;
    public String color;
    public int lines;
    public String name;
    public String type;
    public String channel;
}
